@@ -3,15 +3,32 @@
 #@+node:ekr.20171123095353.1: * @file ../commands/commanderFileCommands.py
 #@@first
 """File commands that used to be defined in leoCommands.py"""
+#@+<< commanderFileCommands imports >>
+#@+node:ekr.20220826120852.1: ** << commanderFileCommands imports >>
 import os
 import sys
 import time
+from typing import Any, TYPE_CHECKING
 from leo.core import leoGlobals as g
 from leo.core import leoImport
+#@-<< commanderFileCommands imports >>
+#@+<< commanderFileCommands annotations >>
+#@+node:ekr.20220826120908.1: ** << commanderFileCommands annotations >>
+if TYPE_CHECKING:  # pragma: no cover
+    from leo.core.leoCommands import Commands as Cmdr
+    from leo.core.leoGui import LeoKeyEvent as Event
+    from leo.core.leoNodes import Position, VNode
+else:
+    Cmdr = Any
+    Event = Any
+    Position = Any
+    VNode = Any
+Self = Cmdr  # For @g.commander_command.
+#@-<< commanderFileCommands annotations >>
 #@+others
 #@+node:ekr.20170221033738.1: ** c_file.reloadSettings
 @g.commander_command('reload-settings')
-def reloadSettings(self, event=None):
+def reloadSettings(self: Self, event: Event=None) -> None:
     """Reload settings in all commanders, saving all existing opened files first"""
     lm = g.app.loadManager
     # Save any changes so they can be seen, for existing files that are not new/untitled.
@@ -31,7 +48,7 @@ def reloadSettings(self, event=None):
         c.reloadConfigurableSettings()
 #@+node:ekr.20200422075655.1: ** c_file.restartLeo
 @g.commander_command('restart-leo')
-def restartLeo(self, event=None):
+def restartLeo(self: Self, event: Event=None) -> None:
     """Restart Leo, reloading all presently open outlines."""
     c, lm = self, g.app.loadManager
     trace = 'shutdown' in g.app.debug
@@ -81,12 +98,12 @@ def restartLeo(self, event=None):
 #@+node:ekr.20031218072017.2820: ** c_file.top level
 #@+node:ekr.20031218072017.2833: *3* c_file.close
 @g.commander_command('close-window')
-def close(self, event=None, new_c=None):
+def close(self: Self, event: Event=None, new_c: Cmdr=None) -> None:
     """Close the Leo window, prompting to save it if it has been changed."""
     g.app.closeLeoWindow(self.frame, new_c=new_c)
 #@+node:ekr.20110530124245.18245: *3* c_file.importAnyFile & helper
 @g.commander_command('import-file')
-def importAnyFile(self, event=None):
+def importAnyFile(self: Self, event: Event=None) -> None:
     """Import one or more files."""
     c = self
     ic = c.importCommands
@@ -173,7 +190,7 @@ g.command_alias('importMOREFiles', importAnyFile)
 g.command_alias('importNowebFiles', importAnyFile)
 g.command_alias('importTabFiles', importAnyFile)
 #@+node:ekr.20200306043104.1: *4* function: import_txt_file
-def import_txt_file(c, fn):
+def import_txt_file(c: Cmdr, fn: str) -> None:
     """Import the .txt file into a new node."""
     u = c.undoer
     g.setGlobalOpenDir(fn)
@@ -189,7 +206,7 @@ def import_txt_file(c, fn):
 #@+node:ekr.20031218072017.1623: *3* c_file.new
 @g.commander_command('file-new')
 @g.commander_command('new')
-def new(self, event=None, gui=None):
+def new(self: Self, event: Event=None, gui: Any=None) -> Cmdr:
     """Create a new Leo window."""
     t1 = time.process_time()
     from leo.core import leoApp
@@ -243,12 +260,12 @@ def new(self, event=None, gui=None):
     return c  # For unit tests and scripts.
 #@+node:ekr.20031218072017.2821: *3* c_file.open_outline & helper
 @g.commander_command('open-outline')
-def open_outline(self, event=None):
+def open_outline(self: Self, event: Event=None) -> None:
     """Open a Leo window containing the contents of a .leo file."""
     c = self
     #@+others  # Defines open_completer function.
     #@+node:ekr.20190518121302.1: *4* function: open_completer
-    def open_completer(c, fileName):
+    def open_completer(c: Cmdr, fileName: str) -> None:
 
         c.bringToFront()
         c.init_error_dialogs()
@@ -263,11 +280,12 @@ def open_outline(self, event=None):
                     g.setGlobalOpenDir(fileName)
             elif c.looksLikeDerivedFile(fileName):
                 # Create an @file node for files containing Leo sentinels.
-                ok = c.importCommands.importDerivedFiles(parent=c.p,
-                    paths=[fileName], command='Open')
+                p = c.importCommands.importDerivedFiles(parent=c.p, paths=[fileName], command='Open')
+                ok = bool(p)
             else:
                 # otherwise, create an @edit node.
-                ok = c.createNodeFromExternalFile(fileName)
+                c.createNodeFromExternalFile(fileName)
+                ok = True
         c.raise_error_dialogs(kind='write')
         g.app.runAlreadyOpenDialog(c)
         # openWithFileName sets focus if ok.
@@ -294,7 +312,7 @@ def open_outline(self, event=None):
 # refresh_pattern = re.compile(r'^(@[\w-]+)')
 
 @g.commander_command('refresh-from-disk')
-def refreshFromDisk(self, event=None):
+def refreshFromDisk(self: Self, event: Event=None) -> None:
     """Refresh an @<file> node from disk."""
     c, p, u = self, self.p, self.undoer
     c.nodeConflictList = []
@@ -352,14 +370,14 @@ def refreshFromDisk(self, event=None):
         c.redraw()
 #@+node:ekr.20210610083257.1: *3* c_file.pwd
 @g.commander_command('pwd')
-def pwd_command(self, event=None):
+def pwd_command(self: Self, event: Event=None) -> None:
     """Print the current working directory."""
     g.es_print('pwd:', os.getcwd())
 #@+node:ekr.20031218072017.2834: *3* c_file.save
 @g.commander_command('save')
 @g.commander_command('file-save')
 @g.commander_command('save-file')
-def save(self, event=None, fileName=None):
+def save(self: Self, event: Event=None, fileName: str=None) -> None:
     """
     Save a Leo outline to a file, using the existing file name unless
     the fileName kwarg is given.
@@ -421,7 +439,6 @@ def save(self, event=None, fileName=None):
             c.fileCommands.save(c.mFileName)
             g.app.recentFilesManager.updateRecentFiles(c.mFileName)
             g.chdir(c.mFileName)
-    # FileCommands.save calls c.redraw_after_icons_changed()
     c.raise_error_dialogs(kind='write')
     # *Safely* restore focus, without using the old w directly.
     if inBody:
@@ -431,7 +448,7 @@ def save(self, event=None, fileName=None):
         c.treeWantsFocus()
 #@+node:ekr.20110228162720.13980: *3* c_file.saveAll
 @g.commander_command('save-all')
-def saveAll(self, event=None):
+def saveAll(self: Self, event: Event=None) -> None:
     """Save all open tabs windows/tabs."""
     c = self
     c.save()  # Force a write of the present window.
@@ -446,7 +463,7 @@ def saveAll(self, event=None):
 @g.commander_command('save-as')
 @g.commander_command('file-save-as')
 @g.commander_command('save-file-as')
-def saveAs(self, event=None, fileName=None):
+def saveAs(self: Self, event: Event=None, fileName: str=None) -> None:
     """
     Save a Leo outline to a file, prompting for a new filename unless the
     fileName kwarg is given.
@@ -499,7 +516,6 @@ def saveAs(self, event=None, fileName=None):
         c.fileCommands.saveAs(c.mFileName)
         g.app.recentFilesManager.updateRecentFiles(c.mFileName)
         g.chdir(c.mFileName)
-    # FileCommands.saveAs calls c.redraw_after_icons_changed()
     c.raise_error_dialogs(kind='write')
     # *Safely* restore focus, without using the old w directly.
     if inBody:
@@ -511,7 +527,7 @@ def saveAs(self, event=None, fileName=None):
 @g.commander_command('save-to')
 @g.commander_command('file-save-to')
 @g.commander_command('save-file-to')
-def saveTo(self, event=None, fileName=None, silent=False):
+def saveTo(self: Self, event: Event=None, fileName: str=None, silent: bool=False) -> None:
     """
     Save a Leo outline to a file, prompting for a new file name unless the
     fileName kwarg is given. Leave the file name of the Leo outline unchanged.
@@ -552,7 +568,7 @@ def saveTo(self, event=None, fileName=None, silent=False):
     c.outerUpdate()
 #@+node:ekr.20031218072017.2837: *3* c_file.revert
 @g.commander_command('revert')
-def revert(self, event=None):
+def revert(self: Self, event: Event=None) -> None:
     """Revert the contents of a Leo outline to last saved contents."""
     c = self
     # Make sure the user wants to Revert.
@@ -570,7 +586,7 @@ def revert(self, event=None):
 #@+node:ekr.20210316075815.1: *3* c_file.save-as-leojs
 @g.commander_command('file-save-as-leojs')
 @g.commander_command('save-file-as-leojs')
-def save_as_leojs(self, event=None):
+def save_as_leojs(self: Self, event: Event=None) -> None:
     """
     Save a Leo outline as a JSON (.leojs) file with a new file name.
     """
@@ -589,7 +605,7 @@ def save_as_leojs(self, event=None):
 #@+node:ekr.20070413045221: *3* c_file.save-as-zipped
 @g.commander_command('file-save-as-zipped')
 @g.commander_command('save-file-as-zipped')
-def save_as_zipped(self, event=None):
+def save_as_zipped(self: Self, event: Event=None) -> None:
     """
     Save a Leo outline as a zipped (.db) file with a new file name.
     """
@@ -608,7 +624,7 @@ def save_as_zipped(self, event=None):
 #@+node:ekr.20210316075357.1: *3* c_file.save-as-xml
 @g.commander_command('file-save-as-xml')
 @g.commander_command('save-file-as-xml')
-def save_as_xml(self, event=None):
+def save_as_xml(self: Self, event: Event=None) -> None:
     """
     Save a Leo outline as a .leo file with a new file name.
 
@@ -628,7 +644,7 @@ def save_as_xml(self, event=None):
     c.fileCommands.putSavedMessage(fileName)
 #@+node:tom.20220310092720.1: *3* c_file.save-node-as-xml
 @g.commander_command('save-node-as-xml')
-def save_node_as_xml_outline(self, event=None):
+def save_node_as_xml_outline(self: Self, event: Event=None) -> None:
     """Save a node with its subtree as a Leo outline file."""
     c = event.c
     xml = c.fileCommands.outline_to_clipboard_string()
@@ -644,7 +660,7 @@ def save_node_as_xml_outline(self, event=None):
 #@+node:ekr.20031218072017.2849: ** Export
 #@+node:ekr.20031218072017.2850: *3* c_file.exportHeadlines
 @g.commander_command('export-headlines')
-def exportHeadlines(self, event=None):
+def exportHeadlines(self: Self, event: Event=None) -> None:
     """Export all headlines to an external file."""
     c = self
     filetypes = [("Text files", "*.txt"), ("All files", "*")]
@@ -659,7 +675,7 @@ def exportHeadlines(self, event=None):
         c.importCommands.exportHeadlines(fileName)
 #@+node:ekr.20031218072017.2851: *3* c_file.flattenOutline
 @g.commander_command('flatten-outline')
-def flattenOutline(self, event=None):
+def flattenOutline(self: Self, event: Event=None) -> None:
     """
     Export the selected outline to an external file.
     The outline is represented in MORE format.
@@ -677,7 +693,7 @@ def flattenOutline(self, event=None):
         c.importCommands.flattenOutline(fileName)
 #@+node:ekr.20141030120755.12: *3* c_file.flattenOutlineToNode
 @g.commander_command('flatten-outline-to-node')
-def flattenOutlineToNode(self, event=None):
+def flattenOutlineToNode(self: Self, event: Event=None) -> None:
     """
     Append the body text of all descendants of the selected node to the
     body text of the selected node.
@@ -704,7 +720,7 @@ def flattenOutlineToNode(self, event=None):
     u.afterChangeNodeContents(root, 'flatten-outline-to-node', bunch)
 #@+node:ekr.20031218072017.2857: *3* c_file.outlineToCWEB
 @g.commander_command('outline-to-cweb')
-def outlineToCWEB(self, event=None):
+def outlineToCWEB(self: Self, event: Event=None) -> None:
     """
     Export the selected outline to an external file.
     The outline is represented in CWEB format.
@@ -725,7 +741,7 @@ def outlineToCWEB(self, event=None):
         c.importCommands.outlineToWeb(fileName, "cweb")
 #@+node:ekr.20031218072017.2858: *3* c_file.outlineToNoweb
 @g.commander_command('outline-to-noweb')
-def outlineToNoweb(self, event=None):
+def outlineToNoweb(self: Self, event: Event=None) -> None:
     """
     Export the selected outline to an external file.
     The outline is represented in noweb format.
@@ -747,7 +763,7 @@ def outlineToNoweb(self, event=None):
         c.outlineToNowebDefaultFileName = fileName
 #@+node:ekr.20031218072017.2859: *3* c_file.removeSentinels
 @g.commander_command('remove-sentinels')
-def removeSentinels(self, event=None):
+def removeSentinels(self: Self, event: Event=None) -> None:
     """
     Convert one or more files, replacing the original files
     while removing any sentinels they contain.
@@ -774,7 +790,7 @@ def removeSentinels(self, event=None):
         c.importCommands.removeSentinelsCommand(names)
 #@+node:ekr.20031218072017.2860: *3* c_file.weave
 @g.commander_command('weave')
-def weave(self, event=None):
+def weave(self: Self, event: Event=None) -> None:
     """Simulate a literate-programming weave operation by writing the outline to a text file."""
     c = self
     fileName = g.app.gui.runSaveFileDialog(c,
@@ -789,7 +805,7 @@ def weave(self, event=None):
 #@+node:ekr.20031218072017.2838: ** Read/Write
 #@+node:ekr.20070806105721.1: *3* c_file.readAtAutoNodes
 @g.commander_command('read-at-auto-nodes')
-def readAtAutoNodes(self, event=None):
+def readAtAutoNodes(self: Self, event: Event=None) -> None:
     """Read all @auto nodes in the presently selected outline."""
     c, p, u = self, self.p, self.undoer
     c.endEditing()
@@ -801,7 +817,7 @@ def readAtAutoNodes(self, event=None):
     c.raise_error_dialogs(kind='read')
 #@+node:ekr.20031218072017.1839: *3* c_file.readAtFileNodes
 @g.commander_command('read-at-file-nodes')
-def readAtFileNodes(self, event=None):
+def readAtFileNodes(self: Self, event: Event=None) -> None:
     """Read all @file nodes in the presently selected outline."""
     c, p, u = self, self.p, self.undoer
     c.endEditing()
@@ -814,7 +830,7 @@ def readAtFileNodes(self, event=None):
     c.redraw()
 #@+node:ekr.20080801071227.4: *3* c_file.readAtShadowNodes
 @g.commander_command('read-at-shadow-nodes')
-def readAtShadowNodes(self, event=None):
+def readAtShadowNodes(self: Self, event: Event=None) -> None:
     """Read all @shadow nodes in the presently selected outline."""
     c, p, u = self, self.p, self.undoer
     c.endEditing()
@@ -826,7 +842,7 @@ def readAtShadowNodes(self, event=None):
     c.raise_error_dialogs(kind='read')
 #@+node:ekr.20070915134101: *3* c_file.readFileIntoNode
 @g.commander_command('read-file-into-node')
-def readFileIntoNode(self, event=None):
+def readFileIntoNode(self: Self, event: Event=None) -> None:
     """Read a file into a single node."""
     c = self
     undoType = 'Read File Into Node'
@@ -851,7 +867,7 @@ def readFileIntoNode(self, event=None):
     c.redraw(p)
 #@+node:ekr.20031218072017.2839: *3* c_file.readOutlineOnly
 @g.commander_command('read-outline-only')
-def readOutlineOnly(self, event=None):
+def readOutlineOnly(self: Self, event: Event=None) -> None:
     """Open a Leo outline from a .leo file, but do not read any derived files."""
     c = self
     c.endEditing()
@@ -875,7 +891,7 @@ def readOutlineOnly(self, event=None):
         g.es("can not open:", fileName)
 #@+node:ekr.20070915142635: *3* c_file.writeFileFromNode
 @g.commander_command('write-file-from-node')
-def writeFileFromNode(self, event=None):
+def writeFileFromNode(self: Self, event: Event=None) -> None:
     """
     If node starts with @read-file-into-node, use the full path name in the headline.
     Otherwise, prompt for a file name.
@@ -908,7 +924,7 @@ def writeFileFromNode(self, event=None):
 #@+node:ekr.20031218072017.2079: ** Recent Files
 #@+node:tbrown.20080509212202.6: *3* c_file.cleanRecentFiles
 @g.commander_command('clean-recent-files')
-def cleanRecentFiles(self, event=None):
+def cleanRecentFiles(self: Self, event: Event=None) -> None:
     """
     Remove items from the recent files list that no longer exist.
 
@@ -919,19 +935,19 @@ def cleanRecentFiles(self, event=None):
     g.app.recentFilesManager.cleanRecentFiles(c)
 #@+node:ekr.20031218072017.2080: *3* c_file.clearRecentFiles
 @g.commander_command('clear-recent-files')
-def clearRecentFiles(self, event=None):
+def clearRecentFiles(self: Self, event: Event=None) -> None:
     """Clear the recent files list, then add the present file."""
     c = self
     g.app.recentFilesManager.clearRecentFiles(c)
 #@+node:vitalije.20170703115710.1: *3* c_file.editRecentFiles
 @g.commander_command('edit-recent-files')
-def editRecentFiles(self, event=None):
+def editRecentFiles(self: Self, event: Event=None) -> None:
     """Opens recent files list in a new node for editing."""
     c = self
     g.app.recentFilesManager.editRecentFiles(c)
 #@+node:ekr.20031218072017.2081: *3* c_file.openRecentFile
 @g.commander_command('open-recent-file')
-def openRecentFile(self, event=None, fn=None):
+def openRecentFile(self: Self, event: Event=None, fn: str=None) -> None:
     c = self
     if g.doHook("recentfiles1", c=c, p=c.p, v=c.p, fileName=fn):
         return
@@ -941,13 +957,13 @@ def openRecentFile(self, event=None, fn=None):
         g.doHook("recentfiles2", c=c2, p=c2.p, v=c2.p, fileName=fn)
 #@+node:tbrown.20080509212202.8: *3* c_file.sortRecentFiles
 @g.commander_command('sort-recent-files')
-def sortRecentFiles(self, event=None):
+def sortRecentFiles(self: Self, event: Event=None) -> None:
     """Sort the recent files list."""
     c = self
     g.app.recentFilesManager.sortRecentFiles(c)
 #@+node:vitalije.20170703115710.2: *3* c_file.writeEditedRecentFiles
 @g.commander_command('write-edited-recent-files')
-def writeEditedRecentFiles(self, event=None):
+def writeEditedRecentFiles(self: Self, event: Event=None) -> None:
     """
     Write content of "edit_headline" node as recentFiles and recreates
     menus.
@@ -957,7 +973,7 @@ def writeEditedRecentFiles(self, event=None):
 #@+node:vitalije.20170831154859.1: ** Reference outline commands
 #@+node:vitalije.20170831154830.1: *3* c_file.updateRefLeoFile
 @g.commander_command('update-ref-file')
-def updateRefLeoFile(self, event=None):
+def updateRefLeoFile(self: Self, event: Event=None) -> None:
     """
     Saves only the **public part** of this outline to the reference Leo
     file. The public part consists of all nodes above the **special
@@ -973,7 +989,7 @@ def updateRefLeoFile(self, event=None):
     c.fileCommands.save_ref()
 #@+node:vitalije.20170831154840.1: *3* c_file.readRefLeoFile
 @g.commander_command('read-ref-file')
-def readRefLeoFile(self, event=None):
+def readRefLeoFile(self: Self, event: Event=None) -> None:
     """
     This command *completely replaces* the **public part** of this outline
     with the contents of the reference Leo file. The public part consists
@@ -989,7 +1005,7 @@ def readRefLeoFile(self, event=None):
     c.fileCommands.updateFromRefFile()
 #@+node:vitalije.20170831154850.1: *3* c_file.setReferenceFile
 @g.commander_command('set-reference-file')
-def setReferenceFile(self, event=None):
+def setReferenceFile(self: Self, event: Event=None) -> None:
     """
     Shows a file open dialog allowing you to select a **reference** Leo
     document to which this outline will be connected.
@@ -1019,7 +1035,7 @@ def setReferenceFile(self, event=None):
 #@+node:ekr.20180312043352.1: ** Themes
 #@+node:ekr.20180312043352.2: *3* c_file.open_theme_file
 @g.commander_command('open-theme-file')
-def open_theme_file(self, event):
+def open_theme_file(self: Self, event: Event) -> None:
     """Open a theme file in a new session and apply the theme."""
     c = event and event.get('c')
     if not c:
