@@ -3,14 +3,31 @@
 #@+node:ekr.20171124073126.1: * @file ../commands/commanderHelpCommands.py
 #@@first
 """Help commands that used to be defined in leoCommands.py"""
+#@+<< commanderHelpCommands imports >>
+#@+node:ekr.20220826122759.1: ** << commanderHelpCommands imports >>
 import os
 import sys
 import time
+from typing import Any, Optional, TYPE_CHECKING
 from leo.core import leoGlobals as g
+#@-<< commanderHelpCommands imports >>
+#@+<< commanderHelpCommands annotations >>
+#@+node:ekr.20220826122824.1: ** << commanderHelpCommands annotations >>
+if TYPE_CHECKING:  # pragma: no cover
+    from leo.core.leoCommands import Commands as Cmdr
+    from leo.core.leoGui import LeoKeyEvent as Event
+    from leo.core.leoNodes import Position, VNode
+else:
+    Cmdr = Any
+    Event = Any
+    Position = Any
+    VNode = Any
+Self = Cmdr  # For @g.commander_command
+#@-<< commanderHelpCommands annotations >>
 #@+others
 #@+node:ekr.20031218072017.2939: ** c_help.about (version number & date)
 @g.commander_command('about-leo')
-def about(self, event=None):
+def about(self: Self, event: Event=None) -> None:
     """Bring up an About Leo Dialog."""
     c = self
     import datetime
@@ -26,7 +43,7 @@ def about(self, event=None):
     g.app.gui.runAboutLeoDialog(c, version, theCopyright, url, email)
 #@+node:vitalije.20170713174950.1: ** c_help.editOneSetting
 @g.commander_command('edit-setting')
-def editOneSetting(self, event=None):
+def editOneSetting(self: Self, event: Event=None) -> None:
     """Opens correct dialog for selected setting type"""
     c, p = self, self.c.p
     func = None
@@ -45,10 +62,11 @@ def editOneSetting(self, event=None):
         func(event)
 #@+node:vitalije.20170708172746.1: ** c_help.editShortcut
 @g.commander_command('edit-shortcut')
-def editShortcut(self, event=None):
+def editShortcut(self: Self, event: Event=None) -> None:
     k = self.k
     if k.isEditShortcutSensible():
-        self.k.setState('input-shortcut', 'input-shortcut')
+        # k.setState('input-shortcut', 'input-shortcut')
+        k.setState('input-shortcut', 1, None)  # Experimental.
         g.es('Press desired key combination')
     else:
         g.es('No possible shortcut in selected body line/headline')
@@ -57,7 +75,7 @@ def editShortcut(self, event=None):
 #@+node:ekr.20031218072017.2940: *3* c_help.leoDocumentation
 @g.commander_command('open-leo-docs-leo')
 @g.commander_command('leo-docs-leo')
-def leoDocumentation(self, event=None):
+def leoDocumentation(self: Self, event: Event=None) -> None:
     """Open LeoDocs.leo in a new Leo window."""
     c = self
     name = "LeoDocs.leo"
@@ -71,7 +89,7 @@ def leoDocumentation(self, event=None):
 #@+node:ekr.20090628075121.5994: *3* c_help.leoQuickStart
 @g.commander_command('open-quickstart-leo')
 @g.commander_command('leo-quickstart-leo')
-def leoQuickStart(self, event=None):
+def leoQuickStart(self: Self, event: Event=None) -> None:
     """Open quickstart.leo in a new Leo window."""
     c = self
     name = "quickstart.leo"
@@ -86,7 +104,7 @@ def leoQuickStart(self, event=None):
 @g.commander_command('open-cheat-sheet-leo')
 @g.commander_command('leo-cheat-sheet')
 @g.commander_command('cheat-sheet')
-def openCheatSheet(self, event=None):
+def openCheatSheet(self: Self, event: Event=None) -> None:
     """Open leo/doc/cheatSheet.leo"""
     c = self
     fn = g.os_path_finalize_join(g.app.loadDir, '..', 'doc', 'CheatSheet.leo')
@@ -102,7 +120,7 @@ def openCheatSheet(self, event=None):
 #@+node:lkj.20190714022527.1: *3* c_help.openDesktopIntegration
 @g.commander_command('open-desktop-integration-leo')
 @g.commander_command('desktop-integration-leo')
-def openDesktopIntegration(self, event=None):
+def openDesktopIntegration(self: Self, event: Event=None) -> None:
     """Open Desktop-integration.leo."""
     c = self
     fileName = g.os_path_finalize_join(
@@ -116,7 +134,7 @@ def openDesktopIntegration(self, event=None):
 #@+node:ekr.20161025090405.1: *3* c_help.openLeoDist
 @g.commander_command('open-leo-dist-leo')
 @g.commander_command('leo-dist-leo')
-def openLeoDist(self, event=None):
+def openLeoDist(self: Self, event: Event=None) -> None:
     """Open leoDist.leo in a new Leo window."""
     c = self
     name = "leoDist.leo"
@@ -129,7 +147,7 @@ def openLeoDist(self, event=None):
 #@+node:ekr.20151225193723.1: *3* c_help.openLeoPy
 @g.commander_command('open-leo-py-leo')
 @g.commander_command('leo-py-leo')
-def openLeoPy(self, event=None):
+def openLeoPy(self: Self, event: Event=None) -> None:
     """Open leoPy.leo or LeoPyRef.leo in a new Leo window."""
     c = self
     names = ('leoPy.leo', 'LeoPyRef.leo',)  # Used in error message.
@@ -144,7 +162,7 @@ def openLeoPy(self, event=None):
 #@+node:ekr.20201013105418.1: *3* c_help.openLeoPyRef
 @g.commander_command('open-leo-py-ref-leo')
 @g.commander_command('leo-py-ref-leo')
-def openLeoPyRef(self, event=None):
+def openLeoPyRef(self: Self, event: Event=None) -> None:
     """Open leoPyRef.leo in a new Leo window."""
     c = self
     path = g.os_path_finalize_join(g.app.loadDir, "..", "core", "LeoPyRef.leo")
@@ -157,7 +175,7 @@ def openLeoPyRef(self, event=None):
 #@+node:ekr.20061018094539: *3* c_help.openLeoScripts
 @g.commander_command('open-scripts-leo')
 @g.commander_command('leo-scripts-leo')
-def openLeoScripts(self, event=None):
+def openLeoScripts(self: Self, event: Event=None) -> None:
     """Open scripts.leo."""
     c = self
     fileName = g.os_path_finalize_join(g.app.loadDir, '..', 'scripts', 'scripts.leo')
@@ -171,7 +189,7 @@ def openLeoScripts(self, event=None):
 @g.commander_command('open-leo-settings')
 @g.commander_command('open-leo-settings-leo')  # #1343.
 @g.commander_command('leo-settings')
-def openLeoSettings(self, event=None):
+def openLeoSettings(self: Self, event: Event=None) -> Optional[Cmdr]:
     """Open leoSettings.leo in a new Leo window."""
     c, lm = self, g.app.loadManager
     path = lm.computeLeoSettingsPath()
@@ -183,7 +201,7 @@ def openLeoSettings(self, event=None):
 @g.commander_command('open-my-leo-settings')
 @g.commander_command('open-my-leo-settings-leo')  # #1343.
 @g.commander_command('my-leo-settings')
-def openMyLeoSettings(self, event=None):
+def openMyLeoSettings(self: Self, event: Event=None) -> Cmdr:
     """Open myLeoSettings.leo in a new Leo window."""
     c, lm = self, g.app.loadManager
     path = lm.computeMyLeoSettingsPath()
@@ -192,7 +210,7 @@ def openMyLeoSettings(self, event=None):
     g.es('not found: myLeoSettings.leo')
     return createMyLeoSettings(c)
 #@+node:ekr.20141119161908.2: *4* function: c_help.createMyLeoSettings
-def createMyLeoSettings(c):
+def createMyLeoSettings(c: Cmdr) -> Optional[Cmdr]:
     """createMyLeoSettings - Return true if myLeoSettings.leo created ok
     """
     name = "myLeoSettings.leo"
@@ -251,7 +269,7 @@ def createMyLeoSettings(c):
 #@+node:ekr.20171124093507.1: ** c_help.Open Leo web pages
 #@+node:ekr.20031218072017.2941: *3* c_help.leoHome
 @g.commander_command('open-online-home')
-def leoHome(self, event=None):
+def leoHome(self: Self, event: Event=None) -> None:
     """Open Leo's Home page in a web browser."""
     import webbrowser
     url = "http://leoeditor.com/"
@@ -261,7 +279,7 @@ def leoHome(self, event=None):
         g.es("not found:", url)
 #@+node:ekr.20131213072223.19441: *3* c_help.openLeoTOC
 @g.commander_command('open-online-toc')
-def openLeoTOC(self, event=None):
+def openLeoTOC(self: Self, event: Event=None) -> None:
     """Open Leo's tutorials page in a web browser."""
     import webbrowser
     url = "http://leoeditor.com/leo_toc.html"
@@ -271,7 +289,7 @@ def openLeoTOC(self, event=None):
         g.es("not found:", url)
 #@+node:ekr.20131213072223.19435: *3* c_help.openLeoTutorials
 @g.commander_command('open-online-tutorials')
-def openLeoTutorials(self, event=None):
+def openLeoTutorials(self: Self, event: Event=None) -> None:
     """Open Leo's tutorials page in a web browser."""
     import webbrowser
     url = "http://leoeditor.com/tutorial.html"
@@ -281,7 +299,7 @@ def openLeoTutorials(self, event=None):
         g.es("not found:", url)
 #@+node:ekr.20060613082924: *3* c_help.openLeoUsersGuide
 @g.commander_command('open-users-guide')
-def openLeoUsersGuide(self, event=None):
+def openLeoUsersGuide(self: Self, event: Event=None) -> None:
     """Open Leo's users guide in a web browser."""
     import webbrowser
     url = "http://leoeditor.com/usersguide.html"
@@ -291,7 +309,7 @@ def openLeoUsersGuide(self, event=None):
         g.es("not found:", url)
 #@+node:ekr.20131213072223.19437: *3* c_help.openLeoVideos
 @g.commander_command('open-online-videos')
-def openLeoVideos(self, event=None):
+def openLeoVideos(self: Self, event: Event=None) -> None:
     """Open Leo's videos page in a web browser."""
     import webbrowser
     url = "http://leoeditor.com/screencasts.html"
@@ -301,7 +319,7 @@ def openLeoVideos(self, event=None):
         g.es("not found:", url)
 #@+node:ekr.20031218072017.2932: ** c_help.openPythonWindow
 @g.commander_command('open-python-window')
-def openPythonWindow(self, event=None):
+def openPythonWindow(self: Self, event: Event=None) -> None:
     """Open Python's Idle debugger in a separate process."""
     m = g.import_module('idlelib')
     if not m:
@@ -316,7 +334,7 @@ def openPythonWindow(self, event=None):
         os.spawnve(os.P_NOWAIT, sys.executable, args, os.environ)
 #@+node:ekr.20131213072223.19532: ** c_help.selectAtSettingsNode
 @g.commander_command('open-local-settings')
-def selectAtSettingsNode(self, event=None):
+def selectAtSettingsNode(self: Self, event: Event=None) -> None:
     """Select the @settings node, if there is one."""
     c = self
     p = c.config.settingsRoot()
