@@ -3,19 +3,36 @@
 #@+node:ekr.20150624112334.1: * @file ../commands/gotoCommands.py
 #@@first
 """Leo's goto commands."""
+#@+<< gotoCommands imports >>
+#@+node:ekr.20220827065126.1: ** << gotoCommands imports >>
 import re
+from typing import Any, List, Optional, Tuple, TYPE_CHECKING
 from leo.core import leoGlobals as g
+#@-<< gotoCommands imports >>
+#@+<< gotoCommands annotations >>
+#@+node:ekr.20220827065107.1: ** << gotoCommands annotations >>
+if TYPE_CHECKING:  # pragma: no cover
+    from leo.core.leoCommands import Commands as Cmdr
+    from leo.core.leoGui import LeoKeyEvent as Event
+    from leo.core.leoNodes import Position, VNode
+else:
+    Cmdr = Any
+    Event = Any
+    Position = Any
+    VNode = Any
+#@-<< gotoCommands annotations >>
 #@+others
 #@+node:ekr.20150625050355.1: ** class GoToCommands
 class GoToCommands:
     """A class implementing goto-global-line."""
-    #@+others
-    #@+node:ekr.20100216141722.5621: *3* goto.ctor
-    def __init__(self, c):
+
+    def __init__(self, c: Cmdr) -> None:
         """Ctor for GoToCommands class."""
         self.c = c
+
+    #@+others
     #@+node:ekr.20100216141722.5622: *3* goto.find_file_line
-    def find_file_line(self, n, p=None):
+    def find_file_line(self, n: int, p: Position=None) -> Tuple[Position, int, bool]:
         """
         Place the cursor on the n'th line (one-based) of an external file.
         Return (p, offset, found) for unit testing.
@@ -46,7 +63,7 @@ class GoToCommands:
             return None, -1, False
         return self.find_script_line(n, p)
     #@+node:ekr.20160921210529.1: *3* goto.find_node_start
-    def find_node_start(self, p, s=None):
+    def find_node_start(self, p: Position, s: str=None) -> Optional[int]:
         """Return the global line number of the first line of p.b"""
         # See #283.
         root, fileName = self.find_root(p)
@@ -64,7 +81,7 @@ class GoToCommands:
                 return i + 1
         return None
     #@+node:ekr.20150622140140.1: *3* goto.find_script_line
-    def find_script_line(self, n, root):
+    def find_script_line(self, n: int, root: Position) -> Tuple[Position, int, bool]:
         """
         Go to line n (zero based) of the script with the given root.
         Return p, offset, found for unit testing.
@@ -83,7 +100,7 @@ class GoToCommands:
         self.fail(lines, n, root)
         return None, -1, False
     #@+node:ekr.20181003080042.1: *3* goto.node_offset_to_file_line
-    def node_offset_to_file_line(self, target_offset, target_p, root):
+    def node_offset_to_file_line(self, target_offset: int, target_p: Position, root: Position) -> int:
         """
         Given a zero-based target_offset within target_p.b, return the line
         number of the corresponding line within root's file.
@@ -91,7 +108,7 @@ class GoToCommands:
         delim1, delim2 = self.get_delims(root)
         file_s = self.get_external_file_with_sentinels(root)
         gnx, h, n, node_offset, target_gnx = None, None, -1, None, target_p.gnx
-        stack = []
+        stack: List[Tuple[str, str, int]] = []
         for s in g.splitLines(file_s):
             n += 1  # All lines contribute to the file's line count.
             # g.trace('%4s %4r %40r %s' % (n, node_offset, h, s.rstrip()))
@@ -110,7 +127,7 @@ class GoToCommands:
                 elif s2.startswith('@-node'):
                     gnx = node_offset = None
                 elif s2.startswith(('@+others', '@+<<'),):
-                    stack.append([gnx, h, node_offset])
+                    stack.append((gnx, h, node_offset))
                     gnx, node_offset = None, None
                 elif s2.startswith(('@-others', '@-<<'),):
                     gnx, h, node_offset = stack.pop()
@@ -123,7 +140,7 @@ class GoToCommands:
         g.trace('\nNot found', target_offset, target_gnx)
         return None
     #@+node:ekr.20150624085605.1: *3* goto.scan_nonsentinel_lines
-    def scan_nonsentinel_lines(self, lines, n, root):
+    def scan_nonsentinel_lines(self, lines: List[str], n: int, root: Position) -> Tuple[str, str, int]:
         """
         Scan a list of lines containing sentinels, looking for the node and
         offset within the node of the n'th (one-based) line.
@@ -172,7 +189,7 @@ class GoToCommands:
             gnx, h, offset = None, None, -1
         return gnx, h, offset
     #@+node:ekr.20150623175314.1: *3* goto.scan_sentinel_lines
-    def scan_sentinel_lines(self, lines, n, root):
+    def scan_sentinel_lines(self, lines: List[str], n: int, root: Position) -> Tuple[str, str, int]:
         """
         Scan a list of lines containing sentinels, looking for the node and
         offset within the node of the n'th (one-based) line.
@@ -208,7 +225,7 @@ class GoToCommands:
         return gnx, h, offset
     #@+node:ekr.20150624142449.1: *3* goto.Utils
     #@+node:ekr.20150625133523.1: *4* goto.fail
-    def fail(self, lines, n, root):
+    def fail(self, lines: Any, n: int, root: Position) -> None:
         """Select the last line of the last node of root's tree."""
         c = self.c
         w = c.frame.body.wrapper
@@ -226,7 +243,7 @@ class GoToCommands:
         c.bodyWantsFocus()
         w.seeInsertPoint()
     #@+node:ekr.20100216141722.5626: *4* goto.find_gnx
-    def find_gnx(self, root, gnx, vnodeName):
+    def find_gnx(self, root: Position, gnx: str, vnodeName: str) -> Tuple[Position, bool]:
         """
         Scan root's tree for a node with the given gnx and vnodeName.
         return (p,found)
@@ -240,7 +257,7 @@ class GoToCommands:
             return None, False
         return root, False
     #@+node:ekr.20100216141722.5627: *4* goto.find_root
-    def find_root(self, p):
+    def find_root(self, p: Position) -> Tuple[Position, str]:
         """
         Find the closest ancestor @<file> node, except @all nodes and @edit nodes.
         return root, fileName.
@@ -249,7 +266,10 @@ class GoToCommands:
         p1 = p.copy()
         # First look for ancestor @file node.
         for p in p.self_and_parents(copy=False):
-            if not p.isAtEditNode() and not p.isAtAllNode():
+            # fileName = not p2.isAtAllNode() and p2.anyAtFileNodeName()
+            # if fileName:
+                # return p2.copy(), fileName
+            if not p.isAtAllNode():
                 fileName = p.anyAtFileNodeName()
                 if fileName:
                     return p.copy(), fileName
@@ -259,12 +279,13 @@ class GoToCommands:
             if p.v == p1.v and p != p1:
                 # Found a joined position.
                 for p2 in p.self_and_parents():
-                    fileName = not p2.isAtAllNode() and p2.anyAtFileNodeName()
-                    if fileName:
-                        return p2.copy(), fileName
+                    if not p2.isAtAllNode():
+                        fileName = p2.anyAtFileNodeName()
+                        if fileName:
+                            return p2.copy(), fileName
         return None, None
     #@+node:ekr.20150625123747.1: *4* goto.get_delims
-    def get_delims(self, root):
+    def get_delims(self, root: Position) -> Tuple[str, str]:
         """Return the delimiters in effect at root."""
         c = self.c
         old_target_language = c.target_language
@@ -278,7 +299,7 @@ class GoToCommands:
             return delims1, None
         return delims2, delims3
     #@+node:ekr.20150624143903.1: *4* goto.get_external_file_with_sentinels
-    def get_external_file_with_sentinels(self, root):
+    def get_external_file_with_sentinels(self, root: Position) -> str:
         """
         root is an @<file> node. If root is an @auto node, return the result of
         writing the file *with* sentinels, even if the external file normally
@@ -301,7 +322,7 @@ class GoToCommands:
             forcePythonSentinels=False,  # See #247.
             useSentinels=True)
     #@+node:ekr.20150623175738.1: *4* goto.get_script_node_info
-    def get_script_node_info(self, s, delim2):
+    def get_script_node_info(self, s: str, delim2: Any) -> Tuple[str, str]:
         """Return the gnx and headline of a #@+node."""
         i = s.find(':', 0)
         j = s.find(':', i + 1)
@@ -315,7 +336,7 @@ class GoToCommands:
             h = h.rstrip(delim2)
         return gnx, h
     #@+node:ekr.20150625124027.1: *4* goto.is_sentinel
-    def is_sentinel(self, delim1, delim2, s):
+    def is_sentinel(self, delim1: Any, delim2: Any, s: str) -> bool:
         """Return True if s is a sentinel line with the given delims."""
         assert delim1
         i = s.find(delim1 + '@')
@@ -324,7 +345,7 @@ class GoToCommands:
             return -1 < i < j
         return -1 < i
     #@+node:ekr.20100728074713.5843: *4* goto.remove_level_stars
-    def remove_level_stars(self, s):
+    def remove_level_stars(self, s: str) -> str:
         i = g.skip_ws(s, 0)
         # Remove leading stars.
         while i < len(s) and s[i] == '*':
@@ -340,7 +361,7 @@ class GoToCommands:
             i += 1
         return s[i:]
     #@+node:ekr.20100216141722.5638: *4* goto.success
-    def success(self, n, n2, p):
+    def success(self, n: int, n2: int, p: Position) -> None:
         """Place the cursor on line n2 of p.b."""
         c = self.c
         w = c.frame.body.wrapper
@@ -358,7 +379,7 @@ class GoToCommands:
     #@-others
 #@+node:ekr.20180517041303.1: ** show-file-line
 @g.command('show-file-line')
-def show_file_line(event):
+def show_file_line(event: Event) -> None:
     c = event.get('c')
     if not c:
         return
