@@ -6,18 +6,31 @@
 #@+<< spellCommands imports >>
 #@+node:ekr.20150514050530.1: ** << spellCommands imports >>
 import re
+from typing import Any, Callable, Dict, List, Optional, Set, TYPE_CHECKING
+# Third-party annotations
 try:
+    # We can't assume the user has enchant..
     # pylint: disable=import-error
-        # We can't assume the user has this.
-    # pip install pyenchant
     import enchant
 except Exception:  # May throw WinError(!)
     enchant = None
 from leo.commands.baseCommands import BaseEditCommandsClass
 from leo.core import leoGlobals as g
 #@-<< spellCommands imports >>
+#@+<< spellCommands annotations >>
+#@+node:ekr.20220828060855.1: ** << spellCommands annotations >>
+if TYPE_CHECKING:  # pragma: no cover
+    from leo.core.leoCommands import Commands as Cmdr
+    from leo.core.leoGui import LeoKeyEvent as Event
+    from leo.core.leoNodes import Position, VNode
+else:
+    Cmdr = Any
+    Event = Any
+    Position = Any
+    VNode = Any
+#@-<< spellCommands annotations >>
 
-def cmd(name):
+def cmd(name: str) -> Callable:
     """Command decorator for the SpellCommandsClass class."""
     return g.new_cmd_decorator(name, ['c', 'spellCommands',])
 
@@ -29,11 +42,11 @@ class BaseSpellWrapper:
     # Subclasses set self.c and self.d
     #@+others
     #@+node:ekr.20180207071114.3: *3* spell.add
-    def add(self, word):
+    def add(self, word: str) -> None:
         """Add a word to the user dictionary."""
         self.d.add(word)
     #@+node:ekr.20150514063305.513: *3* spell.clean_dict
-    def clean_dict(self, fn):
+    def clean_dict(self, fn: str) -> None:
         if g.os_path_exists(fn):
             f = open(fn, mode='rb')
             s = f.read()
@@ -48,7 +61,7 @@ class BaseSpellWrapper:
                 f.write(s2)
                 f.close()
     #@+node:ekr.20180207071114.5: *3* spell.create
-    def create(self, fn):
+    def create(self, fn: str) -> None:
         """Create the given file with empty contents."""
         # Make the directories as needed.
         theDir = g.os_path_dirname(fn)
@@ -69,7 +82,7 @@ class BaseSpellWrapper:
             g.error(f"unexpected error creating: {fn}")
             g.es_exception()
     #@+node:ekr.20180207072351.1: *3* spell.find_user_dict
-    def find_user_dict(self):
+    def find_user_dict(self) -> str:
         """Return the full path to the local dictionary."""
         c = self.c
         join = g.os_path_finalize_join
@@ -88,11 +101,11 @@ class BaseSpellWrapper:
         # #1453: Return the default path.
         return join(g.app.homeDir, '.leo', 'spellpyx.txt')
     #@+node:ekr.20150514063305.515: *3* spell.ignore
-    def ignore(self, word):
+    def ignore(self, word: str) -> None:
 
         self.d.add_to_session(word)
     #@+node:ekr.20150514063305.517: *3* spell.process_word
-    def process_word(self, word):
+    def process_word(self, word: str) -> List[str]:
         """
         Check the word. Return None if the word is properly spelled.
         Otherwise, return a list of alternatives.
@@ -126,37 +139,38 @@ class BaseSpellWrapper:
 class DefaultDict:
     """A class with the same interface as the enchant dict class."""
 
-    def __init__(self, words=None):
-        self.added_words = set()
-        self.ignored_words = set()
-        self.words = set() if words is None else set(words)
+    def __init__(self, words: List[str]=None) -> None:
+        self.added_words: Set[str] = set()
+        self.ignored_words: Set[str] = set()
+        self.words: Set[str] = set() if words is None else set(words)
+
     #@+others
     #@+node:ekr.20180207075740.1: *3* dict.add
-    def add(self, word):
+    def add(self, word: str) -> None:
         """Add a word to the dictionary."""
         self.words.add(word)
         self.added_words.add(word)
     #@+node:ekr.20180207101513.1: *3* dict.add_words_from_dict
-    def add_words_from_dict(self, kind, fn, words):
+    def add_words_from_dict(self, kind: str, fn: str, words: Any) -> None:
         """For use by DefaultWrapper."""
         for word in words or []:
             self.words.add(word)
             self.words.add(word.lower())
     #@+node:ekr.20180207075751.1: *3* dict.add_to_session
-    def add_to_session(self, word):
+    def add_to_session(self, word: str) -> None:
 
         self.ignored_words.add(word)
     #@+node:ekr.20180207080007.1: *3* dict.check
-    def check(self, word):
+    def check(self, word: str) -> bool:
         """Return True if the word is in the dict."""
         for s in (word, word.lower(), word.capitalize()):
             if s in self.words or s in self.ignored_words:
                 return True
         return False
     #@+node:ekr.20180207081634.1: *3* dict.suggest & helpers
-    def suggest(self, word):
+    def suggest(self, word: str) -> List[str]:
 
-        def known(words):
+        def known(words: List[str]) -> List[str]:
             """Return the words that are in the dictionary."""
             return [z for z in list(set(words)) if z in self.words]
 
@@ -170,7 +184,7 @@ class DefaultDict:
     #@+node:ekr.20180207085717.1: *4* dict.edits1 & edits2
     #@@nobeautify
 
-    def edits1(self, word):
+    def edits1(self, word: str) -> List[str]:
         "All edits that are one edit away from `word`."
         letters    = 'abcdefghijklmnopqrstuvwxyz'
         splits     = [(word[:i], word[i:])    for i in range(len(word) + 1)]
@@ -180,7 +194,7 @@ class DefaultDict:
         inserts    = [L + c + R               for L, R in splits for c in letters]
         return list(set(deletes + transposes + replaces + inserts))
 
-    def edits2(self, word):
+    def edits2(self, word: str) -> List[str]:
         "All edits that are two edits away from `word`."
         return [e2 for e1 in self.edits1(word) for e2 in self.edits1(e1)]
     #@-others
@@ -199,7 +213,7 @@ class DefaultWrapper(BaseSpellWrapper):
     """
     #@+others
     #@+node:ekr.20180207071114.2: *3* default. __init__
-    def __init__(self, c):
+    def __init__(self, c: Cmdr) -> None:
         """Ctor for DefaultWrapper class."""
         # pylint: disable=super-init-not-called
         self.c = c
@@ -221,13 +235,13 @@ class DefaultWrapper(BaseSpellWrapper):
                 words = self.read_words(kind, fn)
                 self.d.add_words_from_dict(kind, fn, words)
     #@+node:ekr.20180207110701.1: *3* default.add
-    def add(self, word):
+    def add(self, word: str) -> None:
         """Add a word to the user dictionary."""
         self.d.add(word)
         self.d.add(word.lower())
         self.save_user_dict()
     #@+node:ekr.20180207100238.1: *3* default.find_main_dict
-    def find_main_dict(self):
+    def find_main_dict(self) -> Optional[str]:
         """Return the full path to the global dictionary."""
         c = self.c
         fn = c.config.getString('main-spelling-dictionary')
@@ -238,7 +252,7 @@ class DefaultWrapper(BaseSpellWrapper):
             g.app.homeDir, '.leo', 'main_spelling_dict.txt')
         return fn if g.os_path_exists(fn) else None
     #@+node:ekr.20180207073815.1: *3* default.read_words
-    def read_words(self, kind, fn):
+    def read_words(self, kind: Any, fn: str) -> Set[str]:
         """Return all the words from the dictionary file."""
         words = set()
         try:
@@ -253,7 +267,7 @@ class DefaultWrapper(BaseSpellWrapper):
             g.es_print(f"can not open {kind} dictionary: {fn}")
         return words
     #@+node:ekr.20180207110718.1: *3* default.save_dict
-    def save_dict(self, kind, fn, trace=False):
+    def save_dict(self, kind: str, fn: str, trace: bool=False) -> None:
         """
         Save the dictionary whose name is given, alphabetizing the file.
         Write added words to the file if kind is 'user'.
@@ -273,15 +287,15 @@ class DefaultWrapper(BaseSpellWrapper):
         f.write(g.toEncodedString(s))
         f.close()
     #@+node:ekr.20180211104628.1: *3* default.save_main/user_dict
-    def save_main_dict(self, trace=False):
+    def save_main_dict(self, trace: bool=False) -> None:
 
         self.save_dict('main', self.main_fn, trace=trace)
 
-    def save_user_dict(self, trace=False):
+    def save_user_dict(self, trace: bool=False) -> None:
 
         self.save_dict('user', self.user_fn, trace=trace)
     #@+node:ekr.20180209141933.1: *3* default.show_info
-    def show_info(self):
+    def show_info(self) -> None:
 
         if self.main_fn:
             g.es_print('Default spell checker')
@@ -305,23 +319,24 @@ class EnchantWrapper(BaseSpellWrapper):
     """A wrapper class for PyEnchant spell checker"""
     #@+others
     #@+node:ekr.20150514063305.511: *3* enchant. __init__
-    def __init__(self, c):
+    def __init__(self, c: Cmdr) -> None:
         """Ctor for EnchantWrapper class."""
         # pylint: disable=super-init-not-called
         self.c = c
         self.init_language()
         fn = self.find_user_dict()
-        g.app.spellDict = self.d = self.open_dict_file(fn)
+        self.d: Dict[str, str] = self.open_dict_file(fn)
+        g.app.spellDict = self.d
     #@+node:ekr.20180207073536.1: *3* enchant.create_dict_from_file
-    def create_dict_from_file(self, fn, language):
+    def create_dict_from_file(self, fn: str, language: Any) -> Dict[str, str]:
 
         return enchant.DictWithPWL(language, fn)
     #@+node:ekr.20180207074613.1: *3* enchant.default_dict
-    def default_dict(self, language):
+    def default_dict(self, language: Any) -> Dict[str, str]:
 
         return enchant.Dict(language)
     #@+node:ekr.20180207072846.1: *3* enchant.init_language
-    def init_language(self):
+    def init_language(self) -> None:
         """Init self.language."""
         c = self.c
         language = g.checkUnicode(c.config.getString('enchant-language'))
@@ -337,7 +352,7 @@ class EnchantWrapper(BaseSpellWrapper):
                 language = 'en_US'
         self.language = language
     #@+node:ekr.20180207102856.1: *3* enchant.open_dict_file
-    def open_dict_file(self, fn):
+    def open_dict_file(self, fn: str) -> Dict[str, str]:
         """Open or create the dict with the given fn."""
         language = self.language
         if not fn or not language:
@@ -363,11 +378,11 @@ class EnchantWrapper(BaseSpellWrapper):
             d = enchant.Dict(language)
         return d
     #@+node:ekr.20150514063305.515: *3* spell.ignore
-    def ignore(self, word):
+    def ignore(self, word: str) -> None:
 
         self.d.add_to_session(word)
     #@+node:ekr.20150514063305.517: *3* spell.process_word
-    def process_word(self, word):
+    def process_word(self, word: str) -> List[str]:
         """
         Check the word. Return None if the word is properly spelled.
         Otherwise, return a list of alternatives.
@@ -397,7 +412,7 @@ class EnchantWrapper(BaseSpellWrapper):
             return None
         return d.suggest(word)
     #@+node:ekr.20180209142310.1: *3* spell.show_info
-    def show_info(self):
+    def show_info(self) -> None:
 
         g.es_print('pyenchant spell checker')
         g.es_print(f"user dictionary:   {self.find_user_dict()}")
@@ -413,23 +428,23 @@ class SpellCommandsClass(BaseEditCommandsClass):
     """Commands to support the Spell Tab."""
     #@+others
     #@+node:ekr.20150514063305.482: *3* ctor & reloadSettings(SpellCommandsClass)
-    def __init__(self, c):
+    def __init__(self, c: Cmdr) -> None:
         """
         Ctor for SpellCommandsClass class.
         Inits happen when the first frame opens.
         """
         # pylint: disable=super-init-not-called
         self.c = c
-        self.handler = None
+        self.handler: SpellTabHandler = None
         self.reloadSettings()
 
-    def reloadSettings(self):
+    def reloadSettings(self) -> None:
         """SpellCommandsClass.reloadSettings."""
         c = self.c
         self.page_width = c.config.getInt("page-width")  # for wrapping
     #@+node:ekr.20150514063305.484: *3* openSpellTab
     @cmd('spell-tab-open')
-    def openSpellTab(self, event=None):
+    def openSpellTab(self, event: Event=None) -> None:
         """Open the Spell Checker tab in the log pane."""
         if g.unitTesting:
             return
@@ -445,15 +460,15 @@ class SpellCommandsClass(BaseEditCommandsClass):
         if not self.handler.loaded:
             log.deleteTab(tabName)
         # spell as you type stuff
-        self.suggestions = []
-        self.suggestions_idx = None
-        self.word = None
+        self.suggestions: List[str] = []
+        self.suggestions_idx: int = None
+        self.word: str = None
         self.spell_as_you_type = False
         self.wrap_as_you_type = False
     #@+node:ekr.20150514063305.485: *3* commands...(SpellCommandsClass)
     #@+node:ekr.20171205043931.1: *4* add
     @cmd('spell-add')
-    def add(self, event=None):
+    def add(self, event: Event=None) -> None:
         """
         Simulate pressing the 'add' button in the Spell tab.
 
@@ -468,7 +483,7 @@ class SpellCommandsClass(BaseEditCommandsClass):
             self.openSpellTab()
     #@+node:ekr.20150514063305.486: *4* find
     @cmd('spell-find')
-    def find(self, event=None):
+    def find(self, event: Event=None) -> None:
         """
         Simulate pressing the 'Find' button in the Spell tab.
 
@@ -483,7 +498,7 @@ class SpellCommandsClass(BaseEditCommandsClass):
             self.openSpellTab()
     #@+node:ekr.20150514063305.487: *4* change
     @cmd('spell-change')
-    def change(self, event=None):
+    def change(self, event: Event=None) -> None:
         """Simulate pressing the 'Change' button in the Spell tab."""
         if self.handler:
             self.openSpellTab()
@@ -492,7 +507,7 @@ class SpellCommandsClass(BaseEditCommandsClass):
             self.openSpellTab()
     #@+node:ekr.20150514063305.488: *4* changeThenFind
     @cmd('spell-change-then-find')
-    def changeThenFind(self, event=None):
+    def changeThenFind(self, event: Event=None) -> None:
         """Simulate pressing the 'Change, Find' button in the Spell tab."""
         if self.handler:
             self.openSpellTab()
@@ -504,14 +519,14 @@ class SpellCommandsClass(BaseEditCommandsClass):
             self.openSpellTab()
     #@+node:ekr.20150514063305.489: *4* hide
     @cmd('spell-tab-hide')
-    def hide(self, event=None):
+    def hide(self, event: Event=None) -> None:
         """Hide the Spell tab."""
         if self.handler:
             self.c.frame.log.selectTab('Log')
             self.c.bodyWantsFocus()
     #@+node:ekr.20150514063305.490: *4* ignore
     @cmd('spell-ignore')
-    def ignore(self, event=None):
+    def ignore(self, event: Event=None) -> None:
         """Simulate pressing the 'Ignore' button in the Spell tab."""
         if self.handler:
             self.openSpellTab()
@@ -520,7 +535,7 @@ class SpellCommandsClass(BaseEditCommandsClass):
             self.openSpellTab()
     #@+node:ekr.20150514063305.491: *4* focusToSpell
     @cmd('focus-to-spell-tab')
-    def focusToSpell(self, event=None):
+    def focusToSpell(self, event: Event=None) -> None:
         """Put focus in the spell tab."""
         self.openSpellTab()  # Makes Spell tab visible.
         # This is not a great idea. There is no indication of focus.
@@ -529,9 +544,8 @@ class SpellCommandsClass(BaseEditCommandsClass):
     #@+node:ekr.20150514063305.492: *3* as_you_type_* commands
     #@+node:ekr.20150514063305.493: *4* as_you_type_toggle
     @cmd('spell-as-you-type-toggle')
-    def as_you_type_toggle(self, event):
+    def as_you_type_toggle(self, event: Event) -> None:
         """as_you_type_toggle - toggle spell as you type."""
-        # c = self.c
         if self.spell_as_you_type:
             self.spell_as_you_type = False
             if not self.wrap_as_you_type:
@@ -544,9 +558,8 @@ class SpellCommandsClass(BaseEditCommandsClass):
         g.es("Spell as you type enabled")
     #@+node:ekr.20150514063305.494: *4* as_you_type_wrap
     @cmd('spell-as-you-type-wrap')
-    def as_you_type_wrap(self, event):
+    def as_you_type_wrap(self, event: Event) -> None:
         """as_you_type_wrap - toggle wrap as you type."""
-        # c = self.c
         if self.wrap_as_you_type:
             self.wrap_as_you_type = False
             if not self.spell_as_you_type:
@@ -559,7 +572,7 @@ class SpellCommandsClass(BaseEditCommandsClass):
         g.es("Wrap as you type enabled")
     #@+node:ekr.20150514063305.495: *4* as_you_type_next
     @cmd('spell-as-you-type-next')
-    def as_you_type_next(self, event):
+    def as_you_type_next(self, event: Event) -> None:
         """as_you_type_next - cycle word behind cursor to next suggestion."""
         if not self.suggestions:
             g.es('[no suggestions]')
@@ -569,7 +582,7 @@ class SpellCommandsClass(BaseEditCommandsClass):
         self.as_you_type_replace(word)
     #@+node:ekr.20150514063305.496: *4* as_you_type_undo
     @cmd('spell-as-you-type-undo')
-    def as_you_type_undo(self, event):
+    def as_you_type_undo(self, event: Event) -> None:
         """as_you_type_undo - replace word behind cursor with word
         user typed before it started cycling suggestions.
         """
@@ -578,7 +591,7 @@ class SpellCommandsClass(BaseEditCommandsClass):
             return
         self.as_you_type_replace(self.word)
     #@+node:ekr.20150514063305.497: *4* as_you_type_onkey
-    def as_you_type_onkey(self, tag, kwargs):
+    def as_you_type_onkey(self, tag: str, kwargs: Any) -> None:
         """as_you_type_onkey - handle a keystroke in the body when
         spell as you type is active
 
@@ -629,7 +642,7 @@ class SpellCommandsClass(BaseEditCommandsClass):
                 c.p.b = txt
                 w.setInsertPoint(i + 1)  # must come after c.p.b assignment
     #@+node:ekr.20150514063305.498: *4* as_you_type_replace
-    def as_you_type_replace(self, word):
+    def as_you_type_replace(self, word: str) -> None:
         """as_you_type_replace - replace the word behind the cursor
         with `word`
 
@@ -660,18 +673,18 @@ class SpellTabHandler:
     """A class to create and manage Leo's Spell Check dialog."""
     #@+others
     #@+node:ekr.20150514063305.501: *3* SpellTabHandler.__init__
-    def __init__(self, c, tabName):
+    def __init__(self, c: Cmdr, tabName: Any) -> None:
         """Ctor for SpellTabHandler class."""
         if g.app.gui.isNullGui:
             return
         self.c = c
         self.body = c.frame.body
-        self.currentWord = None
+        self.currentWord: str = None
         # Don't include underscores in words. It just complicates things.
         # [^\W\d_] means any unicode char except underscore or digit.
         self.re_word = re.compile(r"([^\W\d_]+)(['`][^\W\d_]+)?", flags=re.UNICODE)
         self.outerScrolledFrame = None
-        self.seen = set()  # Adding a word to seen will ignore it until restart.
+        self.seen: Set[str] = set()  # Adding a word to seen will ignore it until restart.
         # A text widget for scanning. Must have a parent frame.
         self.workCtrl = g.app.gui.plainTextWidget(c.frame.top)
         if enchant:
@@ -690,7 +703,7 @@ class SpellTabHandler:
             self.tab = None
     #@+node:ekr.20150514063305.502: *3* Commands
     #@+node:ekr.20150514063305.503: *4* add (spellTab)
-    def add(self, event=None):
+    def add(self, event: Event=None) -> None:
         """Add the selected suggestion to the dictionary."""
         if self.loaded:
             w = self.currentWord
@@ -698,7 +711,7 @@ class SpellTabHandler:
                 self.spellController.add(w)
                 self.tab.onFindButton()
     #@+node:ekr.20150514063305.504: *4* change (spellTab)
-    def change(self, event=None):
+    def change(self, event: Event=None) -> bool:
         """Make the selected change to the text"""
         if not self.loaded:
             return False
@@ -730,7 +743,7 @@ class SpellTabHandler:
         c.bodyWantsFocus()
         return False
     #@+node:ekr.20150514063305.505: *4* find & helper
-    def find(self, event=None):
+    def find(self, event: Event=None) -> None:
         """Find the next unknown word."""
         if not self.loaded:
             return
@@ -756,7 +769,7 @@ class SpellTabHandler:
                 k2 = ins + start + len(word)
                 if k2 < len(s) and s[k2].isdigit():
                     continue
-                alts = sc.process_word(word)
+                alts: List[str] = sc.process_word(word)
                 if alts:
                     self.currentWord = word
                     i = ins + start
@@ -783,7 +796,7 @@ class SpellTabHandler:
                 c.bodyWantsFocus()
                 return
     #@+node:ekr.20160415033936.1: *5* showMisspelled
-    def showMisspelled(self, p):
+    def showMisspelled(self, p: Position) -> None:
         """Show the position p, contracting the tree as needed."""
         c = self.c
         redraw = not p.isVisible(c)
@@ -801,10 +814,10 @@ class SpellTabHandler:
         else:
             c.selectPosition(p)
     #@+node:ekr.20150514063305.508: *4* hide
-    def hide(self, event=None):
+    def hide(self, event: Event=None) -> None:
         self.c.frame.log.selectTab('Log')
     #@+node:ekr.20150514063305.509: *4* ignore
-    def ignore(self, event=None):
+    def ignore(self, event: Event=None) -> None:
         """Ignore the incorrect word for the duration of this spell check session."""
         if self.loaded:
             w = self.currentWord
@@ -814,13 +827,13 @@ class SpellTabHandler:
     #@-others
 #@+node:ekr.20180209141207.1: ** @g.command('show-spell-info')
 @g.command('show-spell-info')
-def show_spell_info(event=None):
+def show_spell_info(event: Event=None) -> None:
     c = event.get('c')
     if c:
         c.spellCommands.handler.spellController.show_info()
 #@+node:ekr.20180211104019.1: ** @g.command('clean-main-spell-dict')
 @g.command('clean-main-spell-dict')
-def clean_main_spell_dict(event):
+def clean_main_spell_dict(event: Event) -> None:
     """
     Clean the main spelling dictionary used *only* by the default spell
     checker.
@@ -832,7 +845,7 @@ def clean_main_spell_dict(event):
         DefaultWrapper(c).save_main_dict(trace=True)
 #@+node:ekr.20180211105748.1: ** @g.command('clean-user-spell-dict')
 @g.command('clean-user-spell-dict')
-def clean_user_spell_dict(event):
+def clean_user_spell_dict(event: Event) -> None:
     """
     Clean the user spelling dictionary used *only* by the default spell
     checker. Mostly for debugging, because this happens automatically.
