@@ -10,10 +10,21 @@ import os
 import pickle
 import sqlite3
 import stat
-from typing import Any, Dict, Sequence
+from typing import Any, Dict, Generator, List, Optional, Sequence, Set, TYPE_CHECKING
 import zlib
 from leo.core import leoGlobals as g
 #@-<< leoCache imports >>
+#@+<< leoCache annotations >>
+#@+node:ekr.20220901155714.1: ** << leoCache annotations >>
+if TYPE_CHECKING:  # pragma: no cover
+    from leo.core.leoCommands import Commands as Cmdr
+    from leo.core.leoGui import LeoKeyEvent as Event
+    from leo.core.leoNodes import Position
+else:
+    Cmdr = Any
+    Event = Any
+    Position = Any
+#@-<< leoCache annotations >>
 # pylint: disable=raise-missing-from
 # Abbreviations used throughout.
 abspath = g.os_path_abspath
@@ -29,7 +40,8 @@ split = g.os_path_split
 class CommanderCacher:
     """A class to manage per-commander caches."""
 
-    def __init__(self):
+    def __init__(self) -> None:
+        self.db: Any
         try:
             path = join(g.app.homeLeoDir, 'db', 'global_data')
             self.db = SqlitePickleShare(path)
@@ -37,7 +49,7 @@ class CommanderCacher:
             self.db = {}  # type:ignore
     #@+others
     #@+node:ekr.20100209160132.5759: *3* cacher.clear
-    def clear(self):
+    def clear(self) -> None:
         """Clear the cache for all commanders."""
         # Careful: self.db may be a Python dict.
         try:
@@ -47,28 +59,28 @@ class CommanderCacher:
             g.es_exception()
             self.db = {}  # type:ignore
     #@+node:ekr.20180627062431.1: *3* cacher.close
-    def close(self):
+    def close(self) -> None:
         # Careful: self.db may be a dict.
         if hasattr(self.db, 'conn'):
             # pylint: disable=no-member
             self.db.conn.commit()
             self.db.conn.close()
     #@+node:ekr.20180627042809.1: *3* cacher.commit
-    def commit(self):
+    def commit(self) -> None:
         # Careful: self.db may be a dict.
         if hasattr(self.db, 'conn'):
             # pylint: disable=no-member
             self.db.conn.commit()
     #@+node:ekr.20180611054447.1: *3* cacher.dump
-    def dump(self):
+    def dump(self) -> None:
         """Dump the indicated cache if --trace-cache is in effect."""
         dump_cache(g.app.commander_db, tag='Commander Cache')
     #@+node:ekr.20180627053508.1: *3* cacher.get_wrapper
-    def get_wrapper(self, c, fn=None):
+    def get_wrapper(self, c: Cmdr, fn: str=None) -> "CommanderWrapper":
         """Return a new wrapper for c."""
         return CommanderWrapper(c, fn=fn)
     #@+node:ekr.20100208065621.5890: *3* cacher.test
-    def test(self):
+    def test(self) -> bool:
 
         # pylint: disable=no-member
         if g.app.gui.guiName() == 'nullGui':
@@ -90,7 +102,7 @@ class CommanderCacher:
         db.clear()
         return True
     #@+node:ekr.20100210163813.5747: *3* cacher.save
-    def save(self, c, fn):
+    def save(self, c: Cmdr, fn: str) -> None:
         """
         Save the per-commander cache.
 
@@ -111,40 +123,41 @@ class CommanderCacher:
 class CommanderWrapper:
     """A class to distinguish keys from separate commanders."""
 
-    def __init__(self, c, fn=None):
+    def __init__(self, c: Cmdr, fn: str=None) -> None:
         self.c = c
         self.db = g.app.db
         self.key = fn or c.mFileName
-        self.user_keys = set()
+        self.user_keys: Set[str] = set()
 
-    def get(self, key, default=None):
+    def get(self, key: str, default: Any=None) -> Any:
         value = self.db.get(f"{self.key}:::{key}")
         return default if value is None else value
 
-    def keys(self):
+    def keys(self) -> List[str]:
         return sorted(list(self.user_keys))
 
-    def __contains__(self, key):
+    def __contains__(self, key: Any) -> bool:
         return f"{self.key}:::{key}" in self.db
 
-    def __delitem__(self, key):
+    def __delitem__(self, key: Any) -> None:
         if key in self.user_keys:
             self.user_keys.remove(key)
         del self.db[f"{self.key}:::{key}"]
 
-    def __getitem__(self, key):
+    def __getitem__(self, key: str) -> Any:
         return self.db[f"{self.key}:::{key}"]  # May (properly) raise KeyError
 
-    def __setitem__(self, key, value):
+    def __setitem__(self, key: str, value: Any) -> None:
         self.user_keys.add(key)
         self.db[f"{self.key}:::{key}"] = value
 #@+node:ekr.20180627041556.1: ** class GlobalCacher
 class GlobalCacher:
     """A singleton global cacher, g.app.db"""
 
-    def __init__(self):
+    def __init__(self) -> None:
         """Ctor for the GlobalCacher class."""
         trace = 'cache' in g.app.debug
+        self.db: Any
         try:
             path = join(g.app.homeLeoDir, 'db', 'g_app_db')
             if trace:
@@ -159,7 +172,7 @@ class GlobalCacher:
             self.db = {}  # type:ignore
     #@+others
     #@+node:ekr.20180627045750.1: *3* g_cacher.clear
-    def clear(self):
+    def clear(self) -> None:
         """Clear the global cache."""
         # Careful: self.db may be a Python dict.
         if 'cache' in g.app.debug:
@@ -173,7 +186,7 @@ class GlobalCacher:
             g.es_exception()
             self.db = {}  # type:ignore
     #@+node:ekr.20180627042948.1: *3* g_cacher.commit_and_close()
-    def commit_and_close(self):
+    def commit_and_close(self) -> None:
         # Careful: self.db may be a dict.
         if hasattr(self.db, 'conn'):
             # pylint: disable=no-member
@@ -182,7 +195,7 @@ class GlobalCacher:
             self.db.conn.commit()
             self.db.conn.close()
     #@+node:ekr.20180627045953.1: *3* g_cacher.dump
-    def dump(self, tag=''):
+    def dump(self, tag: str='') -> None:
         """Dump the indicated cache if --trace-cache is in effect."""
         tag0 = 'Global Cache'
         tag2 = f"{tag0}: {tag}" if tag else tag0
@@ -197,19 +210,19 @@ class PickleShareDB:
     #@+others
     #@+node:ekr.20100208223942.5968: *3*  Birth & special methods
     #@+node:ekr.20100208223942.5969: *4*  __init__ (PickleShareDB)
-    def __init__(self, root):
+    def __init__(self, root: str) -> None:
         """
         Init the PickleShareDB class.
         root: The directory that contains the data. Created if it doesn't exist.
         """
-        self.root = abspath(expanduser(root))
+        self.root: str = abspath(expanduser(root))
         if not isdir(self.root) and not g.unitTesting:
             self._makedirs(self.root)
         # Keys are normalized file names.
         # Values are tuples (obj, orig_mod_time)
-        self.cache = {}
+        self.cache: Dict[str, Any] = {}
 
-        def loadz(fileobj):
+        def loadz(fileobj: Any) -> None:
             if fileobj:
                 # Retain this code for maximum compatibility.
                 try:
@@ -221,7 +234,7 @@ class PickleShareDB:
                 return val
             return None
 
-        def dumpz(val, fileobj):
+        def dumpz(val: Any, fileobj: Any) -> None:
             if fileobj:
                 try:
                     # Use Python 2's highest protocol, 2, if possible
@@ -235,11 +248,11 @@ class PickleShareDB:
         self.loader = loadz
         self.dumper = dumpz
     #@+node:ekr.20100208223942.5970: *4* __contains__(PickleShareDB)
-    def __contains__(self, key):
+    def __contains__(self, key: Any) -> bool:
 
         return self.has_key(key)  # NOQA
     #@+node:ekr.20100208223942.5971: *4* __delitem__
-    def __delitem__(self, key):
+    def __delitem__(self, key: str) -> None:
         """ del db["key"] """
         fn = join(self.root, key)
         self.cache.pop(fn, None)
@@ -250,7 +263,7 @@ class PickleShareDB:
             # lost, the other process wins the conflict
             pass
     #@+node:ekr.20100208223942.5972: *4* __getitem__ (PickleShareDB)
-    def __getitem__(self, key):
+    def __getitem__(self, key: str) -> Any:
         """ db['key'] reading """
         fn = join(self.root, key)
         try:
@@ -268,15 +281,15 @@ class PickleShareDB:
         self.cache[fn] = (obj, mtime)
         return obj
     #@+node:ekr.20100208223942.5973: *4* __iter__
-    def __iter__(self):
+    def __iter__(self) -> Generator:
 
         for k in list(self.keys()):
             yield k
     #@+node:ekr.20100208223942.5974: *4* __repr__
-    def __repr__(self):
+    def __repr__(self) -> str:
         return f"PickleShareDB('{self.root}')"
     #@+node:ekr.20100208223942.5975: *4* __setitem__ (PickleShareDB)
-    def __setitem__(self, key, value):
+    def __setitem__(self, key: str, value: Any) -> None:
         """ db['key'] = 5 """
         fn = join(self.root, key)
         parent, junk = split(fn)
@@ -290,11 +303,11 @@ class PickleShareDB:
             if e.errno != 2:
                 raise
     #@+node:ekr.20100208223942.10452: *3* _makedirs
-    def _makedirs(self, fn, mode=0o777):
+    def _makedirs(self, fn: str, mode: int=0o777) -> None:
 
         os.makedirs(fn, mode)
     #@+node:ekr.20100208223942.10458: *3* _openFile (PickleShareDB)
-    def _openFile(self, fn, mode='r'):
+    def _openFile(self, fn: str, mode: str='r') -> Optional[Any]:
         """ Open this file.  Return a file object.
 
         Do not print an error message.
@@ -305,7 +318,7 @@ class PickleShareDB:
         except Exception:
             return None
     #@+node:ekr.20100208223942.10454: *3* _walkfiles & helpers
-    def _walkfiles(self, s, pattern=None):
+    def _walkfiles(self, s: str, pattern: str=None) -> Generator:
         """ D.walkfiles() -> iterator over files in D, recursively.
 
         The optional argument, pattern, limits the results to files
@@ -321,7 +334,7 @@ class PickleShareDB:
                 for f in self._walkfiles(child, pattern):
                     yield f
     #@+node:ekr.20100208223942.10456: *4* _listdir
-    def _listdir(self, s, pattern=None):
+    def _listdir(self, s: str, pattern: str=None) -> List[str]:
         """ D.listdir() -> List of items in this directory.
 
         Use D.files() or D.dirs() instead if you want a listing
@@ -337,21 +350,21 @@ class PickleShareDB:
             names = fnmatch.filter(names, pattern)
         return [join(s, child) for child in names]
     #@+node:ekr.20100208223942.10464: *4* _fn_match
-    def _fn_match(self, s, pattern):
+    def _fn_match(self, s: str, pattern: str) -> bool:
         """ Return True if self.name matches the given pattern.
 
         pattern - A filename pattern with wildcards, for example '*.py'.
         """
         return fnmatch.fnmatch(basename(s), pattern)
     #@+node:ekr.20100208223942.5978: *3* clear (PickleShareDB)
-    def clear(self):
+    def clear(self) -> None:
         # Deletes all files in the fcache subdirectory.
         # It would be more thorough to delete everything
         # below the root directory, but it's not necessary.
         for z in self.keys():
             self.__delitem__(z)
     #@+node:ekr.20100208223942.5979: *3* get
-    def get(self, key, default=None):
+    def get(self, key: str, default: Any=None) -> Any:
 
         try:
             val = self[key]
@@ -359,7 +372,7 @@ class PickleShareDB:
         except KeyError:
             return default
     #@+node:ekr.20100208223942.5980: *3* has_key (PickleShareDB)
-    def has_key(self, key):
+    def has_key(self, key: str) -> bool:
 
         try:
             self[key]
@@ -367,29 +380,30 @@ class PickleShareDB:
             return False
         return True
     #@+node:ekr.20100208223942.5981: *3* items
-    def items(self):
+    def items(self) -> List[Any]:
         return [z for z in self]
     #@+node:ekr.20100208223942.5982: *3* keys & helpers (PickleShareDB)
     # Called by clear, and during unit testing.
 
-    def keys(self, globpat=None):
+    def keys(self, globpat: str=None) -> List[str]:
         """Return all keys in DB, or all keys matching a glob"""
+        files: List[str]
         if globpat is None:
-            files = self._walkfiles(self.root)
+            files = self._walkfiles(self.root)  # type:ignore
         else:
             # Do not call g.glob_glob here.
             files = [z for z in join(self.root, globpat)]
         result = [self._normalized(p) for p in files if isfile(p)]
         return result
     #@+node:ekr.20100208223942.5976: *4* _normalized
-    def _normalized(self, p):
+    def _normalized(self, p: str) -> str:
         """ Make a key suitable for user's eyes """
         # os.path.relpath doesn't work here.
         return self._relpathto(self.root, p).replace('\\', '/')
     #@+node:ekr.20100208223942.10460: *4* _relpathto
     # Used only by _normalized.
 
-    def _relpathto(self, src, dst):
+    def _relpathto(self, src: str, dst: str) -> str:
         """ Return a relative path from self to dst.
 
         If there is no relative path from self to dst, for example if
@@ -423,7 +437,7 @@ class PickleShareDB:
     #@+node:ekr.20100208223942.10462: *4* _splitall
     # Used by relpathto.
 
-    def _splitall(self, s):
+    def _splitall(self, s: str) -> List[str]:
         """ Return a list of the path components in this path.
 
         The first item in the list will be a path.  Its value will be
@@ -445,7 +459,7 @@ class PickleShareDB:
         parts.reverse()
         return parts
     #@+node:ekr.20100208223942.5989: *3* uncache
-    def uncache(self, *items):
+    def uncache(self, *items: Any) -> None:
         """ Removes all, or specified items from cache
 
         Use this after reading a large amount of large objects
@@ -466,16 +480,16 @@ class SqlitePickleShare:
     """ The main 'connection' object for SqlitePickleShare database """
     #@+others
     #@+node:vitalije.20170716201700.2: *3*  Birth & special methods
-    def init_dbtables(self, conn):
+    def init_dbtables(self, conn: Any) -> None:
         sql = 'create table if not exists cachevalues(key text primary key, data blob);'
         conn.execute(sql)
     #@+node:vitalije.20170716201700.3: *4*  __init__ (SqlitePickleShare)
-    def __init__(self, root):
+    def __init__(self, root: str) -> None:
         """
         Init the SqlitePickleShare class.
         root: The directory that contains the data. Created if it doesn't exist.
         """
-        self.root = abspath(expanduser(root))
+        self.root: str = abspath(expanduser(root))
         if not isdir(self.root) and not g.unitTesting:
             self._makedirs(self.root)
         dbfile = ':memory:' if g.unitTesting else join(root, 'cache.sqlite')
@@ -483,9 +497,9 @@ class SqlitePickleShare:
         self.init_dbtables(self.conn)
         # Keys are normalized file names.
         # Values are tuples (obj, orig_mod_time)
-        self.cache = {}
+        self.cache: Dict[str, Any] = {}
 
-        def loadz(data):
+        def loadz(data: Any) -> Optional[Any]:
             if data:
                 # Retain this code for maximum compatibility.
                 try:
@@ -496,7 +510,7 @@ class SqlitePickleShare:
                 return val
             return None
 
-        def dumpz(val):
+        def dumpz(val: Any) -> Any:
             try:
                 # Use Python 2's highest protocol, 2, if possible
                 data = pickle.dumps(val, protocol=2)
@@ -509,11 +523,11 @@ class SqlitePickleShare:
         self.dumper = dumpz
         self.reset_protocol_in_values()
     #@+node:vitalije.20170716201700.4: *4* __contains__(SqlitePickleShare)
-    def __contains__(self, key):
+    def __contains__(self, key: str) -> bool:
 
         return self.has_key(key)  # NOQA
     #@+node:vitalije.20170716201700.5: *4* __delitem__
-    def __delitem__(self, key):
+    def __delitem__(self, key: str) -> None:
         """ del db["key"] """
         try:
             self.conn.execute(
@@ -522,7 +536,7 @@ class SqlitePickleShare:
         except sqlite3.OperationalError:
             pass
     #@+node:vitalije.20170716201700.6: *4* __getitem__
-    def __getitem__(self, key):
+    def __getitem__(self, key: str) -> None:
         """ db['key'] reading """
         try:
             obj = None
@@ -537,15 +551,15 @@ class SqlitePickleShare:
             raise KeyError(key)
         return obj
     #@+node:vitalije.20170716201700.7: *4* __iter__
-    def __iter__(self):
+    def __iter__(self) -> Generator:
 
         for k in list(self.keys()):
             yield k
     #@+node:vitalije.20170716201700.8: *4* __repr__
-    def __repr__(self):
+    def __repr__(self) -> str:
         return f"SqlitePickleShare('{self.root}')"
     #@+node:vitalije.20170716201700.9: *4* __setitem__
-    def __setitem__(self, key, value):
+    def __setitem__(self, key: str, value: Any) -> None:
         """ db['key'] = 5 """
         try:
             data = self.dumper(value)
@@ -555,11 +569,11 @@ class SqlitePickleShare:
         except sqlite3.OperationalError:
             g.es_exception()
     #@+node:vitalije.20170716201700.10: *3* _makedirs
-    def _makedirs(self, fn, mode=0o777):
+    def _makedirs(self, fn: str, mode: int=0o777) -> None:
 
         os.makedirs(fn, mode)
     #@+node:vitalije.20170716201700.11: *3* _openFile (SqlitePickleShare)
-    def _openFile(self, fn, mode='r'):
+    def _openFile(self, fn: str, mode: str='r') -> Optional[Any]:
         """ Open this file.  Return a file object.
 
         Do not print an error message.
@@ -570,7 +584,7 @@ class SqlitePickleShare:
         except Exception:
             return None
     #@+node:vitalije.20170716201700.12: *3* _walkfiles & helpers
-    def _walkfiles(self, s, pattern=None):
+    def _walkfiles(self, s: str, pattern: str=None) -> None:
         """ D.walkfiles() -> iterator over files in D, recursively.
 
         The optional argument, pattern, limits the results to files
@@ -579,7 +593,7 @@ class SqlitePickleShare:
         extension.
         """
     #@+node:vitalije.20170716201700.13: *4* _listdir
-    def _listdir(self, s, pattern=None):
+    def _listdir(self, s: str, pattern: str=None) -> List[str]:
         """ D.listdir() -> List of items in this directory.
 
         Use D.files() or D.dirs() instead if you want a listing
@@ -595,20 +609,20 @@ class SqlitePickleShare:
             names = fnmatch.filter(names, pattern)
         return [join(s, child) for child in names]
     #@+node:vitalije.20170716201700.14: *4* _fn_match
-    def _fn_match(self, s, pattern):
+    def _fn_match(self, s: str, pattern: Any) -> bool:
         """ Return True if self.name matches the given pattern.
 
         pattern - A filename pattern with wildcards, for example '*.py'.
         """
         return fnmatch.fnmatch(basename(s), pattern)
     #@+node:vitalije.20170716201700.15: *3* clear (SqlitePickleShare)
-    def clear(self):
+    def clear(self) -> None:
         # Deletes all files in the fcache subdirectory.
         # It would be more thorough to delete everything
         # below the root directory, but it's not necessary.
         self.conn.execute('delete from cachevalues;')
     #@+node:vitalije.20170716201700.16: *3* get  (SqlitePickleShare)
-    def get(self, key, default=None):
+    def get(self, key: str, default: Any=None) -> Any:
 
         if not self.has_key(key):
             return default
@@ -618,20 +632,20 @@ class SqlitePickleShare:
         except Exception:  # #1444: Was KeyError.
             return default
     #@+node:vitalije.20170716201700.17: *3* has_key (SqlightPickleShare)
-    def has_key(self, key):
+    def has_key(self, key: str) -> bool:
         sql = 'select 1 from cachevalues where key=?;'
         for row in self.conn.execute(sql, (key,)):
             return True
         return False
     #@+node:vitalije.20170716201700.18: *3* items
-    def items(self):
+    def items(self) -> Generator:
         sql = 'select key,data from cachevalues;'
         for key, data in self.conn.execute(sql):
             yield key, data
     #@+node:vitalije.20170716201700.19: *3* keys
     # Called by clear, and during unit testing.
 
-    def keys(self, globpat=None):
+    def keys(self, globpat: str=None) -> Generator:
         """Return all keys in DB, or all keys matching a glob"""
         if globpat is None:
             sql = 'select key from cachevalues;'
@@ -643,7 +657,7 @@ class SqlitePickleShare:
         for key in self.conn.execute(sql, args):
             yield key
     #@+node:vitalije.20170818091008.1: *3* reset_protocol_in_values
-    def reset_protocol_in_values(self):
+    def reset_protocol_in_values(self) -> None:
         PROTOCOLKEY = '__cache_pickle_protocol__'
         if self.get(PROTOCOLKEY, 3) == 2:
             return
@@ -654,7 +668,7 @@ class SqlitePickleShare:
         row = json.loads(json.dumps(row[0])), json.loads(json.dumps(row[1]))
         self['viewrendered_default_layouts'] = row
         #@+node:vitalije.20170818115617.1: *4* do_block
-        def do_block(cur):
+        def do_block(cur: Any) -> Any:
             itms = tuple((self.dumper(self.loader(v)), k) for k, v in cur)
             if itms:
                 self.conn.executemany('update cachevalues set data=? where key=?', itms)
@@ -677,12 +691,12 @@ class SqlitePickleShare:
 
         self.conn.isolation_level = None
     #@+node:vitalije.20170716201700.23: *3* uncache
-    def uncache(self, *items):
+    def uncache(self, *items: Any) -> None:
         """not used in SqlitePickleShare"""
         pass
     #@-others
 #@+node:ekr.20180627050237.1: ** function: dump_cache
-def dump_cache(db, tag):
+def dump_cache(db: Any, tag: str) -> None:
     """Dump the given cache."""
     print(f'\n===== {tag} =====\n')
     if db is None:
@@ -711,7 +725,7 @@ def dump_cache(db, tag):
         heading = f"All others ({tag})" if files else None
         dump_list(heading, d.get('None'))
 
-def dump_list(heading, aList):
+def dump_list(heading: Any, aList: List) -> None:
     if heading:
         print(f'\n{heading}...\n')
     for aTuple in aList:
@@ -736,5 +750,4 @@ def dump_list(heading, aList):
 #@@language python
 #@@tabwidth -4
 #@@pagewidth 70
-
 #@-leo
