@@ -12,14 +12,18 @@ from leo.core import leoGlobals as g
 #@-<< leoChapters imports >>
 #@+<< leoChapters annotations >>
 #@+node:ekr.20220821201049.1: ** << leoChapters annotations >>
-if TYPE_CHECKING:
+if TYPE_CHECKING:  # pragma: no cover
     from leo.core.leoCommands import Commands as Cmdr
+    from leo.core.leoGui import LeoKeyEvent as Event
+    from leo.plugins.qt_frame import LeoQtTreeTab
     from leo.core.leoNodes import Position
+    from leo.plugins.qt_text import QTextEditWrapper as Wrapper
 else:
     Cmdr = Any
+    Event = Any
+    LeoQtTreeTab = Any
     Position = Any
-Event = Any
-Wrapper = Any
+    Wrapper = Any
 #@-<< leoChapters annotations >>
 #@+others
 #@+node:ekr.20150509030349.1: ** cc.cmd (decorator)
@@ -41,7 +45,7 @@ class ChapterController:
         self.re_chapter: re.Pattern = None  # Set where used.
         self.selectedChapter = None
         self.selectChapterLockout = False  # True: cc.selectChapterForPosition does nothing.
-        self.tt: Any = None  # May be set in finishCreate.
+        self.tt: LeoQtTreeTab = None  # May be set in createChaptersIcon.
         self.reloadSettings()
 
     def reloadSettings(self) -> None:
@@ -71,7 +75,7 @@ class ChapterController:
         cc.selectChapterByName('main')
         c.redraw()
     #@+node:ekr.20160411145155.1: *4* cc.makeCommand
-    def makeCommand(self, chapterName: Any, binding: str=None) -> None:
+    def makeCommand(self, chapterName: str, binding: str=None) -> None:
         """Make chapter-select-<chapterName> command."""
         c, cc = self.c, self
         commandName = f"chapter-select-{chapterName}"
@@ -81,7 +85,7 @@ class ChapterController:
         if commandName in c.commandsDict:
             return
 
-        def select_chapter_callback(event: Event, cc: Any=cc, name: Any=chapterName) -> None:
+        def select_chapter_callback(event: Event, cc: ChapterController=cc, name: str=chapterName) -> None:
             """
             Select specific chapter.
             """
@@ -290,7 +294,7 @@ class ChapterController:
         s = s.replace('--', '-')
         return s[:128]
     #@+node:ekr.20070615075643: *4* cc.selectChapterForPosition (calls c.redraw_later)
-    def selectChapterForPosition(self, p: Position, chapter: Any=None) -> None:
+    def selectChapterForPosition(self, p: Position, chapter: "Chapter"=None) -> None:
         """
         Select a chapter containing position p.
         New in Leo 4.11: prefer the given chapter if possible.
