@@ -18,10 +18,12 @@ from leo.core import leoGlobals as g
 #@-<< leoConfig imports >>
 #@+<< leoConfig annotations >>
 #@+node:ekr.20220417212402.1: ** << leoConfig annotations >>
-if TYPE_CHECKING:  # Always False at runtime.
+if TYPE_CHECKING:  # pragma: no cover
     from leo.core.leoNodes import Position
+    from leo.core.leoApp import PreviousSettings
 else:
     Position = Any
+    PreviousSettings = Any
 
 Widget = Any
 #@-<< leoConfig annotations >>
@@ -75,7 +77,7 @@ class ParserBaseClass:
     def __init__(self, c: Cmdr, localFlag: bool) -> None:
         """Ctor for the ParserBaseClass class."""
         self.c = c
-        self.clipBoard: Any = []
+        self.clipBoard: List[str] = []
         # True if this is the .leo file being opened,
         # as opposed to myLeoSettings.leo or leoSettings.leo.
         self.localFlag = localFlag
@@ -882,7 +884,7 @@ class ParserBaseClass:
             if g.os_path_finalize(c.mFileName) != g.os_path_finalize(path):
                 g.es("over-riding setting:", name, "from", path)  # 1341
         # Important: we can't use c here: it may be destroyed!
-        d[key] = g.GeneralSetting(kind,  # type:ignore
+        d[key] = g.GeneralSetting(kind,
             path=c.mFileName,
             tag='setting',
             unl=p.get_UNL() if p else '',
@@ -1492,7 +1494,7 @@ class LocalConfigManager:
     """A class to hold config settings for commanders."""
     #@+others
     #@+node:ekr.20041118104831.2: *3*  c.config.ctor
-    def __init__(self, c: Cmdr, previousSettings: Any=None) -> None:
+    def __init__(self, c: Cmdr, previousSettings: "PreviousSettings"=None) -> None:
         self.c = c
         lm = g.app.loadManager
         if previousSettings:
@@ -1664,7 +1666,7 @@ class LocalConfigManager:
         d = self.get('abbrev', 'abbrev')
         return d or {}
     #@+node:ekr.20120215072959.12523: *5* c.config.getBool
-    def getBool(self, setting: str, default: Any=None) -> bool:
+    def getBool(self, setting: str, default: bool=None) -> bool:
         """Return the value of @bool setting, or the default if the setting is not found."""
         val = self.get(setting, "bool")
         if val in (True, False):
@@ -2021,6 +2023,7 @@ class SettingsTreeParser(ParserBaseClass):
             f = self.dispatchDict.get(kind)
             if f:
                 try:
+                    # mypy: can not call function of unknown type.
                     return f(p, kind, name, val)  # type:ignore
                 except Exception:
                     g.es_exception()
