@@ -13,7 +13,7 @@ from leo.core import leoGlobals as g
 #@-<< leoExternalFiles imports >>
 #@+<< leoExternalFiles annotations >>
 #@+node:ekr.20220821203011.1: ** << leoExternalFiles annotations >>
-if TYPE_CHECKING:
+if TYPE_CHECKING:  # pragma: no cover
     from leo.core.leoCommands import Commands as Cmdr
     from leo.core.leoNodes import Position, VNode
 else:
@@ -614,7 +614,7 @@ class ExternalFilesController:
         """Return s1 + ' ' + s2"""
         return f"{s1} {s2}"
     #@+node:tbrown.20150904102518.1: *4* efc.set_time
-    def set_time(self, path: str, new_time: Any=None) -> None:
+    def set_time(self, path: str, new_time: float=None) -> None:
         """
         Implements c.setTimeStamp.
 
