@@ -29,12 +29,15 @@ from leo.core import leoNodes
 #@-<< leoFileCommands imports >>
 #@+<< leoFileCommands annotations >>
 #@+node:ekr.20220819121640.1: ** << leoFileCommands annotations >>
-if TYPE_CHECKING:
+if TYPE_CHECKING:  # pragma: no cover
     from leo.core.leoCommands import Commands as Cmdr
+    from leo.core.leoGui import LeoKeyEvent as Event
     from leo.core.leoNodes import Position, VNode
 else:
-    Cmdr = Position = VNode = Any
-Event = Any
+    Cmdr = Any
+    Event = Any
+    Position = Any
+    VNode = Any
 #@-<< leoFileCommands annotations >>
 PRIVAREA = '---begin-private-area---'
 #@+others
@@ -200,6 +203,14 @@ class FastRead:
         if attr.startswith('str_'):
             if isinstance(val, (str, bytes)):
                 return g.toUnicode(val)
+        # Support JSON encoded attributes
+        if attr.startswith('json_'):
+            if isinstance(val, (str, bytes)):
+                try:
+                    return json.loads(g.toUnicode(val))
+                except json.JSONDecodeError:
+                    # fall back to standard handling
+                    g.trace(f"attribute not JSON encoded {attr}={g.toUnicode(val)}")
         try:
             # Throws a TypeError if val is not a hex string.
             binString = binascii.unhexlify(val)
@@ -601,7 +612,7 @@ class FileCommands:
         # For writing...
         self.read_only = False
         self.rootPosition: Position = None
-        self.outputFile: Any = None  # StringIO
+        self.outputFile: io.StringIO = None
         self.openDirectory: str = None
         self.usingClipboard = False
         self.currentPosition: Position = None
@@ -1407,7 +1418,6 @@ class FileCommands:
                 if c.config.getBool('save-clears-undo-buffer'):
                     g.es("clearing undo")
                     c.undoer.clearUndoState()
-            c.redraw_after_icons_changed()
         g.doHook("save2", c=c, p=p, fileName=fileName)
         return ok
     #@+node:vitalije.20170831135146.1: *5* fc.save_ref & helpers
@@ -1518,7 +1528,6 @@ class FileCommands:
                     self.putSavedMessage(fileName)
             finally:
                 c.ignoreChangedPaths = False  # #1367.
-            c.redraw_after_icons_changed()
         g.doHook("save2", c=c, p=p, fileName=fileName)
     #@+node:ekr.20031218072017.3044: *5* fc.saveTo
     def saveTo(self, fileName: str, silent: bool=False) -> None:
@@ -1540,7 +1549,6 @@ class FileCommands:
                 c.ignoreChangedPaths = False
             if not silent:
                 self.putSavedMessage(fileName)
-            c.redraw_after_icons_changed()
         g.doHook("save2", c=c, p=p, fileName=fileName)
     #@+node:ekr.20210316034237.1: *4* fc: Writing top-level
     #@+node:vitalije.20170630172118.1: *5* fc.exportToSqlite & helpers
@@ -2161,11 +2169,22 @@ class FileCommands:
         if key.startswith('str_'):
             if isinstance(val, (str, bytes)):
                 val = g.toUnicode(val)
-                attr = f' {key}="{xml.sax.saxutils.escape(val)}"'
+                attr = f' {key}={xml.sax.saxutils.quoteattr(val)}'
                 return attr
             g.trace(type(val), repr(val))
             g.warning("ignoring non-string attribute", key, "in", torv)
             return ''
+        # Support JSON encoded attributes
+        if key.startswith('json_'):
+            try:
+                val = json.dumps(val, ensure_ascii=False, separators=(',', ':'), sort_keys=True)
+            except TypeError:
+                # fall back to pickle
+                g.trace(type(val), repr(val))
+                g.warning("pickling JSON incompatible attribute", key, "in", torv)
+            else:
+                attr = f' {key}={xml.sax.saxutils.quoteattr(val)}'
+                return attr
         return self.pickle(torv=torv, val=val, tag=key)
     #@+node:EKR.20040526202501: *5* fc.putUnknownAttributes
     def putUnknownAttributes(self, v: VNode) -> str:
@@ -2176,7 +2195,7 @@ class FileCommands:
         if isinstance(attrDict, dict):
             val = ''.join(
                 [self.putUaHelper(v, key, val)
-                    for key, val in attrDict.items()])
+                    for key, val in sorted(attrDict.items())])
             return val
         g.warning("ignoring non-dictionary unknownAttributes for", v)
         return ''
