@@ -25,9 +25,10 @@ Leo commanders.
 #@+<< leoIPython imports >>
 #@+node:ekr.20130930062914.15990: ** << leoIPython imports >>
 import sys
+from typing import Any, Dict, List, Optional, Set, TYPE_CHECKING
 from leo.core import leoGlobals as g
 
-def import_fail(s):
+def import_fail(s: str) -> None:
     if not g.unitTesting:
         print(f"leoIpython.py: can not import {s}")
 
@@ -46,10 +47,21 @@ except ImportError:
 
 g.app.ipython_inited = IPKernelApp is not None
 #@-<< leoIPython imports >>
+#@+<< leoIPython annotations >>
+#@+node:ekr.20220901090431.1: ** << leoIPython annotations >>
+if TYPE_CHECKING:  # pragma: no cover
+    from leo.core.leoCommands import Commands as Cmdr
+    from leo.core.leoGui import LeoKeyEvent as Event
+    from leo.core.leoNodes import Position
+else:
+    Cmdr = Any
+    Event = Any
+    Position = Any
+#@-<< leoIPython annotations >>
 #@+others
 #@+node:ekr.20190927110149.1: ** @g.command("ipython-new")
 @g.command("ipython-new")
-def qtshell_f(event):
+def qtshell_f(event: Event) -> None:
     """ Launch new ipython shell window, associated with the same ipython kernel """
     ipk = getattr(g.app, 'ipk', None)
     if not ipk:
@@ -58,7 +70,7 @@ def qtshell_f(event):
     g.app.ipk.new_qt_console(event=event)
 #@+node:ekr.20190927110150.1: ** @g.command("ipython-exec")
 @g.command("ipython-exec")
-def ipython_exec(event):
+def ipython_exec(event: Event) -> None:
     """ Execute script in current node in ipython namespace """
     ipk = getattr(g.app, 'ipk', None)
     if not ipk:
@@ -81,25 +93,24 @@ class InternalIPKernel:
     """
     #@+others
     #@+node:ekr.20130930062914.15994: *3* ileo.__init__
-    def __init__(self, backend='qt'):
+    def __init__(self, backend: str='qt') -> None:
         """Ctor for InternalIPKernal class."""
         # Part 1: create the ivars.
-        self.consoles = []  # List of Qt consoles.
-        self.kernelApp = None  # The IPKernelApp instance.
-        self.namespace = None  # Inited below.
+        self.consoles: List[Any] = []  # List of Qt consoles.
+        self.kernelApp: InternalIPKernel = None  # The IPKernelApp instance.
         # Keys present at startup so we don't print the entire pylab/numpy
         # namespace when the user clicks the 'namespace' button
-        self._init_keys = None
+        self._init_keys: Set[str] = None
         # Start IPython kernel with GUI event loop and pylab support.
         kernelApp = self.pylab_kernel(backend)  # Sets self.kernelApp.
         assert kernelApp == self.kernelApp
-        self.namespace = kernelApp.shell.user_ns  # Import the shell namespace.
+        self.namespace: Dict[str, Any] = kernelApp.shell.user_ns  # Import the shell namespace.
         self._init_keys = set(self.namespace.keys())
         if 'ipython' in g.app.debug:
             self.namespace['kernelApp'] = kernelApp
             self.namespace['app_counter'] = 0
     #@+node:ekr.20130930062914.15998: *3* ileo.cleanup_consoles
-    def cleanup_consoles(self, event=None):
+    def cleanup_consoles(self, event: Event=None) -> None:
         """Kill all ipython consoles.  Called from app.finishQuit."""
         trace = 'ipython' in g.app.debug
         for console in self.consoles:
@@ -110,10 +121,10 @@ class InternalIPKernel:
             # console.terminate()
             console.kill()
     #@+node:ekr.20130930062914.15997: *3* ileo.count
-    def count(self, event=None):
+    def count(self, event: Event=None) -> None:
         self.namespace['app_counter'] += 1
     #@+node:ekr.20130930062914.15996: *3* ileo.new_qt_console
-    def new_qt_console(self, event=None):
+    def new_qt_console(self, event: Event=None) -> Any:
         """
         Start a new qtconsole connected to our kernel.
 
@@ -146,7 +157,7 @@ class InternalIPKernel:
             self.put_warning(e)
         return console
     #@+node:ekr.20160331083020.1: *3* ileo.pdb
-    def pdb(self, message=''):
+    def pdb(self, message: str='') -> None:
         """Fall into pdb."""
         import pdb  # Required: we have just defined pdb as a function!
         pdb = pdb.Pdb(stdout=sys.__stdout__)  # type:ignore # mypy is confused.
@@ -154,14 +165,14 @@ class InternalIPKernel:
             self.put_stdout(message)
         pdb.set_trace()  # This works, but there are no IPython sources.
     #@+node:ekr.20130930062914.15995: *3* ileo.print_namespace
-    def print_namespace(self, event=None):
+    def print_namespace(self, event: Event=None) -> None:
         print("\n***Variables in User namespace***")
         for k, v in self.namespace.items():
             if k not in self._init_keys and not k.startswith('_'):
                 print(f"{k} -> {v!r}")
         sys.stdout.flush()
     #@+node:ekr.20160308090432.1: *3* ileo.put_log
-    def put_log(self, s, raw=False):
+    def put_log(self, s: str, raw: bool=False) -> None:
         """Put a message to the IPython kernel log."""
         if not raw:
             s = f"[leoIpython.py] {s}"
@@ -170,12 +181,12 @@ class InternalIPKernel:
         else:
             self.put_stdout(s)
     #@+node:ekr.20160331084025.1: *3* ileo.put_stdout
-    def put_stdout(self, s):
+    def put_stdout(self, s: str) -> None:
         """Put s to sys.__stdout__."""
         sys.__stdout__.write(s.rstrip() + '\n')
         sys.__stdout__.flush()
     #@+node:ekr.20160308101536.1: *3* ileo.put_warning
-    def put_warning(self, s, raw=False):
+    def put_warning(self, s: Any, raw: bool=False) -> None:
         """
         Put an warning message to the IPython kernel log.
         This will be seen, regardless of Leo's --debug option.
@@ -187,7 +198,7 @@ class InternalIPKernel:
         else:
             self.put_stdout(s)
     #@+node:ekr.20130930062914.15992: *3* ileo.pylab_kernel
-    def pylab_kernel(self, gui):
+    def pylab_kernel(self, gui: Any) -> Any:  # 'gui' arg not used.
         """Launch an IPython kernel with pylab support for the gui."""
         trace = False and not g.unitTesting  # Increased logging.
         # IPKernalApp is a singleton class.
@@ -219,12 +230,12 @@ class InternalIPKernel:
             raise
         return kernelApp
     #@+node:ekr.20190927100624.1: *3* ileo.run
-    def run(self):
+    def run(self) -> None:
         """Start the IPython kernel.  This does not return."""
         self.new_qt_console(event=None)
         self.kernelApp.start()  # This does not return.
     #@+node:ekr.20160329053849.1: *3* ileo.run_script
-    def run_script(self, file_name, script):
+    def run_script(self, file_name: str, script: str) -> None:
         """
         Helper for the ipython-exec command.
         Run the script in the qt console.
@@ -240,7 +251,7 @@ class InternalIPKernel:
         except Exception:
             g.es_exception()
     #@+node:ekr.20171115090205.1: *3* ileo.test
-    def test(self):
+    def test(self) -> None:
 
         from ipykernel.connect import connect_qtconsole
         from ipykernel.kernelapp import IPKernelApp
@@ -260,15 +271,15 @@ class LeoNameSpace:
     commander in g.app.windowList.
     """
 
-    def __init__(self):
+    def __init__(self) -> None:
         """LeoNameSpace ctor."""
-        self.commander = None  # The commander returned by the c property.
-        self.commanders_list = []  # The list of commanders returned by the commanders property.
+        self.commander: Cmdr = None  # The commander returned by the c property.
+        self.commanders_list: List[Cmdr] = []  # The list of commanders returned by the commanders property.
         self.g = g
         self.update()
     #@+others
     #@+node:ekr.20130930062914.16006: *3* LeoNS.c property
-    def __get_c(self):
+    def __get_c(self) -> Optional[Cmdr]:
         """Return the designated commander, or the only open commander."""
         self.update()
         if self.commander and self.commander in self.commanders_list:
@@ -277,7 +288,7 @@ class LeoNameSpace:
             return self.commanders_list[0]
         return None
 
-    def __set_c(self, c):
+    def __set_c(self, c: Cmdr) -> None:
         """Designate the commander to be returned by the getter."""
         self.update()
         if c in self.commanders_list:
@@ -290,7 +301,7 @@ class LeoNameSpace:
         __get_c, __set_c,
         doc="LeoNameSpace c property")
     #@+node:edward.20130930125732.11822: *3* LeoNS.commanders property
-    def __get_commanders(self):
+    def __get_commanders(self) -> List[Cmdr]:
         """Return the designated commander, or the only open commander."""
         self.update()
         return self.commanders_list
@@ -299,9 +310,9 @@ class LeoNameSpace:
         __get_commanders, None,
         doc="LeoNameSpace commanders property (read-only)")
     #@+node:ekr.20130930062914.16009: *3* LeoNS.find_c
-    def find_c(self, path):
+    def find_c(self, path: str) -> Optional[Cmdr]:
         """Return the commander associated with path, or None."""
-        g = self.g  # type:ignore # mypy seems confused. g is a local var.
+        g = self.g
         self.update()
         path = g.os_path_normcase(path)
         short_path = g.shortFileName(path)
@@ -312,7 +323,7 @@ class LeoNameSpace:
                 return c
         return None
     #@+node:ekr.20130930062914.16003: *3* LeoNS.update
-    def update(self):
+    def update(self) -> None:
         """Update the list of available commanders."""
         self.commanders_list = [frame.c for frame in g.app.windowList]
     #@-others
