@@ -5,13 +5,17 @@
 """Gui-independent menu handling for Leo."""
 #@+<< leoMenu imports >>
 #@+node:ekr.20220414095908.1: ** << leoMenu imports >>
-from typing import Any, Callable, Dict, List, Tuple
+from typing import Any, Callable, Dict, List, Tuple, TYPE_CHECKING
 from leo.core import leoGlobals as g
 #@-<< leoMenu imports >>
 #@+<< leoMenu annotations >>
 #@+node:ekr.20220824085300.1: ** << leoMenu annotations >>
-Event = Any
+if TYPE_CHECKING:  # pragma: no cover
+    from leo.core.leoGui import LeoKeyEvent as Event
+else:
+    Event = Any
 Widget = Any
+Wrapper = Any
 #@-<< leoMenu annotations >>
 #@+others
 #@+node:ekr.20031218072017.3750: ** class LeoMenu
@@ -25,6 +29,7 @@ class LeoMenu:
         self.frame = frame
         self.isNull = False
         self.menus: Dict[str, Any] = {}  # Menu dictionary.
+        self.wrapper: Wrapper = None
 
     def finishCreate(self) -> None:
         self.define_enable_dict()
@@ -679,7 +684,7 @@ class LeoMenu:
     def setMenuLabel(self, menu: str, name: str, label: str, underline: int=-1) -> None:
         self.oops()
     #@-others
-#@+node:ekr.20031218072017.3811: ** class NullMenu
+#@+node:ekr.20031218072017.3811: ** class NullMenu(LeoMenu)
 class NullMenu(LeoMenu):
     """A null menu class for testing and batch execution."""
     #@+others
