@@ -37,7 +37,7 @@ from leo.core import leoGlobals as g
 #@-<< leoShadow imports >>
 #@+<< leoShadow annotations >>
 #@+node:ekr.20220821091505.1: ** << leoShadow annotations >>
-if TYPE_CHECKING:
+if TYPE_CHECKING:  # pragma: no cover
     from leo.core.leoCommands import Commands as Cmdr
     from leo.core.leoNodes import Position, VNode
 else:
