@@ -47,25 +47,30 @@
 #@-<< How Leo implements unlimited undo >>
 #@+<< leoUndo imports >>
 #@+node:ekr.20220821074023.1: ** << leoUndo imports >>
-from typing import Any, Callable, TYPE_CHECKING
+from typing import Any, Callable, List, Tuple, TYPE_CHECKING
 from leo.core import leoGlobals as g
 #@-<< leoUndo imports >>
 #@+<< leoUndo annotations >>
 #@+node:ekr.20220821074054.1: ** << leoUndo annotations >>
-if TYPE_CHECKING:
+if TYPE_CHECKING:  # pragma: no cover
     from leo.core.leoCommands import Commands as Cmdr
+    from leo.core.leoGui import LeoKeyEvent as Event
     from leo.core.leoNodes import Position, VNode
+    from leo.plugins.qt_text import QTextEditWrapper as Wrapper
 else:
-    Cmdr = Position = VNode = Any
-Event = Any
-Wrapper = Any
+    Cmdr = Any
+    Event = Any
+    Position = Any
+    VNode = Any
+    Wrapper = Any
 #@-<< leoUndo annotations >>
 # pylint: disable=unpacking-non-sequence
-#@+others
-#@+node:ekr.20150509193222.1: ** u.cmd (decorator)
-def cmd(name: Any) -> Callable:
+
+def cmd(name: str) -> Callable:
     """Command decorator for the Undoer class."""
     return g.new_cmd_decorator(name, ['c', 'undoer',])
+
+#@+others
 #@+node:ekr.20031218072017.3605: ** class Undoer
 class Undoer:
     """A class that implements unlimited undo and redo."""
@@ -94,13 +99,14 @@ class Undoer:
         # New in 4.2...
         self.optionalIvars = []
         # Set the following ivars to keep pylint happy.
+        # mypy doesn't care about these.
         self.afterTree = None
         self.beforeTree = None
         self.children = None
-        self.deleteMarkedNodesData = None
-        self.followingSibs = None
-        self.inHead = None
-        self.kind = None
+        self.deleteMarkedNodesData: g.Bunch = None
+        self.followingSibs: List[VNode] = None
+        self.inHead: bool = None
+        self.kind: str = None
         self.newBack = None
         self.newBody = None
         self.newChildren = None
@@ -187,7 +193,7 @@ class Undoer:
             return self.dumpBead(n - 1)
         return '<no top bead>'
     #@+node:EKR.20040526150818: *4* u.getBead
-    def getBead(self, n: int) -> Any:
+    def getBead(self, n: int) -> g.Bunch:
         """Set Undoer ivars from the bunch at the top of the undo stack."""
         u = self
         if n < 0 or n >= len(u.beads):
@@ -198,14 +204,14 @@ class Undoer:
             print(f" u.getBead: {n:3} of {len(u.beads)}")
         return bunch
     #@+node:EKR.20040526150818.1: *4* u.peekBead
-    def peekBead(self, n: int) -> Any:
+    def peekBead(self, n: int) -> g.Bunch:
 
         u = self
         if n < 0 or n >= len(u.beads):
             return None
         return u.beads[n]
     #@+node:ekr.20060127113243: *4* u.pushBead
-    def pushBead(self, bunch: Any) -> None:
+    def pushBead(self, bunch: g.Bunch) -> None:
         u = self
         # New in 4.4b2:  Add this to the group if it is being accumulated.
         bunch2 = u.bead >= 0 and u.bead < len(u.beads) and u.beads[u.bead]
@@ -231,7 +237,7 @@ class Undoer:
             return name
         return "Undo " + name
     #@+node:ekr.20060127070008: *4* u.setIvarsFromBunch
-    def setIvarsFromBunch(self, bunch: Any) -> None:
+    def setIvarsFromBunch(self, bunch: g.Bunch) -> None:
         u = self
         u.clearOptionalIvars()
         if False and not g.unitTesting:  # Debugging. # pragma: no cover
@@ -315,7 +321,7 @@ class Undoer:
             u.setRedoType("Can't Redo")
         u.cutStack()
     #@+node:EKR.20040530121329: *4* u.restoreTree & helpers
-    def restoreTree(self, treeInfo: Any) -> None:
+    def restoreTree(self, treeInfo: g.Bunch) -> None:
         """Use the tree info to restore all VNode data,
         including all links."""
         u = self
@@ -323,7 +329,7 @@ class Undoer:
         for v, vInfo in treeInfo:
             u.restoreVnodeUndoInfo(vInfo)
     #@+node:ekr.20050415170737.2: *5* u.restoreVnodeUndoInfo
-    def restoreVnodeUndoInfo(self, bunch: Any) -> None:
+    def restoreVnodeUndoInfo(self, bunch: g.Bunch) -> None:
         """Restore all ivars saved in the bunch."""
         v = bunch.v
         v.statusBits = bunch.statusBits
@@ -334,7 +340,7 @@ class Undoer:
             v.unknownAttributes = uA
             v._p_changed = True
     #@+node:ekr.20050415170812.2: *5* u.restoreTnodeUndoInfo
-    def restoreTnodeUndoInfo(self, bunch: Any) -> None:
+    def restoreTnodeUndoInfo(self, bunch: g.Bunch) -> None:
         v = bunch.v
         v.h = bunch.headString
         v.b = bunch.bodyString
@@ -344,7 +350,7 @@ class Undoer:
             v.unknownAttributes = uA
             v._p_changed = True
     #@+node:EKR.20040528075307: *4* u.saveTree & helpers
-    def saveTree(self, p: Position, treeInfo: Any=None) -> Any:
+    def saveTree(self, p: Position, treeInfo: g.Bunch=None) -> g.Bunch:
         """Return a list of tuples with all info needed to handle a general undo operation."""
         # WARNING: read this before doing anything "clever"
         #@+<< about u.saveTree >>
@@ -380,7 +386,7 @@ class Undoer:
             child = child.next()
         return treeInfo
     #@+node:ekr.20050415170737.1: *5* u.createVnodeUndoInfo
-    def createVnodeUndoInfo(self, v: VNode) -> Any:
+    def createVnodeUndoInfo(self, v: VNode) -> g.Bunch:
         """Create a bunch containing all info needed to recreate a VNode for undo."""
         bunch = g.Bunch(
             v=v,
@@ -417,7 +423,7 @@ class Undoer:
     #@+node:ekr.20031218072017.3608: *3* u.Externally visible entries
     #@+node:ekr.20050318085432.4: *4* u.afterX...
     #@+node:ekr.20201109075104.1: *5* u.afterChangeBody
-    def afterChangeBody(self, p: Position, command: str, bunch: Any) -> None:
+    def afterChangeBody(self, p: Position, command: str, bunch: g.Bunch) -> None:
         """
         Create an undo node using d created by beforeChangeNode.
 
@@ -494,7 +500,7 @@ class Undoer:
         # Recalculate the menu labels.
         u.setUndoTypes()
     #@+node:ekr.20050315134017.2: *5* u.afterChangeNodeContents
-    def afterChangeNodeContents(self, p: Position, command: str, bunch: Any) -> None:
+    def afterChangeNodeContents(self, p: Position, command: str, bunch: g.Bunch) -> None:
         """Create an undo node using d created by beforeChangeNode."""
         u = self
         c = self.c
@@ -518,7 +524,7 @@ class Undoer:
         bunch.newYScroll = w.getYScrollPosition() if w else 0
         u.pushBead(bunch)
     #@+node:ekr.20201107145642.1: *5* u.afterChangeHeadline
-    def afterChangeHeadline(self, p: Position, command: str, bunch: Any) -> None:
+    def afterChangeHeadline(self, p: Position, command: str, bunch: g.Bunch) -> None:
         """Create an undo node using d created by beforeChangeHeadline."""
         u = self
         if u.redoing or u.undoing:
@@ -533,7 +539,7 @@ class Undoer:
 
     afterChangeHead = afterChangeHeadline
     #@+node:ekr.20050315134017.3: *5* u.afterChangeTree
-    def afterChangeTree(self, p: Position, command: str, bunch: Any) -> None:
+    def afterChangeTree(self, p: Position, command: str, bunch: g.Bunch) -> None:
         """Create an undo node for general tree operations using d created by beforeChangeTree"""
         u = self
         c = self.c
@@ -551,7 +557,7 @@ class Undoer:
         bunch.newTree = u.saveTree(p)
         u.pushBead(bunch)
     #@+node:ekr.20050424161505: *5* u.afterClearRecentFiles
-    def afterClearRecentFiles(self, bunch: Any) -> None:
+    def afterClearRecentFiles(self, bunch: g.Bunch) -> None:
         u = self
         bunch.newRecentFiles = g.app.config.recentFiles[:]
         bunch.undoType = 'Clear Recent Files'
@@ -600,7 +606,7 @@ class Undoer:
         bunch.newMarked = p.isMarked()
         u.pushBead(bunch)
     #@+node:ekr.20050411193627.5: *5* u.afterCloneNode
-    def afterCloneNode(self, p: Position, command: str, bunch: Any) -> None:
+    def afterCloneNode(self, p: Position, command: str, bunch: g.Bunch) -> None:
         u = self
         if u.redoing or u.undoing:
             return  # pragma: no cover
@@ -616,7 +622,7 @@ class Undoer:
         bunch.newMarked = p.isMarked()
         u.pushBead(bunch)
     #@+node:ekr.20050411193627.8: *5* u.afterDeleteNode
-    def afterDeleteNode(self, p: Position, command: str, bunch: Any) -> None:
+    def afterDeleteNode(self, p: Position, command: str, bunch: g.Bunch) -> None:
         u = self
         if u.redoing or u.undoing:
             return
@@ -630,7 +636,7 @@ class Undoer:
         bunch.newMarked = p.isMarked()
         u.pushBead(bunch)
     #@+node:ekr.20111005152227.15555: *5* u.afterDeleteMarkedNodes
-    def afterDeleteMarkedNodes(self, data: Any, p: Position) -> None:
+    def afterDeleteMarkedNodes(self, data: g.Bunch, p: Position) -> None:
         u = self
         if u.redoing or u.undoing:
             return
@@ -646,7 +652,7 @@ class Undoer:
         bunch.newMarked = p.isMarked()
         u.pushBead(bunch)
     #@+node:ekr.20080425060424.8: *5* u.afterDemote
-    def afterDemote(self, p: Position, followingSibs: Any) -> None:
+    def afterDemote(self, p: Position, followingSibs: List[VNode]) -> None:
         """Create an undo node for demote operations."""
         u = self
         bunch = u.createCommonBunch(p)
@@ -662,7 +668,7 @@ class Undoer:
         # Recalculate the menu labels.
         u.setUndoTypes()
     #@+node:ekr.20050411193627.9: *5* u.afterInsertNode
-    def afterInsertNode(self, p: Position, command: str, bunch: Any) -> None:
+    def afterInsertNode(self, p: Position, command: str, bunch: g.Bunch) -> None:
         u = self
         if u.redoing or u.undoing:
             return
@@ -685,7 +691,7 @@ class Undoer:
             bunch.afterTree = afterTree
         u.pushBead(bunch)
     #@+node:ekr.20050526124257: *5* u.afterMark
-    def afterMark(self, p: Position, command: str, bunch: Any) -> None:
+    def afterMark(self, p: Position, command: str, bunch: g.Bunch) -> None:
         """Create an undo node for mark and unmark commands."""
         # 'command' unused, but present for compatibility with similar methods.
         u = self
@@ -697,7 +703,7 @@ class Undoer:
         bunch.newMarked = p.isMarked()
         u.pushBead(bunch)
     #@+node:ekr.20050410110343: *5* u.afterMoveNode
-    def afterMoveNode(self, p: Position, command: str, bunch: Any) -> None:
+    def afterMoveNode(self, p: Position, command: str, bunch: g.Bunch) -> None:
         u = self
         if u.redoing or u.undoing:
             return
@@ -714,7 +720,7 @@ class Undoer:
         bunch.newP = p.copy()
         u.pushBead(bunch)
     #@+node:ekr.20080425060424.12: *5* u.afterPromote
-    def afterPromote(self, p: Position, children: Any) -> None:
+    def afterPromote(self, p: Position, children: List[VNode]) -> None:
         """Create an undo node for demote operations."""
         u = self
         bunch = u.createCommonBunch(p)
@@ -730,7 +736,7 @@ class Undoer:
         # Recalculate the menu labels.
         u.setUndoTypes()
     #@+node:ekr.20080425060424.2: *5* u.afterSort
-    def afterSort(self, p: Position, bunch: Any) -> None:
+    def afterSort(self, p: Position, bunch: g.Bunch) -> None:
         """Create an undo node for sort operations"""
         u = self
         # c = self.c
@@ -826,7 +832,7 @@ class Undoer:
         bunch.oldParent = p.parent()
         return bunch
     #@+node:ekr.20050411193627.4: *5* u.beforeInsertNode
-    def beforeInsertNode(self, p: Position, pasteAsClone: bool=False, copiedBunchList: Any=None) -> None:
+    def beforeInsertNode(self, p: Position, pasteAsClone: bool=False, copiedBunchList: List[g.Bunch]=None) -> None:
         u = self
         if copiedBunchList is None:
             copiedBunchList = []
@@ -851,7 +857,13 @@ class Undoer:
         bunch.oldParent_v = p._parentVnode()
         return bunch
     #@+node:ekr.20080425060424.3: *5* u.beforeSort
-    def beforeSort(self, p: Position, undoType: Any, oldChildren: Any, newChildren: Any, sortChildren: Any) -> None:
+    def beforeSort(self,
+        p: Position,
+        undoType: str,
+        oldChildren: List[VNode],
+        newChildren: List[VNode],
+        sortChildren: List[VNode],
+    ) -> None:
         """Create an undo node for sort operations."""
         u = self
         bunch = u.createCommonBunch(p)
@@ -904,13 +916,13 @@ class Undoer:
     def doTyping(
         self,
         p: Position,
-        undo_type: Any,
-        oldText: Any,
-        newText: Any,
-        newInsert: Any=None,
-        oldSel: Any=None,
-        newSel: Any=None,
-        oldYview: Any=None,
+        undo_type: str,
+        oldText: str,
+        newText: str,
+        newInsert: int=None,
+        oldSel: Tuple[int, int]=None,
+        newSel: Tuple[int, int]=None,
+        oldYview: int=None,
     ) -> None:
         """
         Save enough information to undo or redo a typing operation efficiently,
@@ -1156,16 +1168,16 @@ class Undoer:
     #@+node:ekr.20050126081529: *5* u.recognizeStartOfTypingWord
     def recognizeStartOfTypingWord(
         self,
-        old_lines: Any,
-        old_row: Any,
-        old_col: Any,
-        old_ch: Any,
-        new_lines: Any,
-        new_row: Any,
-        new_col: Any,
-        new_ch: Any,
-        prev_row: Any,
-        prev_col: Any,
+        old_lines: List[str],
+        old_row: int,
+        old_col: int,
+        old_ch: str,
+        new_lines: List[str],
+        new_row: int,
+        new_col: int,
+        new_ch: str,
+        prev_row: int,
+        prev_col: int,
     ) -> None:
         """
         A potentially user-modifiable method that should return True if the
@@ -1196,7 +1208,7 @@ class Undoer:
             frame.menu.enableMenu(menu, u.redoMenuLabel, u.canRedo())
             frame.menu.enableMenu(menu, u.undoMenuLabel, u.canUndo())
     #@+node:ekr.20110519074734.6094: *4* u.onSelect & helpers
-    def onSelect(self, old_p: Any, p: Position) -> None:
+    def onSelect(self, old_p: Position, p: Position) -> None:
 
         u = self
         if u.per_node_undo:
@@ -1241,8 +1253,6 @@ class Undoer:
                 g.trace(
                     f"\np.b != w.getAllText() p: {p.h} \n"
                     f"w: {w!r} \n{g.callers()}\n")
-                # g.printObj(g.splitLines(p.b), tag='p.b')
-                # g.printObj(g.splitLines(all), tag='getAllText')
             p.v.insertSpot = ins = w.getInsertPoint()
             # From u.doTyping.
             newSel = w.getSelectionRange()
@@ -1257,13 +1267,8 @@ class Undoer:
             g.trace('Not a text wrapper')
             p.v.insertSpot = 0
             p.v.selectionStart, p.v.selectionLength = (0, 0)
-        #
-        # #1749.
-        if p.isDirty():
-            redraw_flag = False
-        else:
-            p.setDirty()  # Do not call p.v.setDirty!
-            redraw_flag = True
+        if not p.isDirty():
+            p.setDirty()
         if not c.isChanged():
             c.setChanged()
         # Update editors.
@@ -1272,13 +1277,9 @@ class Undoer:
         val = p.computeIcon()
         if not hasattr(p.v, "iconVal") or val != p.v.iconVal:
             p.v.iconVal = val
-            redraw_flag = True
-        #
         # Recolor the body.
         c.frame.scanForTabWidth(p)  # Calls frame.setTabWidth()
         c.recolor()
-        if redraw_flag:
-            c.redraw_after_icons_changed()
         w.setFocus()
     #@+node:ekr.20031218072017.2030: *3* u.redo
     @cmd('redo')
@@ -1922,14 +1923,14 @@ class Undoer:
     def undoRedoText(
         self,
         p: Position,
-        leading: Any,
-        trailing: Any,  # Number of matching leading & trailing lines.
-        oldMidLines: Any,
-        newMidLines: Any,  # Lists of unmatched lines.
-        oldNewlines: Any,
-        newNewlines: Any,  # Number of trailing newlines.
+        leading: int,
+        trailing: int,  # Number of matching leading & trailing lines.
+        oldMidLines: List[str],
+        newMidLines: List[str],  # Lists of unmatched lines.
+        oldNewlines: List[str],
+        newNewlines: List[str],  # Number of trailing newlines.
         tag: str="undo",  # "undo" or "redo"
-        undoType: Any=None,
+        undoType: str=None,
     ) -> None:
         """Handle text undo and redo: converts _new_ text into _old_ text."""
         # newNewlines is unused, but it has symmetry.
@@ -1968,7 +1969,7 @@ class Undoer:
         c.frame.body.recolor(p)
         w.seeInsertPoint()  # 2009/12/21
     #@+node:ekr.20050408100042: *4* u.undoRedoTree
-    def undoRedoTree(self, new_data: Any, old_data: Any) -> None:
+    def undoRedoTree(self, new_data: g.Bunch, old_data: g.Bunch) -> Position:
         """Replace p and its subtree using old_data during undo."""
         # Same as undoReplace except uses g.Bunch.
         c, p, u = self.c, self.c.p, self
