@@ -2,6 +2,8 @@
 #@+leo-ver=5-thin
 #@+node:tbrown.20171028115541.1: * @file signal_manager.py
 #@@first
+#@+<< signal_manager docstring >>
+#@+node:ekr.20220901092728.1: ** << signal_manager docstring >>
 """
 signal_manager.py - SignalManager - light weight signal management
 
@@ -10,14 +12,30 @@ even explicit listing of which signals exist.
 
 Terry Brown, terrynbrown@gmail.com, Thu Mar 23 21:13:38 2017
 """
+#@-<< signal_manager docstring >>
+#@+<< signal_manager imports >>
+#@+node:ekr.20220901092745.1: ** << signal_manager imports >>
 from collections import defaultdict
+from typing import Any, Callable, Dict, List, TYPE_CHECKING
+#@-<< signal_manager imports >>
+#@+<< signal_manager annotations >>
+#@+node:ekr.20220901092755.1: ** << signal_manager annotations >>
+if TYPE_CHECKING:  # pragma: no cover
+    from leo.core.leoCommands import Commands as Cmdr
+    from leo.core.leoGui import LeoKeyEvent as Event
+    from leo.core.leoNodes import Position
+else:
+    Cmdr = Any
+    Event = Any
+    Position = Any
+#@-<< signal_manager annotations >>
 #@+others
 #@+node:tbrown.20171028115601.2: ** class SignalData
 class SignalData:
 
-    def __init__(self):
-        self.listeners = defaultdict(list)
-        self.emitters = []
+    def __init__(self) -> None:
+        self.listeners: Dict[Any, Any] = defaultdict(list)
+        self.emitters: List[Callable] = []
         self.locked = False
 #@+node:tbrown.20171028115601.4: ** class MsgSignalHandled
 class MsgSignalHandled:
@@ -26,11 +44,11 @@ class MsgSignalHandled:
     """
     pass
 #@+node:tbrown.20171028115601.5: ** _setup
-def _setup(obj):
+def _setup(obj: Any) -> None:
     if not hasattr(obj, '_signal_data'):
         obj._signal_data = SignalData()
 #@+node:tbrown.20171028115601.6: ** emit
-def emit(source, signal_name, *args, **kwargs):
+def emit(source: Any, signal_name: str, *args: Any, **kwargs: Any) -> None:
     """Emit signal to all listeners"""
     if not hasattr(source, '_signal_data'):
         return
@@ -54,7 +72,7 @@ def emit(source, signal_name, *args, **kwargs):
     if obj_to_lock is not None:
         obj_to_lock._signal_data.locked = False
 #@+node:tbrown.20171028115601.7: ** connect
-def connect(source, signal_name, listener):
+def connect(source: Any, signal_name: str, listener: Any) -> None:
     """Connect to signal"""
     _setup(source)
     source._signal_data.listeners[signal_name].append(listener)
@@ -64,7 +82,7 @@ def connect(source, signal_name, listener):
         _setup(obj)
         obj._signal_data.emitters.append(source)
 #@+node:tbrown.20171028115601.8: ** disconnect_all
-def disconnect_all(listener):
+def disconnect_all(listener: Any) -> None:
     """Disconnect from all signals"""
     for emitter in listener._signal_data.emitters:
         for signal in emitter._signal_data.listeners:
@@ -73,14 +91,14 @@ def disconnect_all(listener):
                 if getattr(i, '__self__', None) != listener
             ]
 #@+node:tbrown.20171028115601.9: ** is_locked
-def is_locked(obj):
+def is_locked(obj: Any) -> bool:
     return hasattr(obj, '_signal_data') and obj._signal_data.locked
 #@+node:tbrown.20171028115601.10: ** lock
-def lock(obj):
+def lock(obj: Any) -> None:
     _setup(obj)
     obj._signal_data.locked = True
 #@+node:tbrown.20171028115601.11: ** unlock
-def unlock(obj):
+def unlock(obj: Any) -> None:
     _setup(obj)
     obj._signal_data.locked = False
 #@+node:tbrown.20171028115601.12: ** class SignalManager
@@ -88,16 +106,16 @@ class SignalManager:
     """SignalManager - light weight signal management mixin."""
     #@+others
     #@+node:tbrown.20171028115601.13: *3* emit
-    def emit(self, signal_name, *args, **kwargs):
+    def emit(self, signal_name: str, *args: Any, **kwargs: Any) -> None:
         """Emit signal to all listeners"""
         emit(self, signal_name, *args, **kwargs)
     #@+node:tbrown.20171028115601.14: *3* connect
-    def connect(self, signal_name, listener):
+    def connect(self, signal_name: str, listener: Any) -> None:
         """Connect to signal"""
         connect(self, signal_name, listener)
     #@-others
 #@+node:tbrown.20171028115601.15: ** main
-def main():
+def main() -> None:
     """test of SignalManager"""
 
     # simple use
@@ -105,10 +123,10 @@ def main():
 
     class Emitter(SignalManager):
 
-        def some_emission(self):
+        def some_emission(self) -> None:
             self.emit('the_emission', 12, [1, 2, 3])
 
-    def hear_emit(n, l):
+    def hear_emit(n: int, l: Any) -> None:
         print(f"Got {n} {l}")
 
     emitter = Emitter()
@@ -120,15 +138,15 @@ def main():
 
     class Tester:
 
-        def __init__(self, name, relay):
+        def __init__(self, name: str, relay: Any) -> None:
             self.name = name
             self.relay = relay
             connect(self.relay, 'work_done', self.check_work)
 
-        def do_work(self):
+        def do_work(self) -> None:
             emit(self.relay, 'work_done', 4.2, animal='otters', _sig_lock=self)
 
-        def check_work(self, num, animal='eels'):
+        def check_work(self, num: Any, animal: str='eels') -> None:
             if is_locked(self):
                 return
             print(f"{self.name} heard about {num} {animal}")
