@@ -91,18 +91,13 @@ This can be much slower if you have a huge database.
 #@-<< docstring >>
 #@+<< imports >>
 #@+node:ekr.20110310091639.14293: ** << imports >>
-import sys
-# pylint: disable=unidiomatic-typecheck
-isPython3 = sys.version_info >= (3, 0, 0)
-try:
-    import builtins # Python 3
-except ImportError:
-    import __builtin__ as builtins # Python 2.
 import os
+import sys
 import sqlite3
 from sqlite3 import ProgrammingError
 import traceback
-import types
+from typing import List
+
 #@-<< imports >>
 consoleEncoding = None
 #@+<< define usage >>
@@ -169,7 +164,7 @@ def cmd_functions(args):
         funcs = cw.get_functions()
     lines = list(set(el[0] + "\t" + el[1] for el in funcs))
     lines.sort()
-    return lines # EKR
+    return lines  # EKR
 #@+node:ekr.20110310091639.14285: *4* cmd_init
 def cmd_init(args):
     print("Initializing CodeWise db at: %s" % DEFAULT_DB)
@@ -183,9 +178,9 @@ def cmd_members(args):
         mems = cw.get_members([args[0]])
         lines = list(set(el + "\t" + pat for el, pat in mems))
     else:
-        lines = cw.classcache.keys()
+        lines = cw.classcache.keys()  # type:ignore
     lines.sort()
-    return lines # EKR
+    return lines  # EKR
 #@+node:ekr.20110310091639.14283: *4* cmd_parse
 def cmd_parse(args):
     assert args
@@ -204,7 +199,7 @@ def cmd_scintilla(args):
         f.close()
 #@+node:ekr.20110310091639.14286: *4* cmd_setup
 def cmd_setup(args):
-    
+
     ctagsfile = os.path.normpath(os.path.expanduser("~/.ctags"))
     if os.path.isfile(ctagsfile):
         print("Using template file: %s" % ctagsfile)
@@ -239,27 +234,24 @@ def callers(n=4, count=0, excludeCaller=True, files=False):
         if not s or len(result) >= n: break
         i += 1
     result.reverse()
-    if count > 0: result = result[: count]
+    if count > 0: result = result[:count]
     sep = '\n' if files else ','
     return sep.join(result)
 #@+node:ekr.20110310093050.14297: *6* _callerName
 def _callerName(n=1, files=False):
-    try: # get the function name from the call stack.
-        f1 = sys._getframe(n) # The stack frame, n levels up.
-        code1 = f1.f_code # The code object
+    try:  # get the function name from the call stack.
+        f1 = sys._getframe(n)  # The stack frame, n levels up.
+        code1 = f1.f_code  # The code object
         name = code1.co_name
         if name == '__init__':
             name = '__init__(%s,line %s)' % (
                 shortFileName(code1.co_filename), code1.co_firstlineno)
-        if files:
-            return '%s:%s' % (shortFileName(code1.co_filename), name)
-        else:
-            return name # The code name
+        return '%s:%s' % (shortFileName(code1.co_filename), name) if files else name
     except ValueError:
-        return '' # The stack is not deep enough.
+        return ''  # The stack is not deep enough.
     except Exception:
         es_exception()
-        return '' # "<no caller name>"
+        return ''  # "<no caller name>"
 #@+node:ekr.20110310093050.14253: *5* doKeywordArgs (codewise)
 def doKeywordArgs(keys, d=None):
     '''Return a result dict that is a copy of the keys dict
@@ -299,28 +291,29 @@ def getLastTracebackFileAndLineNumber():
     if typ == SyntaxError:
         # IndentationError is a subclass of SyntaxError.
         # Much easier in Python 2.6 and 3.x.
-        return val.filename, val.lineno
-    else:
-        # Data is a list of tuples, one per stack entry.
-        # Tupls have the form (filename,lineNumber,functionName,text).
-        data = traceback.extract_tb(tb)
-        if data:
-            item = data[-1] # Get the item at the top of the stack.
-            filename, n, functionName, text = item
-            return filename, n
-        else:
-            # Should never happen.
-            return '<string>', 0
+        return val.filename, val.lineno  # type:ignore
+    #
+    # Data is a list of tuples, one per stack entry.
+    # Tupls have the form (filename,lineNumber,functionName,text).
+    data = traceback.extract_tb(tb)
+    if data:
+        item = data[-1]  # Get the item at the top of the stack.
+        filename, n, functionName, text = item
+        return filename, n
+    #
+    # Should never happen.
+    return '<string>', 0
 #@+node:ekr.20110310093050.14293: *5* pdb (codewise)
 def pdb(message=''):
     """Fall into pdb."""
-    import pdb # Required: we have just defined pdb as a function!
+    import pdb  # Required: we have just defined pdb as a function!
     if message:
         print(message)
     pdb.set_trace()
 #@+node:ekr.20110310093050.14263: *5* pr (codewise)
 # see: http://www.diveintopython.org/xml_processing/unicode.html
-def pr(*args, **keys): # (codewise!)
+
+def pr(*args, **keys):  # (codewise!)
     '''Print all non-keyword args, and put them to the log pane.
     The first, third, fifth, etc. arg translated by translateString.
     Supports color, comma, newline, spaces and tabName keyword arguments.
@@ -334,26 +327,23 @@ def pr(*args, **keys): # (codewise!)
         encoding = sys.stdout.encoding
     else:
         encoding = 'utf-8'
+    # Translates everything to unicode.
     s = translateArgs(args, d)
-        # Translates everything to unicode.
-    func = toUnicode if isPython3 else toEncodedString
-    s = func(s, encoding=encoding, reportErrors=False)
+    s = toUnicode(s, encoding=encoding, reportErrors=False)
     if newline:
-        s += u('\n') if isPython3 else '\n'
+        s += u('\n')
     # Python's print statement *can* handle unicode, but
     # sitecustomize.py must have sys.setdefaultencoding('utf-8')
-    sys.stdout.write(s)
-        # Codewise: unit tests do not change sys.stdout.
+    sys.stdout.write(s)  # Unit tests do not change sys.stdout.
 #@+node:ekr.20180311193230.1: *5* shortFileName (codewise)
 def shortFileName(fileName, n=None):
     '''Return the base name of a path.'''
     # pylint: disable=invalid-unary-operand-type
     if not fileName:
         return ''
-    elif n is None or n < 1:
+    if n is None or n < 1:
         return os.path.basename(fileName)
-    else:
-        return '/'.join(fileName.replace('\\', '/').split('/')[-n:])
+    return '/'.join(fileName.replace('\\', '/').split('/')[-n :])
 #@+node:ekr.20110310093050.14268: *5* trace (codewise)
 # Convert all args to strings.
 
@@ -365,10 +355,10 @@ def trace(*args, **keys):
     align = d.get('align')
     if align is None: align = 0
     # Compute the caller name.
-    try: # get the function name from the call stack.
-        f1 = sys._getframe(1) # The stack frame, one level up.
-        code1 = f1.f_code # The code object
-        name = code1.co_name # The code name
+    try:  # get the function name from the call stack.
+        f1 = sys._getframe(1)  # The stack frame, one level up.
+        code1 = f1.f_code  # The code object
+        name = code1.co_name  # The code name
     except Exception:
         name = ''
     if name == "?":
@@ -401,7 +391,9 @@ def translateArgs(args, d):
     if not consoleEncoding:
         e = sys.getdefaultencoding()
         consoleEncoding = e if isValidEncoding(e) else 'utf-8'
-    result = []; n = 0; spaces = d.get('spaces')
+    result: List[str] = []
+    n = 0
+    spaces = d.get('spaces')
     for arg in args:
         n += 1
         # print('translateArgs: arg',arg,type(arg),isString(arg),'will trans',(n%2)==1)
@@ -423,32 +415,19 @@ def translateArgs(args, d):
 
 def isBytes(s):
     '''Return True if s is Python3k bytes type.'''
-    if isPython3:
-        return isinstance(s, bytes)
-    else:
-        return False
+    return isinstance(s, bytes)
 
 def isCallable(obj):
-    if isPython3:
-        return hasattr(obj, '__call__')
-    else:
-        return callable(obj)
+    return hasattr(obj, '__call__')
 
 def isString(s):
     '''Return True if s is any string, but not bytes.'''
-    # pylint: disable=no-member
-    if isPython3:
-        return type(s) == type('a') # NOQA
-    else:
-        return type(s) in types.StringTypes
+    return isinstance(s, str)
 
 def isUnicode(s):
     '''Return True if s is a unicode string.'''
-    # pylint: disable=no-member
-    if isPython3:
-        return type(s) == type('a') # NOQA
-    else:
-        return type(s) == types.UnicodeType # NOQA
+    return isinstance(s, str)
+
 #@+node:ekr.20110310093050.14283: *5* isValidEncoding (codewise)
 def isValidEncoding(encoding):
     if not encoding:
@@ -459,9 +438,9 @@ def isValidEncoding(encoding):
     try:
         codecs.lookup(encoding)
         return True
-    except LookupError: # Windows.
+    except LookupError:  # Windows.
         return False
-    except AttributeError: # Linux.
+    except AttributeError:  # Linux.
         return False
 #@+node:ekr.20110310093050.14286: *5* toEncodedString (codewise)
 def toEncodedString(s, encoding='utf-8', reportErrors=False):
@@ -492,21 +471,11 @@ def toUnicode(s, encoding='utf-8', reportErrors=False):
             error("Error converting %s from %s encoding to unicode" % (s, encoding))
     return s
 #@+node:ekr.20110310093050.14288: *5* u & ue (codewise)
-if isPython3:
+def u(s):
+    return s
 
-    def u(s):
-        return s
-
-    def ue(s, encoding):
-        return s if isUnicode(s) else str(s, encoding)
-
-else:
-
-    def u(s):
-        return builtins.unicode(s)
-
-    def ue(s, encoding):
-        return builtins.unicode(s, encoding)
+def ue(s, encoding):
+    return s if isUnicode(s) else str(s, encoding)
 #@+node:ekr.20110310091639.14290: *3* main
 def main():
 
@@ -537,7 +506,7 @@ def printlines(lines):
     for l in lines:
         try:
             print(l)
-        except Exception: # EKR: UnicodeEncodeError:
+        except Exception:  # EKR: UnicodeEncodeError:
             pass
 #@+node:ekr.20110310091639.14280: *3* run_ctags
 def run_ctags(paths):
@@ -549,7 +518,7 @@ def run_ctags(paths):
 def test(self):
     pass
 #@+node:ekr.20110310091639.14256: ** class CodeWise
-class CodeWise(object):
+class CodeWise:
     #@+others
     #@+node:ekr.20110310091639.14257: *3* __init__(CodeWise)
     def __init__(self, dbpath=None):
@@ -594,6 +563,7 @@ class CodeWise(object):
                 return self.dbconn.cursor()
             except ProgrammingError:
                 print("No cursor for codewise DB, closed database?")
+        return None
     #@+node:ekr.20110310091639.14262: *3* class_id
     def class_id(self, classname):
         """ return class id. May create new class """
@@ -629,7 +599,8 @@ class CodeWise(object):
             c.execute('select name, class, file, searchpattern from function')
         else:
             prefix = str(prefix)
-            c.execute('select name, class, file, searchpattern from function where name like (?)', (
+            c.execute(
+                'select name, class, file, searchpattern from function where name like (?)', (
                 prefix + '%',))
         return [(name, pat, klassid, fileid) for name, klassid, fileid, pat in c]
     #@+node:ekr.20110310091639.14265: *3* file_id
@@ -664,7 +635,8 @@ class CodeWise(object):
         clid = self.class_id(class_name)
         fid = self.file_id(file_name)
         c = self.cursor()
-        c.execute('insert into function(class, name, searchpattern, file) values (?, ?, ?, ?)',
+        c.execute(
+            'insert into function(class, name, searchpattern, file) values (?, ?, ?, ?)',
                   [clid, func_name, aux, fid])
     #@+node:ekr.20110310091639.14267: *3* feed_scintilla
     def feed_scintilla(self, apifile_obj):
@@ -675,8 +647,6 @@ class CodeWise(object):
         qt.QApplication.style?4() -> QStyle
         """
         for l in apifile_obj:
-            if not isPython3:
-                l = builtins.unicode(l, 'utf8', 'replace')
             parts = l.split('?')
             fullsym = parts[0].rsplit('.', 1)
             klass, func = fullsym
@@ -693,9 +663,6 @@ class CodeWise(object):
     #@+node:ekr.20110310091639.14268: *3* feed_ctags
     def feed_ctags(self, tagsfile_obj):
         for l in tagsfile_obj:
-            #print l
-            if not isPython3:
-                l = builtins.unicode(l, 'utf8', 'replace')
             if l.startswith('!'):
                 continue
             fields = l.split('\t')
@@ -717,7 +684,8 @@ class CodeWise(object):
             c = self.cursor()
             #print fields
             fid = self.file_id(fil)
-            c.execute('insert into function(class, name, searchpattern, file) values (?, ?, ?, ?)',
+            c.execute(
+                'insert into function(class, name, searchpattern, file) values (?, ?, ?, ?)',
                       [idd, m, pat, fid])
         self.dbconn.commit()
         #c.commit()
@@ -760,7 +728,7 @@ class CodeWise(object):
                 self.add_source('tagdir', a)
     #@-others
 #@+node:ekr.20110310091639.14275: ** class ContextSniffer
-class ContextSniffer(object):
+class ContextSniffer:
     """ Class to analyze surrounding context and guess class
 
     For simple dynamic code completion engines
@@ -798,6 +766,7 @@ class ContextSniffer(object):
 #@@language python
 #@@tabwidth -4
 #@@pagewidth 70
+
 if __name__ == "__main__":
     main()
 #@-leo
