@@ -2,18 +2,19 @@
 #@+node:ekr.20170428084208.68: * @file ../external/npyscreen/wgmultiline.py
 #!/usr/bin/python
 # pylint: disable=no-member
-import leo.core.leoGlobals as g
-assert g
-import copy
-from . import wgwidget       as widget
-from . import wgtextbox      as textbox
-import textwrap
-import curses
-from . import wgtitlefield   as titlefield
-from . import fmPopup        as Popup
-import weakref
+# type: ignore
 import collections
-MORE_LABEL = "- more -" # string to tell user there are more options
+import copy
+import curses
+import textwrap
+import weakref
+from . import wgwidget as widget
+from . import wgtextbox as textbox
+from . import wgtitlefield as titlefield
+from . import fmPopup as Popup
+from leo.core import leoGlobals as g
+assert g
+MORE_LABEL = "- more -"  # string to tell user there are more options
 #@+others
 #@+node:ekr.20170428084208.69: ** Declarations
 #@+node:ekr.20170428084208.70: ** class FilterPopupHelper
@@ -22,13 +23,13 @@ class FilterPopupHelper(Popup.Popup):
     #@+node:ekr.20170428084208.71: *3* create
     def create(self):
         super(FilterPopupHelper, self).create()
-        self.filterbox = self.add(titlefield.TitleText, name='Find:', )
+        self.filterbox = self.add(titlefield.TitleText, name='Find:',)
         self.nextrely += 1
-        self.statusline = self.add(textbox.Textfield, color = 'LABEL', editable = False)
+        self.statusline = self.add(textbox.Textfield, color='LABEL', editable=False)
 
     #@+node:ekr.20170428084208.72: *3* updatestatusline
     def updatestatusline(self):
-        self.owner_widget._filter   = self.filterbox.value
+        self.owner_widget._filter = self.filterbox.value
         filtered_lines = self.owner_widget.get_filtered_indexes()
         len_f = len(filtered_lines)
         if self.filterbox.value == None or self.filterbox.value == '':
@@ -58,7 +59,7 @@ class MultiLine(widget.Widget):
     Given the standard definition, the same effect can be achieved by
     altering the __str__() method of displayed objects
     """
-    _MINIMUM_HEIGHT = 2 # Raise an error if not given this.
+    _MINIMUM_HEIGHT = 2  # Raise an error if not given this.
     _contained_widgets = textbox.Textfield
     _contained_widget_height = 1
     #@+others
@@ -75,15 +76,14 @@ class MultiLine(widget.Widget):
         widgets_inherit_color=False,
         always_show_cursor=False,
         allow_filtering=True,
-        ** keywords
+        **keywords
     ):
         self.never_cache = False
         self.exit_left = exit_left
         self.exit_right = exit_right
         self.allow_filtering = allow_filtering
         self.widgets_inherit_color = widgets_inherit_color
-        super(MultiLine, self).__init__(screen, **keywords)
-            # Call the base class.
+        super(MultiLine, self).__init__(screen, **keywords)  # Call the base class.
         if self.height < self.__class__._MINIMUM_HEIGHT:
             raise widget.NotEnoughSpaceForWidget(
                 "Height of %s allocated. Not enough space allowed for %s" % (
@@ -127,15 +127,15 @@ class MultiLine(widget.Widget):
         trace_widgets = True
         self._my_widgets = []
         height = self.height // self.__class__._contained_widget_height
-        if trace: g.trace(self.__class__.__name__, height) #, g.callers(2))
+        if trace: g.trace(self.__class__.__name__, height)  #, g.callers(2))
             # Called from BoxTitle.make_contained_widget.
         for h in range(height):
             # EKR: it's LeoMLTree._contained_widgets that we have to emulate.
             self._my_widgets.append(
                 self._contained_widgets(
                     self.parent,
-                    rely=(h*self._contained_widget_height)+self.rely,
-                    relx = self.relx,
+                    rely=(h * self._contained_widget_height) + self.rely,
+                    relx=self.relx,
                     max_width=self.width,
                     max_height=self.__class__._contained_widget_height
             ))
@@ -157,18 +157,18 @@ class MultiLine(widget.Widget):
             return "**** Error ****"
     #@+node:ekr.20170428084208.79: *3* MultiLine.calculate_area_needed
     def calculate_area_needed(self):
-        return 0,0
+        return 0, 0
     #@+node:ekr.20170428084208.80: *3* MultiLine.reset_cursor
     def reset_cursor(self):
         self.start_display_at = 0
-        self.cursor_line      = 0
+        self.cursor_line = 0
     #@+node:ekr.20170428084208.81: *3* MultiLine.reset_display_cache
     def reset_display_cache(self):
         self._last_values = False
-        self._last_value  = False
+        self._last_value = False
     #@+node:ekr.20170428084208.82: *3* MultiLine.update (LeoMLTree overrides this)
     def update(self, clear=True):
-        trace = False # LeoMLTree.update overrides this.
+        trace = False  # LeoMLTree.update overrides this.
         if trace and self.hidden:
             g.trace('hidden')
         if self.hidden and clear:
@@ -237,7 +237,7 @@ class MultiLine(widget.Widget):
             self._last_start_display_at = self.start_display_at
             self._before_print_lines()
             indexer = 0 + self.start_display_at
-            for line in self._my_widgets[: -1]:
+            for line in self._my_widgets[:-1]:
                 self._print_line(line, indexer)
                 line.task = "PRINTLINE"
                 line.update(clear=True)
@@ -305,7 +305,7 @@ class MultiLine(widget.Widget):
     #@+node:ekr.20170428084208.84: *3* MultiLine._print_line
     def _print_line(self, line, value_indexer):
 
-        trace = False # LeoMLTree.update overrides this.
+        trace = False  # LeoMLTree.update overrides this.
         if self.widgets_inherit_color and self.do_colors():
             line.color = self.color
         self._set_line_values(line, value_indexer)
@@ -328,10 +328,10 @@ class MultiLine(widget.Widget):
         line.hidden = False
     #@+node:ekr.20170428084208.86: *4* MultiLine._set_line_blank
     def _set_line_blank(self, line):
-        line.value    = None
-        line.show_bold= False
-        line.name     = None
-        line.hidden   = True
+        line.value = None
+        line.show_bold = False
+        line.name = None
+        line.hidden = True
     #@+node:ekr.20170428084208.87: *4* MultiLine._set_line_highlighting
     def _set_line_highlighting(self, line, value_indexer):
         if value_indexer in self._filtered_values_cache:
@@ -352,6 +352,7 @@ class MultiLine(widget.Widget):
         line.show_bold = value
     #@+node:ekr.20170428084208.90: *4* MultiLine.set_is_line_cursor
     def set_is_line_cursor(self, line, value):
+
         # g.trace('Multiline')
         line.highlight = value
     #@+node:ekr.20170504211232.1: *3* MultiLine.filters
@@ -390,7 +391,7 @@ class MultiLine(widget.Widget):
         else:
             return False
     #@+node:ekr.20170428084208.95: *4* MultiLine.jump_to_first_filtered
-    def jump_to_first_filtered(self, ):
+    def jump_to_first_filtered(self,):
         self.h_cursor_beginning(None)
         self.move_next_filtered(include_this_line=True)
     #@+node:ekr.20170428084208.96: *4* MultiLine.clear_filter
@@ -403,7 +404,7 @@ class MultiLine(widget.Widget):
         if self._filter == None:
             return False
         for possible in self._filtered_values_cache:
-            if (possible==self.cursor_line and include_this_line==True):
+            if (possible == self.cursor_line and include_this_line == True):
                 self.update()
                 break
             elif possible > self.cursor_line:
@@ -411,8 +412,8 @@ class MultiLine(widget.Widget):
                 self.update()
                 break
         try:
-            if self.cursor_line-self.start_display_at > len(self._my_widgets) or \
-            self._my_widgets[self.cursor_line-self.start_display_at].task == MORE_LABEL:
+            if self.cursor_line - self.start_display_at > len(self._my_widgets) or \
+            self._my_widgets[self.cursor_line - self.start_display_at].task == MORE_LABEL:
                 if self.slow_scroll:
                     self.start_display_at += 1
                 else:
@@ -452,43 +453,43 @@ class MultiLine(widget.Widget):
     def set_up_handlers(self):
         '''MultiLine.set_up_handlers.'''
         super(MultiLine, self).set_up_handlers()
-        self.handlers.update ({
-            curses.KEY_UP:      self.h_cursor_line_up,
-            ord('k'):           self.h_cursor_line_up,
-            curses.KEY_LEFT:    self.h_cursor_line_up,
-            curses.KEY_DOWN:    self.h_cursor_line_down,
-            ord('j'):           self.h_cursor_line_down,
-            curses.KEY_RIGHT:   self.h_cursor_line_down,
-            curses.KEY_NPAGE:   self.h_cursor_page_down,
-            curses.KEY_PPAGE:   self.h_cursor_page_up,
-            curses.ascii.TAB:   self.h_exit_down,
-            curses.ascii.NL:    self.h_select_exit,
-            curses.KEY_HOME:    self.h_cursor_beginning,
-            curses.KEY_END:     self.h_cursor_end,
-            ord('g'):           self.h_cursor_beginning,
-            ord('G'):           self.h_cursor_end,
-            ord('x'):           self.h_select,
+        self.handlers.update({
+            curses.KEY_UP: self.h_cursor_line_up,
+            ord('k'): self.h_cursor_line_up,
+            curses.KEY_LEFT: self.h_cursor_line_up,
+            curses.KEY_DOWN: self.h_cursor_line_down,
+            ord('j'): self.h_cursor_line_down,
+            curses.KEY_RIGHT: self.h_cursor_line_down,
+            curses.KEY_NPAGE: self.h_cursor_page_down,
+            curses.KEY_PPAGE: self.h_cursor_page_up,
+            curses.ascii.TAB: self.h_exit_down,
+            curses.ascii.NL: self.h_select_exit,
+            curses.KEY_HOME: self.h_cursor_beginning,
+            curses.KEY_END: self.h_cursor_end,
+            ord('g'): self.h_cursor_beginning,
+            ord('G'): self.h_cursor_end,
+            ord('x'): self.h_select,
             # "^L":        self.h_set_filtered_to_selected,
-            curses.ascii.SP:    self.h_select,
-            curses.ascii.ESC:   self.h_exit_escape,
-            curses.ascii.CR:    self.h_select_exit,
+            curses.ascii.SP: self.h_select,
+            curses.ascii.ESC: self.h_exit_escape,
+            curses.ascii.CR: self.h_select_exit,
         })
         if self.allow_filtering:
-            self.handlers.update ( {
-                ord('l'):       self.h_set_filter,
-                ord('L'):       self.h_clear_filter,
-                ord('n'):       self.move_next_filtered,
-                ord('N'):       self.move_previous_filtered,
-                ord('p'):       self.move_previous_filtered,
+            self.handlers.update({
+                ord('l'): self.h_set_filter,
+                ord('L'): self.h_clear_filter,
+                ord('n'): self.move_next_filtered,
+                ord('N'): self.move_previous_filtered,
+                ord('p'): self.move_previous_filtered,
                 # "^L":        self.h_set_filtered_to_selected,
-            } )
+            })
         if self.exit_left:
             self.handlers.update({
-                curses.KEY_LEFT:    self.h_exit_left
+                curses.KEY_LEFT: self.h_exit_left
             })
         if self.exit_right:
             self.handlers.update({
-                curses.KEY_RIGHT:   self.h_exit_right
+                curses.KEY_RIGHT: self.h_exit_right
             })
         self.complex_handlers = [
             # (self.t_input_isprint, self.h_find_char)
@@ -501,7 +502,7 @@ class MultiLine(widget.Widget):
         searchingfor = chr(input).upper()
         for counter in range(len(self.values)):
             try:
-                if self.values[counter].find(searchingfor) is not -1:
+                if self.values[counter].find(searchingfor) != -1:
                     self.cursor_line = counter
                     break
             except AttributeError:
@@ -531,25 +532,25 @@ class MultiLine(widget.Widget):
         self.cursor_line = 0
     #@+node:ekr.20170428084208.107: *4* MultiLine.h_cursor_end
     def h_cursor_end(self, ch):
-        self.cursor_line= len(self.values)-1
+        self.cursor_line = len(self.values) - 1
         if self.cursor_line < 0:
             self.cursor_line = 0
     #@+node:ekr.20170428084208.108: *4* MultiLine.h_cursor_page_down
     def h_cursor_page_down(self, ch):
-        self.cursor_line += (len(self._my_widgets)-1) # -1 because of the -more-
-        if self.cursor_line >= len(self.values)-1:
-            self.cursor_line = len(self.values) -1
-        if not (self.start_display_at + len(self._my_widgets) -1 ) > len(self.values):
-            self.start_display_at += (len(self._my_widgets)-1)
-            if self.start_display_at > len(self.values) - (len(self._my_widgets)-1):
-                self.start_display_at = len(self.values) - (len(self._my_widgets)-1)
+        self.cursor_line += (len(self._my_widgets) - 1)  # -1 because of the -more-
+        if self.cursor_line >= len(self.values) - 1:
+            self.cursor_line = len(self.values) - 1
+        if not (self.start_display_at + len(self._my_widgets) - 1) > len(self.values):
+            self.start_display_at += (len(self._my_widgets) - 1)
+            if self.start_display_at > len(self.values) - (len(self._my_widgets) - 1):
+                self.start_display_at = len(self.values) - (len(self._my_widgets) - 1)
 
     #@+node:ekr.20170428084208.109: *4* MultiLine.h_cursor_page_up
     def h_cursor_page_up(self, ch):
-        self.cursor_line -= (len(self._my_widgets)-1)
+        self.cursor_line -= (len(self._my_widgets) - 1)
         if self.cursor_line < 0:
             self.cursor_line = 0
-        self.start_display_at -= (len(self._my_widgets)-1)
+        self.start_display_at -= (len(self._my_widgets) - 1)
         if self.start_display_at < 0: self.start_display_at = 0
     #@+node:ekr.20170428084208.110: *4* MultiLine.h_cursor_line_up
     def h_cursor_line_up(self, ch):
@@ -566,13 +567,13 @@ class MultiLine(widget.Widget):
         self.cursor_line += 1
         if self.cursor_line >= len(self.values):
             if self.scroll_exit:
-                self.cursor_line = len(self.values)-1
+                self.cursor_line = len(self.values) - 1
                 self.h_exit_down(ch)
                 return True
             else:
-                self.cursor_line -=1
+                self.cursor_line -= 1
                 return True
-        if self._my_widgets[self.cursor_line-self.start_display_at].task == MORE_LABEL:
+        if self._my_widgets[self.cursor_line - self.start_display_at].task == MORE_LABEL:
             if self.slow_scroll:
                 self.start_display_at += 1
             else:
@@ -607,13 +608,12 @@ class MultiLine(widget.Widget):
         self.h_select(ch)
         if self.return_exit or self.select_exit:
             self.editing = False
-            self.how_exited=True
+            self.how_exited = True
     #@+node:ekr.20170428084208.116: *4* MultiLine.edit
     def edit(self):
         trace = False and not g.unitTesting
         if trace:
             g.trace('===== (MultiLine:%s)' % self.__class__.__name__)
-            # g.trace('CALLERS', g.callers(verbose=True))
         self.editing = True
         self.how_exited = None
         #if self.value: self.cursor_line = self.value
@@ -657,11 +657,11 @@ class MultiLineAction(MultiLine):
     def set_up_handlers(self):
         '''MultiLineAction.set_up_handlers.'''
         super(MultiLineAction, self).set_up_handlers()
-        self.handlers.update ( {
-            curses.ascii.NL:    self.h_act_on_highlighted,
-            curses.ascii.CR:    self.h_act_on_highlighted,
-            ord('x'):           self.h_act_on_highlighted,
-            curses.ascii.SP:    self.h_act_on_highlighted,
+        self.handlers.update({
+            curses.ascii.NL: self.h_act_on_highlighted,
+            curses.ascii.CR: self.h_act_on_highlighted,
+            ord('x'): self.h_act_on_highlighted,
+            curses.ascii.SP: self.h_act_on_highlighted,
         })
 
 
@@ -674,7 +674,7 @@ class MultiLineActionWithShortcuts(MultiLineAction):
     def set_up_handlers(self):
         '''MultiLineActionWithShortcuts.set_up_handlers.'''
         super(MultiLineActionWithShortcuts, self).set_up_handlers()
-        self.add_complex_handlers( ((self.h_find_shortcut_action, self.h_execute_shortcut_action),) )
+        self.add_complex_handlers(((self.h_find_shortcut_action, self.h_execute_shortcut_action),))
 
 
     #@+node:ekr.20170428084208.124: *3* MultiLineActionWithShortcuts.h_find_shortcut_action
@@ -705,7 +705,7 @@ class MultiLineActionWithShortcuts(MultiLineAction):
 class Pager(MultiLine):
     #@+others
     #@+node:ekr.20170428084208.127: *3* Pager.__init__
-    def __init__(self, screen, autowrap=False,  center=False, **keywords):
+    def __init__(self, screen, autowrap=False, center=False, **keywords):
         super(Pager, self).__init__(screen, **keywords)
         self.autowrap = autowrap
         self.center = center
@@ -746,12 +746,12 @@ class Pager(MultiLine):
             lines = lines.split('\n')
         except AttributeError:
             pass
-        self.values = self._wrap_message_lines(lines, self.width-1)
+        self.values = self._wrap_message_lines(lines, self.width - 1)
         self._values_cache_for_wrapping = self.values
 
     #@+node:ekr.20170428084208.132: *3* Pager.centerValues
     def centerValues(self):
-        self.values  = [ l.strip().center(self.width-1) for l in self.values ]
+        self.values = [l.strip().center(self.width - 1) for l in self.values]
 
     #@+node:ekr.20170428084208.133: *3* Pager.update
     def update(self, clear=True):
@@ -777,7 +777,7 @@ class Pager(MultiLine):
         # Now do the final line
         line = self._my_widgets[-1]
 
-        if values_len <= indexer+1:
+        if values_len <= indexer + 1:
             self._print_line(line, indexer)
         else:
             line.value = MORE_LABEL
@@ -810,7 +810,7 @@ class Pager(MultiLine):
     #@+node:ekr.20170428084208.136: *3* Pager.h_scroll_line_down
     def h_scroll_line_down(self, input):
         self.start_display_at += 1
-        if self.scroll_exit and self.start_display_at >= len(self.values)-self.start_display_at+1:
+        if self.scroll_exit and self.start_display_at >= len(self.values) - self.start_display_at + 1:
             self.editing = False
             self.how_exited = widget.EXITED_DOWN
 
@@ -839,25 +839,25 @@ class Pager(MultiLine):
         '''Pager.set_up_handlers.'''
         super(Pager, self).set_up_handlers()
         self.handlers = {
-            curses.KEY_UP:      self.h_scroll_line_up,
-            curses.KEY_LEFT:    self.h_scroll_line_up,
-            curses.KEY_DOWN:    self.h_scroll_line_down,
-            curses.KEY_RIGHT:   self.h_scroll_line_down,
-            curses.KEY_NPAGE:   self.h_scroll_page_down,
-            curses.KEY_PPAGE:   self.h_scroll_page_up,
-            curses.KEY_HOME:    self.h_show_beginning,
-            curses.KEY_END:     self.h_show_end,
-            curses.ascii.NL:    self.h_exit,
-            curses.ascii.CR:    self.h_exit,
-            curses.ascii.SP:    self.h_scroll_page_down,
-            curses.ascii.TAB:   self.h_exit,
-            ord('j'):           self.h_scroll_line_down,
-            ord('k'):           self.h_scroll_line_up,
-            ord('x'):           self.h_exit,
-            ord('q'):           self.h_exit,
-            ord('g'):           self.h_show_beginning,
-            ord('G'):           self.h_show_end,
-            curses.ascii.ESC:   self.h_exit_escape,
+            curses.KEY_UP: self.h_scroll_line_up,
+            curses.KEY_LEFT: self.h_scroll_line_up,
+            curses.KEY_DOWN: self.h_scroll_line_down,
+            curses.KEY_RIGHT: self.h_scroll_line_down,
+            curses.KEY_NPAGE: self.h_scroll_page_down,
+            curses.KEY_PPAGE: self.h_scroll_page_up,
+            curses.KEY_HOME: self.h_show_beginning,
+            curses.KEY_END: self.h_show_end,
+            curses.ascii.NL: self.h_exit,
+            curses.ascii.CR: self.h_exit,
+            curses.ascii.SP: self.h_scroll_page_down,
+            curses.ascii.TAB: self.h_exit,
+            ord('j'): self.h_scroll_line_down,
+            ord('k'): self.h_scroll_line_up,
+            ord('x'): self.h_exit,
+            ord('q'): self.h_exit,
+            ord('g'): self.h_show_beginning,
+            ord('G'): self.h_show_end,
+            curses.ascii.ESC: self.h_exit_escape,
         }
         self.complex_handlers = []
 
@@ -923,7 +923,7 @@ class BufferPager(Pager):
             pass
 
         self.clearBuffer()
-        self.buffer(self._wrap_message_lines(lines, self.width-1))
+        self.buffer(self._wrap_message_lines(lines, self.width - 1))
         self._values_cache_for_wrapping = copy.deepcopy(self.values)
 
     #@+node:ekr.20170428084208.153: *3* BufferPager.buffer
