@@ -4,7 +4,6 @@
 #@@first
 """The new, tokenize based, @auto importer for Python."""
 import re
-import sys
 import tokenize
 import token
 from typing import Any, Callable, Dict, Generator, List, Optional, Tuple
@@ -210,10 +209,6 @@ def token_based_python_importer(c: Cmdr, root: Any, s: str) -> None:
     lines: List[str] = s.splitlines(True)
     rawtokens: List
 
-    if sys.version_info < (3, 7, 0):  # pragma: no cover
-        g.es_print('The python importer requires python 3.7 or above')
-        return
-
     #@+others
     #@+node:vitalije.20211208092910.1: *3* function: getdefn & helper
     def getdefn(start: int) -> def_tuple:
@@ -255,7 +250,9 @@ def token_based_python_importer(c: Cmdr, root: Any, s: str) -> None:
         # Look ahead to see if we have a one-line definition (INDENT comes after the NEWLINE).
         i1, t = find_token(i + 1, token.INDENT)  # t is used below.
         i2, t2 = find_token(i + 1, token.NEWLINE)
-        oneliner = i1 > i2 if t and t2 else False
+        if t is None or t2 is None:
+            return None
+        oneliner = i1 > i2
 
         # Find the end of this definition
         if oneliner:
