@@ -63,6 +63,7 @@ import pathlib
 import sys
 import random
 import textwrap
+from typing import List
 # Leo imports
 from leo.core import leoGlobals as g
 try:
@@ -203,10 +204,14 @@ def main():
 #@+node:ekr.20211021202356.1: ** class Slides
 if QtWidgets:
 
-    class Slides(QtWidgets.QWidget):
+    class Slides(QtWidgets.QWidget):  # type:ignore
 
+        files_list: List[str]
+        scale: float = 1.0
         slide_number = -1
+        starting_directory: str = None
         timer = QtCore.QBasicTimer()
+        verbose: bool = False
 
         #@+others
         #@+node:ekr.20211024030844.1: *3* Slides.closeEvent
@@ -252,9 +257,13 @@ if QtWidgets:
                 send2trash(file_name)
                 del self.files_list[self.slide_number]
                 print(f"Deleted {file_name}")
-                self.slide_number = max(0, self.slide_number - 1)
-                self.next_slide()
-                self.raise_()
+                if self.files_list:
+                    self.slide_number = max(0, self.slide_number - 1)
+                    self.next_slide()
+                    self.raise_()
+                else:
+                    print('No more slides')
+                    self.quit()
         #@+node:ekr.20211021200821.2: *3* Slides.get_files
         def get_files(self, path):
             """Return all files in path, including all subdirectories."""
@@ -268,7 +277,10 @@ if QtWidgets:
 
             i = event.key()
             s = event.text()
-            # mods = event.modifiers()
+            mods = event.modifiers()
+            if mods and 'ShiftModifier' not in repr(mods):
+                print(f"picture_viewer.py: ignoring modified key: {s!r} {i}")
+                return
             if s == 'd':
                 self.delete()
             elif s == 'f':
@@ -316,12 +328,12 @@ if QtWidgets:
         def move_to(self):
             """Issue a prompt and move the file if the user agrees."""
             file_name = self.files_list[self.slide_number]
-            path = QtWidgets.QFileDialog().getExistingDirectory()
+            path: str = QtWidgets.QFileDialog().getExistingDirectory()
             if path:
                 new_path = os.path.join(path, os.path.basename(file_name))
                 if os.path.exists(new_path):
                     print("File exists:", new_path)
-                    pathlib.Path(file_name).unlink(new_path)
+                    pathlib.Path(file_name).unlink(new_path)  # type:ignore
                 else:
                     pathlib.Path(file_name).rename(new_path)
                 del self.files_list[self.slide_number]
@@ -331,10 +343,11 @@ if QtWidgets:
         #@+node:ekr.20211021200821.8: *3* Slides.next_slide
         def next_slide(self):
 
-            if not self.files_list:
-                self.quit()
             if self.slide_number + 1 < len(self.files_list):
                 self.slide_number += 1  # Don't wrap.
+            else:
+                print('No more slides')
+                self.quit()
             if self.reset_zoom:
                 self.scale = 1.0
             self.show_slide()
@@ -504,14 +517,25 @@ if QtWidgets:
             pixmap = QtGui.QPixmap(file_name)
             try:
                 TransformationMode = QtCore.Qt if isQt5 else QtCore.Qt.TransformationMode
-                image = pixmap.scaledToHeight(
-                    int(self.height() * self.scale),
-                    TransformationMode.SmoothTransformation,  # pylint: disable=no-member
-                )
+                AspectRatioMode = QtCore.Qt if isQt5 else QtCore.Qt.AspectRatioMode
+                if 0:
+                    w = self.scroll_area.width()
+                    h = self.scroll_area.height()
+                    pixmap1 = pixmap.scaled(w, h, AspectRatioMode.KeepAspectRatio)
+                    ### Not yet.
+                    image = pixmap1.scaledToHeight(
+                        int(self.height() * self.scale),
+                        TransformationMode.SmoothTransformation,  # pylint: disable=no-member
+                    )
+                else:  # Legacy
+                    image = pixmap.scaledToHeight(
+                        int(self.height() * self.scale),
+                        TransformationMode.SmoothTransformation,  # pylint: disable=no-member
+                    )
                 self.picture.setPixmap(image)
                 self.picture.adjustSize()
             except Exception:
-                self.next_slide()
+                g.es_exception()
         #@+node:ekr.20211021200821.15: *3* Slides.sort
         def sort(self, sort_kind):
             """sort files_list based on sort_kind."""
