@@ -4,8 +4,8 @@
 #@@first
 """This file contains the gui wrapper for Qt: g.app.gui."""
 # pylint: disable=import-error
-#@+<< imports qt_gui.py >>
-#@+node:ekr.20140918102920.17891: ** << imports qt_gui.py >>
+#@+<< qt_gui imports  >>
+#@+node:ekr.20140918102920.17891: ** << qt_gui imports >>
 import datetime
 import functools
 import re
@@ -28,20 +28,22 @@ from leo.plugins import qt_text
 # This defines the commands defined by @g.command.
 from leo.plugins import qt_commands
 assert qt_commands
-#@-<< imports qt_gui.py >>
-#@+<< type aliases qt_gui.py >>
-#@+node:ekr.20220415183421.1: ** << type aliases qt_gui.py >>
-if TYPE_CHECKING:  # Always False at runtime.
+from leo.core.leoTips import UserTip
+#@-<< qt_gui imports  >>
+#@+<< qt_gui annotations >>
+#@+node:ekr.20220415183421.1: ** << qt_gui annotations >>
+if TYPE_CHECKING:  # pragma: no cover
     from leo.core.leoCommands import Commands as Cmdr
-    from leo.core.leoNodes import Position as Pos
+    from leo.core.leoGui import LeoKeyEvent as Event
+    from leo.core.leoNodes import Position
+    from leo.plugins.qt_text import QTextEditWrapper as Wrapper
 else:
     Cmdr = Any
-    Pos = Any
-Event = Any
+    Event = Any
+    Position = Any
+    Wrapper = Any
 Widget = Any
-Wrapper = Any
-
-#@-<< type aliases qt_gui.py >>
+#@-<< qt_gui annotations >>
 #@+others
 #@+node:ekr.20110605121601.18134: ** init (qt_gui.py)
 def init() -> bool:
@@ -66,13 +68,12 @@ class LeoQtGui(leoGui.LeoGui):
         super().__init__('qt')  # Initialize the base class.
         self.active = True
         self.consoleOnly = False  # Console is separate from the log.
-        self.iconimages: Dict = {}
+        self.iconimages: Dict[str, Any] = {}  # Keys are paths, values are Icons.
         self.globalFindDialog: Widget = None
-        self.idleTimeClass: Any = qt_idle_time.IdleTime
+        self.idleTimeClass = qt_idle_time.IdleTime
         self.insert_char_flag = False  # A flag for eventFilter.
         self.mGuiName = 'qt'
-        self.main_window = None  # The *singleton* QMainWindow.
-        self.plainTextWidget: Any = qt_text.PlainTextWrapper
+        self.plainTextWidget = qt_text.PlainTextWrapper
         self.show_tips_flag = False  # #2390: Can't be inited in reload_settings.
         self.styleSheetManagerClass = StyleSheetManager
         # Be aware of the systems native colors, fonts, etc.
@@ -276,11 +277,11 @@ class LeoQtGui(leoGui.LeoGui):
         """Create a qt find tab in the indicated frame."""
         pass  # Now done in dw.createFindTab.
 
-    def createLeoFrame(self, c: Cmdr, title: str) -> Wrapper:
+    def createLeoFrame(self, c: Cmdr, title: str) -> Widget:
         """Create a new Leo frame."""
         return qt_frame.LeoQtFrame(c, title, gui=self)
 
-    def createSpellTab(self, c: Cmdr, spellHandler: Any, tabName: str) -> Wrapper:
+    def createSpellTab(self, c: Cmdr, spellHandler: Any, tabName: str) -> Widget:
         if g.unitTesting:
             return None
         return qt_frame.LeoQtSpellTab(c, spellHandler, tabName)
@@ -308,7 +309,7 @@ class LeoQtGui(leoGui.LeoGui):
         c: Cmdr,
         title: str,
         message: str='Select Date/Time',
-        init: Any=None,
+        init: datetime.datetime=None,
         step_min: Dict=None,
     ) -> None:
         """Create and run a qt date/time selection dialog.
@@ -336,7 +337,7 @@ class LeoQtGui(leoGui.LeoGui):
             for a minimum 5 minute increment on the minute field.
             """
 
-            def __init__(self, parent: Widget=None, init: Any=None, step_min: Any=None) -> None:
+            def __init__(self, parent: Widget=None, init: bool=None, step_min: Dict=None) -> None:
                 if step_min is None:
                     step_min = {}
                 self.step_min = step_min
@@ -356,9 +357,9 @@ class LeoQtGui(leoGui.LeoGui):
 
             def __init__(
                 self,
-                parent: Wrapper=None,
+                parent: Widget=None,
                 message: str='Select Date/Time',
-                init: Any=None,
+                init: Any=None,  # Hard to annotate.
                 step_min: Dict=None,
             ) -> None:
                 if step_min is None:
@@ -615,7 +616,6 @@ class LeoQtGui(leoGui.LeoGui):
         """
         Create and run an Qt open file dialog.
         """
-        # pylint: disable=arguments-differ
         if g.unitTesting:
             return ''
 
@@ -646,7 +646,7 @@ class LeoQtGui(leoGui.LeoGui):
             if c and files:
                 c.last_dir = g.os_path_dirname(files[-1])
             # A consequence of the evil "multiple" kwarg.
-            return files  # type:ignore
+            return files
         s = g.os_path_normslashes(val)
         if c and s:
             c.last_dir = g.os_path_dirname(s)
@@ -657,7 +657,7 @@ class LeoQtGui(leoGui.LeoGui):
         title: str='Properties',
         data: Any=None,
         callback: Callable=None,
-        buttons: Any=None,
+        buttons: List[str]=None,
     ) -> Tuple[str, Dict]:
         """Dispay a modal TkPropertiesDialog"""
         if not g.unitTesting:
@@ -872,7 +872,6 @@ class LeoQtGui(leoGui.LeoGui):
         Check to see if c.frame is in a tabbed ui, and if so, make sure
         the tab is visible
         """
-        # pylint: disable=arguments-differ
         if 'focus' in g.app.debug:
             g.trace(c1)
         if hasattr(g.app.gui, 'frameFactory'):
@@ -884,7 +883,6 @@ class LeoQtGui(leoGui.LeoGui):
     #@+node:ekr.20190601054958.1: *4* qt_gui.get_focus (no longer used)
     def get_focus(self, c: Cmdr=None, raw: bool=False, at_idle: bool=False) -> Widget:
         """Returns the widget that has focus."""
-        # pylint: disable=arguments-differ
         trace = 'focus' in g.app.debug
         trace_idle = False
         trace = trace and (trace_idle or not at_idle)
@@ -906,7 +904,6 @@ class LeoQtGui(leoGui.LeoGui):
     #@+node:ekr.20190601054959.1: *4* qt_gui.set_focus
     def set_focus(self, c: Cmdr, w: Wrapper) -> None:
         """Put the focus on the widget."""
-        # pylint: disable=arguments-differ
         if not w:
             return
         if getattr(w, 'widget', None):
@@ -1098,34 +1095,12 @@ class LeoQtGui(leoGui.LeoGui):
                 # return False, indicating that the widget must handle
                 # qevent, which *presumably* is the best that can be done.
                 g.app.gui.insert_char_flag = True
-    #@+node:ekr.20190819072045.1: *3* qt_gui.make_main_window
-    def make_main_window(self) -> None:
-        """Make the *singleton* QMainWindow."""
-        window = QtWidgets.QMainWindow()
-        window.setObjectName('LeoGlobalMainWindow')
-        # Calling window.show() here causes flash.
-        self.attachLeoIcon(window)
-        # Monkey-patch
-        window.closeEvent = self.close_event  # Use self: g.app.gui does not exist yet.
-        self.runAtIdle(self.set_main_window_style_sheet)  # No StyleSheetManager exists yet.
-        return window
-
-    def set_main_window_style_sheet(self) -> None:
-        """Style the main window, using the first .leo file."""
-        commanders = g.app.commanders()
-        if commanders:
-            c = commanders[0]
-            ssm = c.styleSheetManager
-            ssm.set_style_sheets(w=self.main_window)
-            self.main_window.setWindowTitle(c.frame.title)  # #1506.
-        else:
-            g.trace("No open commanders!")
     #@+node:ekr.20110605121601.18528: *3* qt_gui.makeScriptButton
     def makeScriptButton(
         self,
         c: Cmdr,
         args: Any=None,
-        p: Pos=None,  # A node containing the script.
+        p: Position=None,  # A node containing the script.
         script: str=None,  # The script itself.
         buttonText: str=None,
         balloonText: str='Script Button',
@@ -1161,7 +1136,7 @@ class LeoQtGui(leoGui.LeoGui):
             b: Widget=b,
             c: Cmdr=c,
             buttonText: str=buttonText,
-            p: Pos=p and p.copy(),
+            p: Position=p and p.copy(),
             script: str=script
         ) -> None:
             if c.disableCommandsMessage:
@@ -1300,16 +1275,6 @@ class LeoQtGui(leoGui.LeoGui):
             g.es_exception()
             print('can not init leo.core.leoIPython.py')
             sys.exit(1)
-    #@+node:ekr.20190822174038.1: *3* qt_gui.set_top_geometry
-    already_sized = False
-
-    def set_top_geometry(self, w: int, h: int, x: int, y: int) -> None:
-        """Set the geometry of the main window."""
-        if 'size' in g.app.debug:
-            g.trace('(qt_gui) already_sized', self.already_sized, w, h, x, y)
-        if not self.already_sized:
-            self.already_sized = True
-            self.main_window.setGeometry(QtCore.QRect(x, y, w, h))
     #@+node:ekr.20180117053546.1: *3* qt_gui.show_tips & helpers
     @g.command('show-tips')
     def show_next_tip(self, event: Event=None) -> None:
@@ -1321,7 +1286,7 @@ class LeoQtGui(leoGui.LeoGui):
     #@+node:ekr.20220123052350.1: *4* << define DialogWithCheckBox >>
     class DialogWithCheckBox(QtWidgets.QMessageBox):  # type:ignore
 
-        def __init__(self, controller: Any, checked: bool, tip: str) -> None:
+        def __init__(self, controller: Any, checked: bool, tip: UserTip) -> None:
             super().__init__()
             c = g.app.log.c
             self.leo_checked = True
@@ -1341,7 +1306,8 @@ class LeoQtGui(leoGui.LeoGui):
             cb = QtWidgets.QCheckBox()
             cb.setObjectName('TipCheckbox')
             cb.setText('Show Tip On Startup')
-            state = QtConst.CheckState.Checked if checked else QtConst.CheckState.Unchecked  # #2383
+            # #2383: State is a tri-state, so use the official constants.
+            state = QtConst.CheckState.Checked if checked else QtConst.CheckState.Unchecked
             cb.setCheckState(state)  # #2127.
             cb.stateChanged.connect(controller.onClick)
             layout.addWidget(cb, 4, 0, -1, -1)
@@ -1519,11 +1485,11 @@ class LeoQtGui(leoGui.LeoGui):
         self,
         parent: Widget,
         name: str,
-        hPolicy: Any=None,
-        vPolicy: Any=None,
+        hPolicy: Policy=None,
+        vPolicy: Policy=None,
         lineWidth: int=1,
-        shadow: Any=None,
-        shape: Any=None,
+        shadow: Shadow=None,
+        shape: Shape=None,
     ) -> Widget:
         """Create a Qt Frame."""
         if shadow is None:
@@ -1566,17 +1532,13 @@ class LeoQtGui(leoGui.LeoGui):
         w.setText(label)
         return w
     #@+node:ekr.20190819092523.1: *4* qt_gui.createTabWidget
-    def createTabWidget(self,
-        parent: Widget, name: str, hPolicy: Any=None, vPolicy: Any=None,
-    ) -> Widget:
+    def createTabWidget(self, parent: Widget, name: str, hPolicy: Policy=None, vPolicy: Policy=None) -> Widget:
         w = QtWidgets.QTabWidget(parent)
         self.setSizePolicy(w, kind1=hPolicy, kind2=vPolicy)
         w.setObjectName(name)
         return w
     #@+node:ekr.20190819091214.1: *4* qt_gui.setSizePolicy
-    def setSizePolicy(self,
-        widget: Widget, kind1: Any=None, kind2: Any=None,
-    ) -> None:
+    def setSizePolicy(self, widget: Widget, kind1: Policy=None, kind2: Policy=None) -> None:
         if kind1 is None:
             kind1 = Policy.Ignored
         if kind2 is None:
@@ -1637,62 +1599,6 @@ class StyleClassManager:
             props = [i for i in props if i not in prop]
 
         self.set_sclasses(w, props)
-    #@+node:tbrown.20150724090431.7: *3* sclass_tests
-    def sclass_tests(self) -> None:
-        """Test style class property manipulation functions"""
-        # pylint: disable=len-as-condition
-
-        class Test_W:
-            """simple standin for QWidget for testing"""
-
-            def __init__(self) -> None:
-                self.x = ''
-
-            def property(self, name: str, default: str=None) -> Any:
-                return self.x or default
-
-            def setProperty(self, name: str, value: Any) -> None:
-                self.x = value
-
-        w = Test_W()
-
-        assert not self.has_sclass(w, 'nonesuch')
-        assert not self.has_sclass(w, ['nonesuch'])
-        assert not self.has_sclass(w, ['nonesuch', 'either'])
-        assert len(self.sclasses(w)) == 0
-
-        self.add_sclass(w, 'test')
-
-        assert not self.has_sclass(w, 'nonesuch')
-        assert self.has_sclass(w, 'test')
-        assert self.has_sclass(w, ['test'])
-        assert not self.has_sclass(w, ['test', 'either'])
-        assert len(self.sclasses(w)) == 1
-
-        self.add_sclass(w, 'test')
-        assert len(self.sclasses(w)) == 1
-        self.add_sclass(w, ['test', 'test', 'other'])
-        assert len(self.sclasses(w)) == 2
-        assert self.has_sclass(w, 'test')
-        assert self.has_sclass(w, 'other')
-        assert self.has_sclass(w, ['test', 'other', 'test'])
-        assert not self.has_sclass(w, ['test', 'other', 'nonesuch'])
-
-        self.remove_sclass(w, ['other', 'nothere'])
-        assert self.has_sclass(w, 'test')
-        assert not self.has_sclass(w, 'other')
-        assert len(self.sclasses(w)) == 1
-
-        self.toggle_sclass(w, 'third')
-        assert len(self.sclasses(w)) == 2
-        assert self.has_sclass(w, ['test', 'third'])
-        self.toggle_sclass(w, 'third')
-        assert len(self.sclasses(w)) == 1
-        assert not self.has_sclass(w, ['test', 'third'])
-
-        self.clear_sclasses(w)
-        assert len(self.sclasses(w)) == 0
-        assert not self.has_sclass(w, 'test')
     #@+node:tbrown.20150724090431.8: *3* sclasses
     def sclasses(self, w: Wrapper) -> List[str]:
         """return list of style classes for QWidget w"""
@@ -1847,10 +1753,8 @@ class StyleSheetManager:
         sheet = w.styleSheet()
         print(f"style sheet for: {w}...\n\n{sheet}")
     #@+node:ekr.20110605121601.18175: *4* ssm.set_style_sheets
-    def set_style_sheets(self, all: bool=True, top: Widget=None, w: Wrapper=None) -> None:
+    def set_style_sheets(self, all: bool=True, top: Widget=None, w: Widget=None) -> None:
         """Set the master style sheet for all widgets using config settings."""
-        if g.app.loadedThemes:
-            return
         c = self.c
         if top is None:
             top = c.frame.top
@@ -2146,7 +2050,7 @@ class StyleSheetManager:
         return newsheet
     #@+node:ekr.20180316092116.1: *3* ssm.Widgets
     #@+node:ekr.20140913054442.19390: *4* ssm.get_master_widget
-    def get_master_widget(self, top: Any=None) -> Widget:
+    def get_master_widget(self, top: Widget=None) -> Widget:
         """
         Carefully return the master widget.
         c.frame.top is a DynamicWindow.
