@@ -3,11 +3,11 @@
 #@+node:ekr.20140907131341.18707: * @file ../plugins/qt_tree.py
 #@@first
 """Leo's Qt tree class."""
-#@+<< imports: qt_tree.py >>
-#@+node:ekr.20140907131341.18709: ** << imports: qt_tree.py >>
+#@+<< qt_tree imports >>
+#@+node:ekr.20140907131341.18709: ** << qt_tree imports >>
 import re
 import time
-from typing import Any, Callable, Dict, Generator, List, Tuple
+from typing import Any, Callable, Dict, List, Tuple
 from typing import TYPE_CHECKING
 from leo.core.leoQt import isQt6, QtCore, QtGui, QtWidgets
 from leo.core.leoQt import EndEditHint, Format, ItemFlag, KeyboardModifier
@@ -15,25 +15,29 @@ from leo.core import leoGlobals as g
 from leo.core import leoFrame
 from leo.core import leoPlugins  # Uses leoPlugins.TryNext.
 from leo.plugins import qt_text
-#@-<< imports: qt_tree.py >>
-#@+<< type aliases: qt_tree.py >>
-#@+node:ekr.20220417193741.1: ** << type aliases: qt_tree.py >>
-if TYPE_CHECKING:  # Always False at runtime.
+#@-<< qt_tree imports >>
+#@+<< qt_tree annotations >>
+#@+node:ekr.20220417193741.1: ** << qt_tree annotations >>
+if TYPE_CHECKING:  # pragma: no cover
     from leo.core.leoCommands import Commands as Cmdr
-    from leo.core.leoNodes import Position
-    from leo.core.leoNodes import VNode
+    from leo.core.leoFrame import LeoQTreeWidget
+    from leo.core.leoGui import LeoKeyEvent as Event
+    from leo.core.leoNodes import Position, VNode
+    from leo.plugins.qt_frame import LeoQtFrame
+    from leo.plugins.qt_text import QTextEditWrapper as Wrapper
 else:
     Cmdr = Any
+    Event = Any
+    LeoQtFrame = Any
+    LeoQTreeWidget = Any
     Position = Any
     VNode = Any
+    Wrapper = Any
 Editor = Any
-Event = Any
 Icon = Any
 Item = Any
-Selection = Tuple[int, int, int]
 Widget = Any
-Wrapper = Any
-#@-<< type aliases: qt_tree.py >>
+#@-<< qt_tree annotations >>
 #@+others
 #@+node:ekr.20160514120051.1: ** class LeoQtTree
 class LeoQtTree(leoFrame.LeoTree):
@@ -41,7 +45,7 @@ class LeoQtTree(leoFrame.LeoTree):
     #@+others
     #@+node:ekr.20110605121601.18404: *3* qtree.Birth
     #@+node:ekr.20110605121601.18405: *4* qtree.__init__
-    def __init__(self, c: Cmdr, frame: Wrapper) -> None:
+    def __init__(self, c: Cmdr, frame: LeoQtFrame) -> None:  # Frame is a LeoQtFrame.
         """Ctor for the LeoQtTree class."""
         super().__init__(frame)
         self.c = c
@@ -61,18 +65,19 @@ class LeoQtTree(leoFrame.LeoTree):
         self.vnode2itemsDict: Dict[VNode, List[Item]] = {}  # values are lists of items.
         self.editWidgetsDict: Dict[Editor, Wrapper] = {}  # keys are native edit widgets, values are wrappers.
         self.reloadSettings()
-        # Components.
-        self.canvas: Wrapper = self  # An official ivar used by Leo's core.
-        self.headlineWrapper: Wrapper = qt_text.QHeadlineWrapper  # This is a class.
-        # w is a LeoQTreeWidget, a subclass of QTreeWidget.
-        self.treeWidget: Widget = frame.top.treeWidget  # An internal ivar.
+        # Components...
+        self.canvas = self  # An official ivar used by Leo's core.
+        self.headlineWrapper: Any = qt_text.QHeadlineWrapper  # This is a class.
+        self.treeWidget: LeoQTreeWidget = frame.top.treeWidget
         w = self.treeWidget
-        #
-        # "declutter", node appearance tweaking
+        # Declutter data...
         self.declutter_patterns: List[Any] = None  # list of pairs of patterns for decluttering
         self.declutter_data: Dict[Any, Any] = {}
         self.loaded_images: Dict[str, Icon] = {}
-        if 0:  # Drag and drop
+
+        if 0:  # None of this works.
+            #@+<< Drag and drop >>
+            #@+node:ekr.20220913074246.1: *5* << Drag and drop >>
             w.setDragEnabled(True)
             w.viewport().setAcceptDrops(True)
             w.showDropIndicator = True
@@ -87,9 +92,9 @@ class LeoQtTree(leoFrame.LeoTree):
 
                 def mimeData(self, indexes: str) -> None:
                     g.trace()
+            #@-<< Drag and drop >>
 
         # Early inits...
-
         try:
             w.headerItem().setHidden(True)
         except Exception:
@@ -159,134 +164,6 @@ class LeoQtTree(leoFrame.LeoTree):
         """Clear all widgets in the tree."""
         w = self.treeWidget
         w.clear()
-    #@+node:ekr.20180810052056.1: *4* qtree.drawVisible & helpers (not used)
-    def drawVisible(self, p: Position) -> None:
-        """
-        Add only the visible nodes to the outline.
-
-        Not used, as this causes scrolling issues.
-        """
-        t1 = time.process_time()
-        c = self.c
-        parents: List[Any] = []
-        # Clear the widget.
-        w = self.treeWidget
-        w.clear()
-        # Clear the dicts.
-        self.initData()
-        if c.hoistStack:
-            first_p = c.hoistStack[-1].p
-            target_p = first_p.nodeAfterTree().visBack(c)
-        else:
-            first_p = c.rootPosition()
-            target_p = None
-        n = 0
-        for p in self.yieldVisible(first_p, target_p):
-            n += 1
-            level = p.level()
-            parent_item = w if level == 0 else parents[level - 1]
-            item = QtWidgets.QTreeWidgetItem(parent_item)
-            item.setFlags(item.flags() | ItemFlag.ItemIsEditable)
-            item.setChildIndicatorPolicy(
-                item.ShowIndicator if p.hasChildren()
-                else item.DontShowIndicator)
-            item.setExpanded(bool(p.hasChildren() and p.isExpanded()))
-            self.items.append(item)
-            # Update parents.
-            parents = [] if level == 0 else parents[:level]
-            parents.append(item)
-            # Update the dicts.
-            itemHash = self.itemHash(item)
-            self.item2positionDict[itemHash] = p.copy()
-            self.item2vnodeDict[itemHash] = p.v
-            self.position2itemDict[p.key()] = item
-            d = self.vnode2itemsDict
-            v = p.v
-            aList = d.get(v, [])
-            aList.append(item)
-            d[v] = aList
-            # Enter the headline.
-            item.setText(0, p.h)
-            if self.use_declutter:
-                item._real_text = p.h
-            # Draw the icon.
-            v.iconVal = v.computeIcon()
-            icon = self.getCompositeIconImage(p, v.iconVal)
-            if icon:
-                self.setItemIcon(item, icon)
-            # Set current item.
-            if p == c.p:
-                w.setCurrentItem(item)
-        # Useful, for now.
-        t2 = time.process_time()
-        if t2 - t1 > 0.1:
-            g.trace(f"{n} nodes, {t2 - t1:5.2f} sec")
-    #@+node:ekr.20180810052056.2: *5* qtree.yieldVisible (not used)
-    def yieldVisible(self, first_p: Position, target_p: Position=None) -> Generator:
-        """
-        A generator yielding positions from first_p to target_p.
-        """
-        c = self.c
-        p = first_p.copy()
-        yield p
-        while p:
-            if p == target_p:
-                return
-            v = p.v
-            if (v.children and (
-                # Use slower test for clones:
-                len(v.parents) > 1 and p in v.expandedPositions or
-                # Use a quick test for non-clones:
-                len(v.parents) <= 1 and (v.statusBits & v.expandedBit) != 0
-            )):
-                # p.moveToFirstChild()
-                p.stack.append((v, p._childIndex),)
-                p.v = v.children[0]
-                p._childIndex = 0
-                yield p
-                continue
-            # if p.hasNext():
-            parent_v = p.stack[-1][0] if p.stack else c.hiddenRootNode
-            if p._childIndex + 1 < len(parent_v.children):
-                # p.moveToNext()
-                p._childIndex += 1
-                p.v = parent_v.children[p._childIndex]
-                yield p
-                continue
-            #
-            # A fast version of p.moveToThreadNext().
-            # We look for a parent with a following sibling.
-            while p.stack:
-                # p.moveToParent()
-                p.v, p._childIndex = p.stack.pop()
-                # if p.hasNext():
-                parent_v = p.stack[-1][0] if p.stack else c.hiddenRootNode
-                if p._childIndex + 1 < len(parent_v.children):
-                    # p.moveToNext()
-                    p._childIndex += 1
-                    p.v = parent_v.children[p._childIndex]
-                    break  # Found: moveToThreadNext()
-            else:
-                break  # Not found.
-            # Found moveToThreadNext()
-            yield p
-            continue
-        if target_p:
-            g.trace('NOT FOUND:', target_p.h)
-    #@+node:ekr.20180810052056.3: *5* qtree.slowYieldVisible
-    def slowYieldVisible(self, first_p: Position, target_p: Position=None) -> Generator:
-        """
-        A generator yielding positions from first_p to target_p.
-        """
-        c = self.c
-        p = first_p.copy()
-        while p:
-            yield p
-            if p == target_p:
-                return
-            p.moveToVisNext(c)
-        if target_p:
-            g.trace('NOT FOUND:', target_p.h)
     #@+node:ekr.20110605121601.17873: *4* qtree.full_redraw & helpers
     def full_redraw(self, p: Position=None) -> Position:
         """
@@ -328,27 +205,27 @@ class LeoQtTree(leoFrame.LeoTree):
     redraw_now = full_redraw  #type:ignore
     #@+node:vitalije.20200329160945.1: *5* tree declutter code
     #@+node:tbrown.20150807090639.1: *6* qtree.declutter_node & helpers
-    def declutter_node(self, c: Cmdr, p: Position, item: Item) -> Icon:
+    def declutter_node(self, c: Cmdr, v: VNode, item: Item) -> Icon:
         """declutter_node - change the appearance of a node
 
         :param commander c: commander containing node
         :param position p: position of node
         :param QWidgetItem item: tree node widget item
 
-        returns composite icon for this node
+        returns composite icon for this node, containing the icon box and perhaps other icons.
         """
         dd = self.declutter_data
-        iconVal = p.v.computeIcon()
+        iconVal = v.computeIcon()
         iconName = f'box{iconVal:02d}.png'
         loaded_images = self.loaded_images
         #@+others
         #@+node:vitalije.20200329153544.1: *7* sorted_icons
-        def sorted_icons(p: Position) -> List[str]:
+        def sorted_icons(v: VNode) -> List[str]:
             """
             Returns a list of icon filenames for this node.
             The list is sorted to owner the 'where' key of image dicts.
             """
-            icons = c.editCommands.getIconList(p)
+            icons = c.editCommands.getIconList(v)
             a = [x['file'] for x in icons if x['where'] == 'beforeIcon']
             a.append(iconName)
             a.extend(x['file'] for x in icons if x['where'] == 'beforeHeadline')
@@ -366,13 +243,19 @@ class LeoQtTree(leoFrame.LeoTree):
             replacement, s = None, None
 
             if cmd == 'REPLACE':
-                s = pattern.sub(arg, text)
+                try:
+                    s = pattern.sub(arg, text)
+                except re.error as e:
+                    g.log(
+                        f'Error in declutter REPLACE "{e!s}"\n'
+                        f'  RULE:{pattern.pattern!r}\n'
+                        f'  REPLACE:{arg!r}\n  HEADLINE:{text!r}', color='error')
             elif cmd == 'REPLACE-HEAD':
                 s = text[: m.start()].rstrip()
             elif cmd == 'REPLACE-TAIL':
                 s = text[m.end() :].lstrip()
             elif cmd == 'REPLACE-REST':
-                s = (text[:m.start] + text[m.end() :]).strip()
+                s = (text[:m.start()] + text[m.end() :]).strip()
 
             # 's' is string when 'cmd' is recognised
             # and is None otherwise
@@ -400,6 +283,11 @@ class LeoQtTree(leoFrame.LeoTree):
                     # wrap list 'new_icons' in a saved argument as
                     # the list is recreated before each call.
                     new_icons.append(param)
+            elif cmd == 'DOCICON':
+                param = g.os_path_join(g.os_path_dirname(c.fileName()), param)
+                def modifier(item: Item, param: str) -> None:
+                    # As above, but for document relative icons
+                    new_icons.append(param)
             elif cmd == 'BG':
                 def modifier(item: Item, param: str) -> None:
                     item.setBackground(0, QtGui.QBrush(QtGui.QColor(param)))
@@ -454,29 +342,25 @@ class LeoQtTree(leoFrame.LeoTree):
                 if f not in loaded_images:
                     loaded_images[f] = g.app.gui.getImageImage(f)
         #@-others
-        if (p.h, iconVal) in dd:
-            # Apply saved adjustments to the text and to the _style_
-            # of the node
-            new_icons, modifiers_and_args = dd[(p.h, iconVal)]
+        if (v.h, iconVal) in dd:
+            # Apply saved adjustments to the text and to the _style_ of the node.
+            new_icons, modifiers_and_args = dd[(v.h, iconVal)]
             for modifier, arg in modifiers_and_args:
                 modifier(item, arg)
-
-            new_icons = sorted_icons(p) + new_icons
+            new_icons = sorted_icons(v) + new_icons
         else:
-            text = p.h
+            text = v.h
             new_icons = []
             modifiers_and_args = []
             for pattern, cmds in self.get_declutter_patterns():
                 m = pattern.match(text) or pattern.search(text)
                 if m:
                     modifiers_and_args.extend(apply_declutter_rules(cmds))
-
-            # Save the lists of the icons and the adjusting operations
-            # for future reuse.
-            dd[(p.h, iconVal)] = new_icons, modifiers_and_args
-            new_icons = sorted_icons(p) + new_icons
+            # Save the lists of the icons and the adjusting operations.
+            dd[(v.h, iconVal)] = new_icons, modifiers_and_args
+            new_icons = sorted_icons(v) + new_icons
             preload_images()
-        self.nodeIconsDict[p.gnx] = new_icons
+        self.nodeIconsDict[v.gnx] = new_icons
         h = ':'.join(new_icons)
         icon = g.app.gui.iconimages.get(h)
         if not icon:
@@ -484,6 +368,7 @@ class LeoQtTree(leoFrame.LeoTree):
             images = [loaded_images.get(x) for x in new_icons]
             icon = self.make_composite_icon(images)
             g.app.gui.iconimages[h] = icon
+        # There is always at least a box icon.
         return icon
     #@+node:vitalije.20200327162532.1: *6* qtree.get_declutter_patterns
     def get_declutter_patterns(self) -> List[Any]:
@@ -504,7 +389,11 @@ class LeoQtTree(leoFrame.LeoTree):
             if cmd.startswith('#'):
                 pass
             elif cmd == 'RULE':
-                patterns.append((re.compile(arg), []))
+                try:
+                    patterns.append((re.compile(arg), []))
+                except re.error:
+                    g.log('Invalid declutter pattern: %r' % (arg,), color='error')
+                    patterns.append((re.compile('a^'), [])) # create a rule that matches nothing
             else:
                 if patterns:
                     patterns[-1][1].append((cmd, arg))
@@ -558,14 +447,12 @@ class LeoQtTree(leoFrame.LeoTree):
         # #1310: Add a tool tip.
         item.setToolTip(0, p.h)
         if self.use_declutter:
-            icon = self.declutter_node(c, p, item)
+            icon = self.declutter_node(c, p.v, item)
             if icon:
                 item.setIcon(0, icon)
             return item
-        # Draw the icon.
-        v.iconVal = v.computeIcon()
-        # **Slow**, but allows per-vnode icons.
-        icon = self.getCompositeIconImage(p, v.iconVal)
+        # Draw the icon: **Slow**, but allows per-vnode icons.
+        icon = self.getCompositeIconImage(p.v)
         if icon:
             item.setIcon(0, icon)
         return item
@@ -632,29 +519,15 @@ class LeoQtTree(leoFrame.LeoTree):
         """Redraw all Qt outline items cloned to c.p."""
         if self.busy:
             return
-        p = self.c.p
+        c, p = self.c, self.c.p
         if p:
             h = p.h  # 2010/02/09: Fix bug 518823.
             for item in self.vnode2items(p.v):
                 if self.isValidItem(item):
                     self.setItemText(item, h)
-        # Bug fix: 2009/10/06
-        self.redraw_after_icons_changed()
-    #@+node:ekr.20110605121601.17883: *4* qtree.redraw_after_icons_changed
-    def redraw_after_icons_changed(self) -> None:
-
-        if self.busy:
-            return
-        self.redrawCount += 1  # To keep a unit test happy.
-        c = self.c
-        try:
-            self.busy = True  # Suppress call to setHeadString in onItemChanged!
-            self.getCurrentItem()
-            for p in c.rootPosition().self_and_siblings(copy=False):
-                # Updates icons in p and all visible descendants of p.
-                self.updateVisibleIcons(p)
-        finally:
-            self.busy = False
+                    if self.use_declutter:  # #2844.
+                        icon = self.declutter_node(c, p, item)
+                        item.setIcon(0, icon)  # 0 is the column number.
     #@+node:ekr.20110605121601.17884: *4* qtree.redraw_after_select
     def redraw_after_select(self, p: Position=None) -> None:
         """Redraw the entire tree when an invisible node is selected."""
@@ -877,47 +750,31 @@ class LeoQtTree(leoFrame.LeoTree):
     def setFocus(self) -> None:
         g.app.gui.set_focus(self.c, self.treeWidget)
     #@+node:ekr.20110605121601.18409: *3* qtree.Icons
-    #@+node:ekr.20110605121601.18410: *4* qtree.drawIcon
-    def drawIcon(self, p: Position) -> None:
-        """Redraw the icon at p."""
-        self.updateIcon(p)
-        # the following code is wrong. It constructs a new item
-        # and assignes the icon to it. However this item is never
-        # added to the treeWidget so it is soon garbage collected
-            # w = self.treeWidget
-            # itemOrTree = self.position2item(p) or w
-            # item = QtWidgets.QTreeWidgetItem(itemOrTree)
-            # icon = self.getIcon(p)
-            # self.setItemIcon(item, icon)
-    #@+node:ekr.20110605121601.18411: *4* qtree.getIcon & helper
-    def getIcon(self, p: Position) -> Icon:
+    #@+node:ekr.20110605121601.18411: *4* qtree.getIcon & helpers
+    def getIcon(self, v: VNode) -> Icon:
         """Return the proper icon for position p."""
         if self.use_declutter:
-            item = self.position2item(p)
-            return item and self.declutter_node(self.c, p, item)
-        p.v.iconVal = iv = p.v.computeIcon()
-        return self.getCompositeIconImage(p, iv)
-
-
+            items = self.vnode2items(v)
+            if items:
+                return self.declutter_node(self.c, v, items[0])
+        return self.getCompositeIconImage(v)
     #@+node:vitalije.20200329153148.1: *5* qtree.icon_filenames_for_node
-    def icon_filenames_for_node(self, p: Position, val: int) -> List[str]:
-        """Prepares and returns a list of icon filenames
-           related to this node.
-        """
-        nicon = f'box{val:02d}.png'
-        fnames = self.nodeIconsDict.get(p.gnx)
+    def icon_filenames_for_node(self, v: VNode) -> List[str]:
+        """Returns a list of icon filenames for v."""
+        nicon = f'box{v.iconVal:02d}.png'
+        fnames = self.nodeIconsDict.get(v.gnx)
         if not fnames:
-            icons = self.c.editCommands.getIconList(p)
+            icons = self.c.editCommands.getIconList(v)
             fnames = [x['file'] for x in icons if x['where'] == 'beforeIcon']
             fnames.append(nicon)
             fnames.extend(x['file'] for x in icons if x['where'] == 'beforeHeadline')
-            self.nodeIconsDict[p.gnx] = fnames
+            self.nodeIconsDict[v.gnx] = fnames
         pat = re.compile(r'^box\d\d\.png$')
         loaded_images = self.loaded_images
         for i, f in enumerate(fnames):
             if pat.match(f):
                 fnames[i] = nicon
-                self.nodeIconsDict[p.gnx] = fnames
+                self.nodeIconsDict[v.gnx] = fnames
                 f = nicon
             if f not in loaded_images:
                 loaded_images[f] = g.app.gui.getImageImage(f)
@@ -930,23 +787,21 @@ class LeoQtTree(leoFrame.LeoTree):
         images = [i.scaledToHeight(height) for i in images]
         width = sum([i.width() for i in images]) + hsep * (len(images) - 1)
         pix = QtGui.QImage(width, height, Format.Format_ARGB32_Premultiplied)
-        pix.fill(QtGui.QColor(0, 0, 0, 0).rgba())  # transparent fill, rgbA
-        # .rgba() call required for Qt4.7, later versions work with straight color
+        pix.fill(QtGui.QColor(0, 0, 0, 0))
         painter = QtGui.QPainter()
-        if not painter.begin(pix):
-            print("Failed to init. painter for icon")
-            # don't return, the code still makes an icon for the cache
-            # which stops this being called again and again
+        ok = painter.begin(pix)
         x = 0
         for i in images:
             painter.drawPixmap(x, 0, i)
             x += i.width() + hsep
-        painter.end()
+        if ok:
+            painter.end()
         return QtGui.QIcon(QtGui.QPixmap.fromImage(pix))
     #@+node:ekr.20110605121601.18412: *5* qtree.getCompositeIconImage
-    def getCompositeIconImage(self, p: Position, val: int) -> Icon:
-        """Get the icon at position p."""
-        fnames = self.icon_filenames_for_node(p, val)
+    def getCompositeIconImage(self, v: VNode) -> Icon:
+        """Get the icon at v."""
+        v.iconVal = v.computeIcon()
+        fnames = self.icon_filenames_for_node(v)
         h = ':'.join(fnames)
         icon = g.app.gui.iconimages.get(h)
         loaded_images = self.loaded_images
@@ -965,33 +820,6 @@ class LeoQtTree(leoFrame.LeoTree):
             # but there is no itemChanged event handler.
             item.setIcon(0, icon)
 
-    #@+node:ekr.20110605121601.17951: *4* qtree.updateIcon & updateAllIcons
-    def updateIcon(self, p: Position) -> None:
-        """Update p's icon."""
-        if not p:
-            return
-        val = p.v.computeIcon()
-        if p.v.iconVal != val:
-            self.nodeIconsDict.pop(p.gnx, None)
-            self.getIcon(p)  # sets p.v.iconVal
-
-    def updateAllIcons(self, p: Position) -> None:
-        if not p:
-            return
-        self.nodeIconsDict.pop(p.gnx, None)
-        icon = self.getIcon(p)  # sets p.v.iconVal
-        # Update all cloned items.
-        items = self.vnode2items(p.v)
-        for item in items:
-            self.setItemIcon(item, icon)
-    #@+node:ekr.20110605121601.17952: *4* qtree.updateVisibleIcons
-    def updateVisibleIcons(self, p: Position) -> None:
-        """Update the icon for p and the icons
-        for all visible descendants of p."""
-        self.updateAllIcons(p)
-        if p.hasChildren() and p.isExpanded():
-            for child in p.children():
-                self.updateVisibleIcons(child)
     #@+node:ekr.20110605121601.18414: *3* qtree.Items
     #@+node:ekr.20110605121601.17943: *4*  qtree.item dict getters
     def itemHash(self, item: Item) -> str:
@@ -1276,8 +1104,8 @@ class LeoQtTree(leoFrame.LeoTree):
         return None
     #@+node:ekr.20110605121601.17909: *4* qtree.editLabel and helper
     def editLabel(self,
-        p: Position, selectAll: bool=False, selection: Selection=None,
-    ) -> Tuple[Editor, Wrapper]:
+        p: Position, selectAll: bool=False, selection: Tuple=None,
+    ) -> Tuple[Editor, Any]:
         """Start editing p's headline."""
         if self.busy:
             return None
@@ -1300,8 +1128,8 @@ class LeoQtTree(leoFrame.LeoTree):
         return e, wrapper
     #@+node:ekr.20110605121601.18422: *5* qtree.editLabelHelper
     def editLabelHelper(self,
-        item: Any, selectAll: bool=False, selection: Selection=None,
-    ) -> Tuple[Item, Wrapper]:
+        item: Any, selectAll: bool=False, selection: Tuple=None,
+    ) -> Tuple[Item, Any]:
         """Helper for qtree.editLabel."""
         c, vc = self.c, self.c.vimCommands
         w = self.treeWidget
